      program euhedral
c--- check cosines while reading hkl
c
c 0.669 1.036 1.467 0.964 1.329 1.381    1.13055
c
c--- readins only reads in curent directory
c
c read cell parameters, faces and mu (.ins [and .rmat])
c read pointgroup
c read hklf4 
c build motherlist
c
csetfilter:
c get and apply filter criteria  (multiplicity, thetamin, intensity and angular distribution)
c   draw some projection
c repeat until satisfied
c create hkl4 file
c calculate uncorrected RMIN=X*Rint+Y*Rpsi
c
csetup:
c     select refinement type(default: 1. Volume 2. Face 3etc. Volume  4.exit 5.setfilter)
crefinement:
c       create ins file for platon
c       run platon
c       read .hkp file
c       calculate RMIN=X*Rint+Y*Rpsi
c       repeat refinement until RMIN minimum
c
c     (plot crystal form with platon)
c   goto setup
c
      implicit none
      include 'axes.inc'
      include 'const.inc'
      include 'euhedral.inc'
c---- functions
      logical   str_equal
      logical   interrupted
      logical   RmatOK
      logical   FacesOK
      logical   FaceInRange
c---- local vars
      integer   nd
      character d*80
      integer   ier
      character prompt*80
      integer   i
      integer   j 
      character message*256
      integer   ihkl(3)
      character hklfile*80
      real      f
      real      v(3)
      logical   dirty
      character version*80
c
      out = 6
      call finish(-out)
      call str_unit(out)
      call str_real_len(2)
      call get_inp_upcase('off')
      call get_commandline(nd)
      call initaxes
      call append_file('euhedral.log',log,message,ier)
      call str_out(message)
      if(ier.ne.0) goto 900
      version = "19-Nov-2001"
c--- please update doc/index.html also
      call logwrite2('Started Euhedral version',version)
      call EuhedralInit
      call copyvec(z_axis,rotax)

      call get_mode(d)
      menu = str_equal(d,'I')  ! Interactive
      dirty = .false.
   10 continue
      call CheckAbort
      call get_inp(d,nd,' ',*900)
      if(d.eq.'?') then ! command
        call PrintHelp
      else if(d.eq.'addface') then  ! command
        call InputFace(nd,*900)
      else if(d.eq.'addfacehkl') then  ! command
        call InputFaceHKL(nd,*900)
      else if(d.eq.'addfaceperp') then  ! command
        call InputFacePerp(nd,*900)
      else if(d.eq.'addfacesum') then  ! command
        call InputFaceSum(nd,*900)
      else if(d.eq.'amoeba') then ! command
        call setuprefinement(ier)
        if(ier.ne.0) goto 10
        call RefineDistanceSimplex(message,ier)
        call errorwarning(message,ier)
      else if(d.eq.'angdistmax') then ! command
        call InputReal(angdistmax,d,dirty,nd,*900)
      else if(d.eq.'angmax') then ! command
        call InputPosAngle(Angmax,d,dirty,nd,*900)
      else if(d.eq.'angmin') then ! command
        call InputPosAngle(Angmin,d,dirty,nd,*900)
      else if(d(1:9).eq.'buildcube'.or.d.eq.'buildsphere') then ! command
        call BuildModel(d,nd,dirty,*900)
      else if(d.eq.'bestsphere') then ! command
        if(menu) then
          call InputTol(tol,dirty,nd,*900)
          call InputDistance(initdistance,dirty,nd,*900)
        endif
        call get_inp_default(d,nd,'Volume/Distance/Amoeba [va]',
     +       *900)
        call BestSphere(d)
      else if(d.eq.'cell') then ! no user command
        do i=1,3
          call str_string(abc(i),'#',prompt)
          call get_inp(d,nd,prompt,*900)
          call get_real(cellaxes(i),d)
        enddo
        do i=1,3
          call str_string(albega(i),'#',prompt)
          call get_inp(d,nd,prompt,*900)
          call get_grad(cellangles(i),d)
        enddo
        cellcheck = .true.
      else if(d.eq.'centric') then ! command
        call InputBoolean(centricshape,d,dirty,nd,*900)
      else if(d.eq.'colour') then ! command
        call InputColour(nd,dirty,*900)
      else if(d.eq.'copy') then ! command
        call str_string(compound,'name [#]',prompt)
        call get_inp_default(compound,nd,prompt,*900)
        if(FacesOK()) then
          call WriteAndRead(ier)
          if(ier.ne.0) goto 10
          if(windowexists) then
            call WindowTitle(compound)
          endif
        endif
      else if(d.eq.'correction') then ! command
        prompt = 'Correction Method ABST/ABSG [#]'
        call str_string(correctionmethod,prompt,prompt)
        call get_inp_default(d,nd,prompt,*900)
        if(d.eq.'ABST'.or.d.eq.'ABSG') then
          if(d.ne.correctionmethod) dirty = .true.
          correctionmethod = d
        else
          call str_string(d,'unknown correction method #',message)
          call warning(message)
        endif
        call logwrite2('correction',d)
      else if(d.eq.'croty') then ! command
        if(graphics) then
          if(FacesOK()) then
            f = viewrot
            dirty = .true.
            call str_out('Type control-c to interrupt')
            do while(.true.)
              do viewrot=f,TwoPi,viewspeed
                call plotfaces
                if(interrupted()) then
                  goto 10
                endif
              enddo
              f = viewspeed
            enddo
          endif
        endif
      else if(d.eq.'crystrot') then ! command
        call setuprefinement(ier)
        if(ier.ne.0) goto 10
        call BuildOrient
        if(menu) then
          call InputPhi(dirty,nd,*900)
          call InputPosAngle(Angmin,'angmin',dirty,nd,*900)
        endif
        call BuildOrientationList
        if(menu) then
          call get_inp_default(d,nd,'Do Rot refinement? [Yes]',*900)
        else
          d = 'Y'
        endif
        if(str_equal(d,'Y')) then
          call RefineOrient(message,ier)
          call errorwarning(message,ier)
          if(ier.eq.0) call saveresult(nd,*900)
        endif
      else if(d.eq.'crystrot1'.or.d.eq.'crystrothkl'.or. ! command
     +       d.eq.'crystrotface') then ! command
        call setuprefinement(ier)
        if(ier.ne.0) goto 10
        if(menu) then
          if(d.eq.'crystrothkl') then
            call InputRotaxHKL(dirty,nd,ier,*900)
          else if(d.eq.'crystrotface') then
            call InputRotaxFace(dirty,nd,ier,*900)
          else                  ! i.e. crystrot1
            call InputRotax(dirty,nd,*900)
            ier = 0
          endif
          call InputPhi(dirty,nd,*900)
          call InputPosAngle(Angmin,'angmin',dirty,nd,*900)
        else
          ier = 0
        endif
        if(ier.eq.0) then
          nrotaxes = 1
          call savevec(rotax,rotaxes,nrotaxes)
          call BuildOrientationList
          if(menu) then
            call get_inp_default(d,nd,'Do Rot refinement? [Yes]',*900)
          else
            d = 'Y'
          endif
          if(str_equal(d,'Y')) then
            call RefineOrient(message,ier)
            call errorwarning(message,ier)
            if(ier.eq.0) call saveresult(nd,*900)
          endif
        endif
      else if(d.eq.'crystrotsimplex') then ! command
        call setuprefinement(ier)
        if(ier.ne.0) goto 10
        if(menu) then
          call InputAngle(OrientSimplexAngle,'Initial rotation angle',
     +         dirty,nd,*900)
        endif
        call RefineOrientationSimplex(message,ier)
        call errorwarning(message,ier)
      else if(d.eq.'deleteface') then ! command
        call deleteface(nd,ier)
        if(ier.ne.0) goto 900
      else if(d.eq.'dirty') then ! no user command
        call InputBooleanNoDirty(dirty,d,nd,*900)
      else if(d.eq.'distance') then ! command
        call setuprefinement(ier)
        if(ier.ne.0) goto 10
        call RefineDistanceParabolic(message,ier)
        call errorwarning(message,ier)
      else if(d.eq.'edit') then ! command
        call build_file(compound,' ','.ins',d,i)
        call editor(d,ier)
        call readins(message,ier)
        call errorwarning(message,ier)
      else if(d.eq.'exit') then ! command
        if(dirty) then
          call savefile(.true.,message,ier)
          call errorwarning(message,ier)
        endif
        call str_flush
        ier = 0
        goto 900
      else if(d.eq.'extreme') then ! command
        call str_real_len_push(5)
        call InputReal(extreme,d,dirty,nd,*900)
        call str_real_len_pop
      else if(d.eq.'face') then ! command
        call editface(nd,*900)
      else if(d.eq.'faces') then ! no user command. Used by GUI
        call str_integer_write(6,nfaces,'faces #')
      else if(d.eq.'filter') then ! command
        call Filter(nd,dirty,*900)
      else if(d.eq.'help') then ! command
        if(nd.gt.0) then
          call get_inp(d,nd,' ',*900)
          call image_help(d,ier)
        else
          call image_help(' ',ier)
        endif
      else if(d.eq.'highindex') then ! command
        call InputInteger(highindex,d,dirty,nd,*900)
        if(facesavailable) then
          call FaceHighAll
          call WriteAndRead(ier)
        endif
      else if(d.eq.'hkl') then ! command
        call InputHkl(ihkl,nd,*900)
        call PrintHkl(ihkl)
      else if(d.eq.'initangle') then ! command
        call InputAngle(OrientSimplexAngle,'Initial rotation angle',
     +       dirty,nd,*900)
      else if(d.eq.'initdistance') then ! command
        call InputDistance(initdistance,dirty,nd,*900)
      else if(d.eq.'intmin') then ! command
        call InputReal(intmin,d,dirty,nd,*900)
      else if(d.eq.'longtilt') then ! command
        call longtilt(nd,dirty,*900)
      else if(d.eq.'manualrot') then ! command
        call manualrot(dirty,nd,ier)
        if(ier.ne.0) goto 900
      else if(d.eq.'deltatilt') then ! command
        call InputInteger(deltatilt,d,dirty,nd,*900)
        if(deltatilt.lt.0) deltatilt = 1
        if(deltatilt.gt.MaxDeltaTilt) deltatilt = MaxDeltaTilt
      else if(d.eq.'menu') then ! used by GUI
        call InputBooleanNoDirty(menu,d,nd,*900)
      else if(d.eq.'mode') then ! command
        prompt = projmodes(1)
        do i=2,maxmodes
          call str_2string(prompt,projmodes(i),'#/#',prompt)
        enddo
        call str_2string(prompt,projmodes(projectionmode),'# [#]',
     +       prompt)
        call get_inp_default(d,nd,prompt,*900)
        projectionmode = 0
        do i=1,maxmodes
          if(str_equal(d,projmodes(i))) then
            projectionmode = i
          endif
        enddo
        if(projectionmode.eq.0) call str_string_out(d,'unknown mode:#')
      else if(d.eq.'movie') then ! command
        call InputBoolean(movie,d,dirty,nd,*900)
      else if(d.eq.'name') then ! command
        call str_string(compound,'name [#]',prompt)
        call get_inp_default(compound,nd,prompt,*900)
      else if(d.eq.'newworld') then ! no user command
        newworld = .true.
      else if(d.eq.'nhklmin') then ! command
        call InputInteger(nhklmin,d,dirty,nd,*900)
      else if(d.eq.'nocheck') then ! command
        cellcheck = .false.
      else if(d.eq.'normat') then ! command
        myrmatavailable = .false.
        rmatfile = ' '
      else if(d.eq.'onerot') then ! command
        call ViewOneRotation
      else if(d.eq.'partialrot') then ! command
        call ViewPartialRotation
      else if(d.eq.'pg') then ! command
        call Inputpointgroup(nd,*900)
        if(npoint.ne.0.and.nref.gt.0) then
          call buildMother
          call str_num_out(nref,'# reflection^')
          call str_integer_out(Nunique,'(# unique)')
          call OutputR2
        endif
      else if(d.eq.'phistart') then ! no user command
        call InputAngle(phistart,'Phi start',dirty,nd,*900)
      else if(d.eq.'phistop') then ! no user command
        call InputAngle(phistop,'Phi stop',dirty,nd,*900)
      else if(d.eq.'delphi') then ! no user command
        call InputPosAngle(delphi,'Rotation Increment',dirty,nd,*900)
      else if(d.eq.'phi') then ! command
        call InputPhi(dirty,nd,*900)
      else if(d.eq.'plot') then ! command
        call readins(message,ier)
        call errorwarning(message,ier)
        if(ier.eq.0) then
          call PlotShape(message,ier)
          if(ier.ne.0) call warning(message)
        endif
      else if(d.eq.'plotaxes') then ! command
        call InputBoolean(doplotaxes,d,dirty,nd,*900)
        call plotfaces
      else if(d.eq.'plotlab') then ! command
        call InputBoolean(doplotlab,d,dirty,nd,*900)
        call plotfaces
      else if(d.eq.'plotcorner') then ! command
        call InputBoolean(doplotcorner,d,dirty,nd,*900)
        call plotfaces
      else if(d.eq.'pointsize') then ! command
        call InputInteger(pointsize,d,dirty,nd,*900)
      else if(d.eq.'posx') then ! command
        call InputInteger(posx,d,dirty,nd,*900)
        newwindow = .true.
      else if(d.eq.'posy') then ! command
        call InputInteger(posy,d,dirty,nd,*900)
        newwindow = .true.
      else if(d.eq.'printmatrices') then ! debug only
        call printmatrices
      else if(d.eq.'printorient') then ! debug only
        call printorient
      else if(d.eq.'ps') then ! command
        call get_inp(d,nd,'Output file name',*900)
        call WindowPs(d,1,ier)
      else if(d.eq.'randomrot') then ! command
        call randomrot(nd,dirty,*900)
      else if(d.eq.'readhkl') then ! command
        useablehklfile = .false.
        if(RmatOK()) then
          call get_file('*.hkl',' ',nd,hklfile,ier)
          if(ier.ne.0) goto 10
c          if(ier.ne.0) goto 900
          call readhkl(hklfile,message,ier)
          call errorwarning(message,ier)
          call logwrite(message)
          if(ier.eq.0) call printratio
          if(nref.gt.0) then
            do i=pDcosinesF,pDcosinesL
              if(dR(i,1).ne.0.0) useablehklfile = .true.
            enddo
            if(.not.useablehklfile) then
              call outflushlog('direction cosines are missing.')
            endif
          endif
        endif
      else if(d.eq.'readins') then ! command
        call get_file('*.ins',' ',nd,compound,ier)
        if(ier.ne.0) goto 10
c        if(ier.ne.0) goto 900
        call readins(message,ier)
        call errorwarning(message,ier)
        if(ier.ne.0) goto 10
c        if(ier.ne.0) goto 900
        if(windowexists) then
          call WindowTitle(compound)
        endif
        if(menu) then
          call Inputpointgroup(nd,*900)
        else
          if(npoint.eq.0) then
            call getnpoint(pointgroup,npoint)
            if(npoint.eq.0) then
              call warning('unknown pointgroup')
              pointgroup = '?'
            else
              dirty = .true.
            endif
          endif
        endif
        call str_string(pointgroup,'PointGroup #',message)
        call str_out(message)
        call logwrite(message)
      else if(d.eq.'reduce') then ! command
        call readins(message,ier)
        call errorwarning(message,ier)
        if(ier.eq.0) then
          call FaceReduceAll
          call WriteAndRead(ier)
        endif
      else if(d.eq.'replaceface') then ! command
        call replaceface(nd,ier)
        if(ier.ne.0) goto 900
      else if(d.eq.'rmat') then ! command
        call get_file('*.rmat',rmatfile,nd,rmatfile,ier)
        if(ier.eq.0) then
          call readrmatfile(message,ier)
          call str_out(message)
          if(ier.eq.0) then
            call SetupCell
            myrmatavailable = .true.
            if(facesavailable) then
              call FaceTest(NoExtraOutput,DoDeactivate,message,ier)
              call str_out(message)
            endif
            call plotfaces
          endif
        endif
      else if(d.eq.'rotax') then ! no user command
        call InputRotax(dirty,nd,*900)
      else if(d.eq.'rotaxface') then ! no user command
        call InputRotaxFace(dirty,nd,ier,*900)
      else if(d.eq.'rotaxhkl') then ! no user command
        call InputRotaxHKL(dirty,nd,ier,*900)
      else if(d.eq.'save') then ! command
        call savefile(.true.,message,ier)
        call errorwarning(message,ier)
        dirty = .false.
      else if(d.eq.'size') then ! command
        call InputInteger(size,d,dirty,nd,*900)
        newwindow = .true.
      else if(d.eq.'squishhkl') then  ! command
        call get_inp_default(d,nd,'allow cosine difference [0.001]',
     +       *900)
        call get_real(f,d)
        call squishhkl(f)
      else if(d.eq.'status') then ! command
        call savefile(.false.,message,ier)
      else if(d.eq.'show') then ! command
        if(facesavailable.or.myrmatavailable) then
          call Readins(message,ier)
          call errorwarning(message,ier)
          call str_flush
          if(ier.eq.0) then
            call str_string_out(compound,'Compound=#')
            call str_string_out(pointgroup,'pointgroup=#')
            call str_flush
            if(RmatOK()) then
              call write2mat(out,myrmat,mydmat,'rmat','dmat')
              call writecell(out,mydmat,' ')
            endif
          endif
        endif
        if(useablehklfile) then
          if(nref.gt.0) then
            call str_integer_out(nref,'# reflections')
            call str_integer_out(Nunique,'of which # unique.')
            call OutputR2
            call str_flush
          endif
          call str_flush
        endif
      else if(d.eq.'showview') then ! command
        call ShowView
      else if(d.eq.'surface') then
        if(facesOK()) then
          call showfaces('s')
        endif
      else if(d.eq.'test') then ! command
        call FaceTest(ExtraOutput,DoNotDeactivate,message,ier)
        call errorwarning(message,ier)
      else if(d.eq.'test2') then ! no user command
        call FaceTest(ExtraOutput,DoDeactivate,message,ier)
        call errorwarning(message,ier)
      else if(d.eq.'thetamin') then ! command
        call InputPosAngle(thetamin,d,dirty,nd,*900)
      else if(d.eq.'tilt') then ! command
        call tilt(nd,dirty,*900)
      else if(d.eq.'tol') then ! command
        call Inputtol(tol,dirty,nd,*900)
      else if(d.eq.'vertical') then ! command
        call InputBoolean(vertical,d,dirty,nd,*900)
        call plotfaces
      else if(d.eq.'viewxyz') then ! command
        call str_real_len_push(5)
        do i=1,3
          call str_string(xyz(i),'View# [#]',prompt)
          call str_real(viewvec(i),prompt,prompt)
          call get_inp_default(d,nd,prompt,*900)
          call get_real(viewvec(i),d)
        enddo
        call str_real_len_pop
        newworld = .true.
        viewrot = 0.0
        dirty = .true.
        call PlotFaces
      else if(d.eq.'viewpartial') then ! command
        call InputPosAngle(viewpartial,d,dirty,nd,*900)
      else if(d.eq.'viewrot') then ! command
        call InputAngle(viewrot,d,dirty,nd,*900)
        call PlotFaces
      else if(d.eq.'viewperp') then ! command
        if(facesOK()) then
          call str_integer(nfaces,'Face number 1-# [1]',prompt)
          call get_inp_default(d,nd,prompt,*900)
          call get_integer(i,d)
          if(FaceInRange(i)) then
            call getvec(facexyz,i,viewvec)
            newworld = .true.
            viewrot = 0.0
            dirty = .true.
            call PlotFaces
          endif
        endif
      else if(d.eq.'viewdir') then ! command
        if(RmatOK()) then
          call MatVec(mydmat,viewvec,v)
          call scalevector(25,v,v)
          call copyvecFI(v,ihkl)
          call Inputhkl(ihkl,nd,*900)
          call MatVecFIF(myrmat,ihkl,viewvec)
          newworld = .true.
          viewrot = 0.0
          dirty = .true.
          call PlotFaces
        endif
      else if(d.eq.'viewspeed') then ! command
        call InputPosAngle(viewspeed,d,dirty,nd,*900)
      else if(d.eq.'volume') then ! command
        call setuprefinement(ier)
        if(ier.ne.0) goto 10
        call RefineVolume(message,ier)
        call errorwarning(message,ier)
      else if(d.eq.'weight') then ! command
        call InputReal(weight1,'weight Rint',dirty,nd,*900)
        call InputReal(weight2,'weight Rpsi',dirty,nd,*900)
      else if(d.eq.'windowinfo') then
        if(windowexists) then
          call WindowInfo(6)
        endif
      else if(d.eq.'windowlog') then ! command
        call get_inp_default(d,nd,'Window log level (1-7) [4]',*900)
        call get_integer(j,d)
        call setwindowlog(j)
      else if(d.eq.'workfile') then ! command
        call str_string(workfile,'Workfilename [#]',prompt)
        call get_inp_default(d,nd,prompt,*900)
      else if(d.eq.'writeins') then ! command
        if(FacesOK()) then
          call WriteInsFile(compound,message,ier)
          call errorwarning(message,ier)
        endif
      else if(d.eq.'writehkl') then ! command
        call writehklfile(workfile,message,ier)
        call errorwarning(message,ier)
      else if(d.eq.'xtal') then ! command
        call InputXtal(nd,*900)
        call showfaces(' ')
        call TestAndSave(Ier)
      else
        call str_string(d,'Unknown #',message)
        call warning(message)
      endif
      call str_flush
      goto 10
  900 continue
      call logwrite(d)
      if(graphics) call WindowExit
      call logwrite('Euhedral finished')
      close(log)
      call finish(out)
      if(ier.eq.0) then
        call exitnormal
      else
        call exitfatal
      endif
      end

      logical function allowed(im,hb)
      integer im
      integer hb
      include 'euhedral.inc'
      allowed = (mI(pMn,im,hb).ge.nhklmin.and.
     +     mR(pMtheta,im,hb).ge.thetamin.and.
     +     mR(pMmean,im,hb).ge.intmin.and.
     +     mR(pMangular,im,hb).le.angdistmax)
      return
      end

      real FUNCTION BRENT(AX,BX,CX,F,TOL,XMIN)
      real    ax
      real    bx
      real    cx
      real    f
      real    tol
      real    xmin
c---- functions
      external f
      logical  interrupted
c---- constants
      integer itmax
      real    cgold
      real    zeps
      PARAMETER (ITMAX=100,CGOLD=.3819660,ZEPS=1.0E-10)
c---- common
      logical abortrefinement
      common /common_refinement/ abortrefinement
c---- local vars
      real    a,b
      real    u,v,w,x
      real    d,e,etemp
      real    fu,fx,fv,fw
      integer iter
      real    xm
      real    tol1,tol2
      real    p,q,r
c
      A=MIN(AX,CX)
      B=MAX(AX,CX)
      V=BX
      W=V
      X=V
      E=0.
      d=0.0 ! prevent compiler warning
      abortrefinement = .false.
      FX=F(X)
      if(interrupted()) then
        fx = 999.9
        xmin = 999.9
        goto 3
      endif
      FV=FX
      FW=FX
      DO 11 ITER=1,ITMAX
        XM=0.5*(A+B)
        TOL1=TOL*ABS(X)+ZEPS
        TOL2=2.*TOL1
        IF(ABS(X-XM).LE.(TOL2-.5*(B-A))) GOTO 3
        IF(ABS(E).GT.TOL1) THEN
          R=(X-W)*(FX-FV)
          Q=(X-V)*(FX-FW)
          P=(X-V)*Q-(X-W)*R
          Q=2.*(Q-R)
          IF(Q.GT.0.) P=-P
          Q=ABS(Q)
          ETEMP=E
          E=D
          IF(ABS(P).GE.ABS(.5*Q*ETEMP).OR.P.LE.Q*(A-X).OR. 
     *        P.GE.Q*(B-X)) GOTO 1
          D=P/Q
          U=X+D
          IF(U-A.LT.TOL2 .OR. B-U.LT.TOL2) D=SIGN(TOL1,XM-X)
          GOTO 2
        ENDIF
1       IF(X.GE.XM) THEN
          E=A-X
        ELSE
          E=B-X
        ENDIF
        D=CGOLD*E
2       IF(ABS(D).GE.TOL1) THEN
          U=X+D
        ELSE
          U=X+SIGN(TOL1,D)
        ENDIF
        FU=F(U)
        if(interrupted()) then
          fx = 999.9
          xmin = 999.9
          goto 3
        endif
        IF(FU.LE.FX) THEN
          IF(U.GE.X) THEN
            A=X
          ELSE
            B=X
          ENDIF
          V=W
          FV=FW
          W=X
          FW=FX
          X=U
          FX=FU
        ELSE
          IF(U.LT.X) THEN
            A=U
          ELSE
            B=U
          ENDIF
          IF(FU.LE.FW .OR. W.EQ.X) THEN
            V=W
            FV=FW
            W=U
            FW=FU
          ELSE IF(FU.LE.FV .OR. V.EQ.X .OR. V.EQ.W) THEN
            V=U
            FV=FU
          ENDIF
        ENDIF
11    CONTINUE
      PAUSE 'Brent exceed maximum iterations.'
3     XMIN=X
      BRENT=FX
      RETURN
      END                       ! brent

      real function ParabolicDistanceFunk(x)
      real      x
      include 'euhedral.inc'
      real      r
      facefactor(refineface1) = x
      if(refineface2.gt.0) facefactor(refineface2) = x
      write(correctlabel,100) x
  100 format(f7.4)
      call correct(r)
      ParabolicDistanceFunk = r
      return
      end                       ! ParabolicDistanceFunk

      logical function FaceInRange(n)
      integer n
      include 'euhedral.inc'
      character message*256
      logical test
c
      if(n.gt.0.and.n.le.nfaces) then
        test = .true.
      else
        call str_2integer(n,nfaces,'Face # out of range (1-#)',message)
        call warning(message)
        test = .false.
      endif
      FaceInRange = test
      return
      end                       ! FaceInRange

      logical function FacesOK()
      include 'euhedral.inc'
      if(.not.facesavailable) then
        call warning('No faces available')
      endif
      FacesOK = facesavailable
      return
      end                       ! RmatOK

      integer function hklhash(h1,k1,l1)
      integer h1,k1,l1
      include 'euhedral.inc'
      hklhash=1+mod(128*(128*abs(h1)+abs(k1))+abs(l1),HASHBINS)
      return
      end                       ! hklhash

      real function R2WithFilter()
      real r2
      R2WithFilter = R2(.false.)
      return
      end

      real function R2NoFilter()
      real r2
      R2NoFilter = R2(.true.)
      return
      end

      real function R2(always)
      logical always
      include 'euhedral.inc'
c---- functions
      logical allowed
c---- local vars
      integer im,hb
      real    rpsi
      real    rint
      integer n
      real    sumpsi
      real    sumr
      real    sumi
      real    mean
      integer p

      if(Nunique.eq.0) then
        r2 = 0.0
        return
      endif

      sumr = 0.0
      sumi = 0.0
      sumpsi = 0.0
      n = 0
      do hb = 1,hashbins
        do im = 1,nmother(hb)
          if(mI(pMn,im,hb).gt.1) then
            if(always.or.allowed(im,hb)) then
              sumpsi = sumpsi + mR(pMmax,im,hb)/mR(pMmin,im,hb) - 1
              n = n + 1
              sumi = sumi + mR(pMsum,im,hb)
              mean = mR(pMsum,im,hb)/mI(pMn,im,hb)
              p = mI(pMfirst,im,hb)
              do while (p.ne.0)
                sumr = sumr + abs(dR(pDintensity,p)-mean)
                p = dI(pDsister,p)
              enddo
            endif
          endif
        enddo
      enddo
      if(n.gt.0) then
        rpsi = sumpsi/n
        rint = sumr/sumi
        r2 = WEIGHT1*RINT + WEIGHT2*RPSI
      else
        r2 = 999.9
      endif
      return
      end                       ! R2

      logical function RmatOK()
      include 'euhedral.inc'
      if(.not.myrmatavailable) then
        call warning('Read insfile first')
      endif
      RmatOK = myrmatavailable
      return
      end                       ! RmatOK

      real function SimplexDistanceFunk(x)
      include 'euhedral.inc'
      real      x(maxparam)
c----
      integer   ndim
      integer   pointer(2,maxparam)
      common /amoeba_com/ ndim,pointer
c---- local vars
      real      r
      integer   i
      logical   possible

      possible = .true.
      do i=1,ndim
        if(x(i).le.0.0) then
          possible = .false.
        endif
        facefactor(pointer(1,i)) = x(i)
        if(pointer(2,i).ne.0) then
          facefactor(pointer(2,i)) = x(i)
        endif
      enddo
      write(correctlabel,600) (facefactor(pointer(1,i)),i=1,ndim)
  600 format(12f6.3)
      if(.not.possible) then
        call str_out(correctlabel)
        call str_out('Impossible')
        call str_flush
        SimplexDistanceFunk = 999.9
      else
        call correct(r)
        SimplexDistanceFunk = r
      endif
      return
      end                       ! SimplexDistanceFunk

      real function SimplexOrientationFunk(x)
      include 'const.inc'
      include 'euhedral.inc'
      real      x(3)
c---- local vars
      real      r
      real      rotmat(3,3)
      integer   i
      call SimplexRotmat(x,rotmat)
      write(correctlabel,600) (x(i)*grad,i=1,3)
  600 format(3f7.2)
      call FaceOrient(.false.,rotmat)
      call correct(r)
      call restorefaces
      SimplexOrientationFunk = r
      return
      end                       ! SimplexOrientationFunk

      real function VolumeFunk(x)
      real      x
      include 'euhedral.inc'
c---- local vars
      real      r
      integer   i
      if(x.lt.0.1) then
        VolumeFunk = badr
        write(correctlabel,100) x
        return
      endif
      do i=1,nfaces
        facefactor(i) = x
      enddo
      write(correctlabel,100) x
  100 format(f7.4)
      call correct(r)
      VolumeFunk = r
      return
      end                       ! VolumeFunk

      subroutine AddFace(extra,friedel,v,d,ier)
      logical extra             ! INPUT controls amount of output
      logical friedel           ! INPUT do/donot add friedel
      real    v(3)              ! INPUT face normal in laboratory system
      real    d                 ! INPUT face distance
      integer ier               ! OUTPUT 0=ok
      include 'euhedral.inc'
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='AddFace')
c---- functions
      real    AbsVec
c---- local vars      
      real    hkl(3)
      real    scaled(3)
      integer ihkl(3)
      character message*256
c
      if(absVec(v).lt.1.0e-5) then
        call str_out('Zero length vector')
        ier = 1
        return
      endif
      if(d.lt.1.0e-5) then
        ier = 1
        call str_out('Zero distance')
        return
      endif
      call matvec(mydmat,v,hkl)
      call scalevector(25,hkl,scaled)
      call copyvecFI(scaled,ihkl)
      if(extra) then
        call str_real_len_push(5)
        message = '# v= # # # hkl= # # # scaled= # # # HKL= # # #'
        call str_string(ROUTINE,message,message)
        call str_vector(v,message,message)
        call str_vector(hkl,message,message)
        call str_vector(scaled,message,message)
        call str_iarray(3,ihkl,message,message)
        call str_real_len_pop
        call str_out(message)
        call logwrite(message)
      endif
      call addfaceHKL(ihkl,d,ier)
      if(ier.eq.0) then
        if(friedel) then
          call facvec(-1.0,scaled,scaled)
          call copyvecFI(scaled,ihkl)
          call addfaceHKL(ihkl,d,ier)
        endif
        if(extra) call str_flush
      endif
      return
      end                       ! AddFace

      subroutine AddFaceHKL(ihkl,dist,ier)
      integer ihkl(3)           ! INPUT face indices
      real    dist              ! INPUT face distance
      integer ier               ! OUTPUT
      include 'euhedral.inc'
      logical new
      integer npos
      if(nfaces.lt.maxfaces) then
        call addvecuniqueI(ihkl,facehkl,nfaces,npos,new)
        if(new) then
          facedist(nfaces) = dist
          facefactor(nfaces) = 1.0
          faceactive(nfaces) = .true.
          facesurface(nfaces) = -1.0
          ier = 0
        else 
          ier = 1
          call str_iarray_out(3,ihkl,'Not added # # #')
          call str_integer_out(npos,'Already in list at #')
          call str_flush
        endif
      else
        call TooManyFatal(nfaces,maxfaces,'faces')
      endif
      return
      end                       ! AddFaceHKL

      subroutine AddRotax(f1,f2,f3)
      real f1  ! INPUT components of rotax in laboratory system
      real f2
      real f3
      include 'euhedral.inc'
      real v(3)
c
      if(nrotaxes.lt.maxrotaxes) then
        v(1) = f1
        v(2) = f2
        v(3) = f3
c      if(graphics) call plotvector(v,colfg)
        nrotaxes = nrotaxes + 1
        call savevec(v,rotaxes,nrotaxes)
      else
        call TooManyFatal(nrotaxes,maxrotaxes,'rotaxes')
      endif
      return
      end                       ! AddRotax

      SUBROUTINE AMOEBA(P,Y,MP,NP,NDIM,FTOL,FUNK,ITER)
c-----  minimises
      real      p
      real      y
      integer   mp
      integer   np
      integer   ndim
      real      ftol
      real      funk
      external  funk
      integer   iter
c---- functions
      logical   interrupted
c---- constants
      logical debug
      parameter (debug=.true.)
      character*(*) ROUTINE
      parameter (ROUTINE='Amoeba')
c---- local vars
      DIMENSION P(MP,NP)
      dimension Y(MP)

      integer   nmax
      PARAMETER (NMAX=20)
      real      alpha
      parameter (ALPHA=1.0)
      real      beta
      parameter (BETA=0.5)
      real      gamma
      parameter (GAMMA=2.0)
      integer   itmax
      parameter (ITMAX=500)  ! 500
      real      pr(nmax)
      real      PRR(NMAX)
      real      PBAR(NMAX)
      integer   mpts
      integer   ILO
      integer   ihi
      integer   inhi
      integer   i
      real      rtol
      integer   j
      real      ypr
      real      yprr
      character form*80

      MPTS=NDIM+1
      ITER=0
1     ILO=1
      IF(Y(1).GT.Y(2))THEN
        IHI=1
        INHI=2
      ELSE
        IHI=2
        INHI=1
      ENDIF
      DO I=1,MPTS
        IF(Y(I).LT.Y(ILO)) ILO=I
        IF(Y(I).GT.Y(IHI))THEN
          INHI=IHI
          IHI=I
        ELSE IF(Y(I).GT.Y(INHI))THEN
          IF(I.NE.IHI) INHI=I
        ENDIF
      enddo
      RTOL=2.*ABS(Y(IHI)-Y(ILO))/(ABS(Y(IHI))+ABS(Y(ILO)))
      if(debug) then
        call str_flush
        call str_out(ROUTINE)
        call str_integer_out(iter,'cycle #')
        call str_integer_out(ihi,'ihi=#')
        call str_real_out(y(ihi),'y(ihi)=#')
        call str_integer_out(ilo,'ilo=#')
        call str_real_out(y(ilo),'y(ilo)=#')
        call str_real_out(rtol,'rtol #')
        call str_flush
        call str_integer(ndim,'(i2,#f7.3,f9.5)',form)
        do i=1,mpts
          write(6,form) i,(p(i,j),j=1,ndim),y(i)
        enddo
      endif
      IF(RTOL.LT.FTOL) RETURN
      IF(ITER.EQ.ITMAX) then
        call str_out(ROUTINE)
        call str_out('Amoeba exceeding maximum iterations.')
        call str_flush
        return
      endif
      if(interrupted()) goto 900
      ITER=ITER+1
      DO J=1,NDIM
        PBAR(J)=0.
      enddo
      DO I=1,MPTS
        IF(I.NE.IHI)THEN
          DO J=1,NDIM
            PBAR(J)=PBAR(J)+P(I,J)
          enddo
        ENDIF
      enddo
      DO J=1,NDIM
        PBAR(J)=PBAR(J)/NDIM
        PR(J)=(1.+ALPHA)*PBAR(J)-ALPHA*P(IHI,J)
      enddo
      if(debug) then
        call str_integer_out(ihi,'Reflect #.')
        call str_flush
      endif
      YPR=FUNK(PR)
      if(interrupted()) goto 900
      IF(YPR.LE.Y(ILO))THEN
        if(debug) call str_out('Better than the best.')
        DO J=1,NDIM
          PRR(J)=GAMMA*PR(J)+(1.-GAMMA)*PBAR(J)
        enddo
        if(debug) then
          call str_out('Try to extrapolate.')
          call str_flush
        endif
        YPRR=FUNK(PRR)
        if(interrupted()) goto 900
        IF(YPRR.LT.Y(ILO))THEN
          if(debug) call str_out('Extrapolation better than best.')
          DO J=1,NDIM
            P(IHI,J)=PRR(J)
          enddo
          Y(IHI)=YPRR
          if(debug) call str_integer_out(IHI,'Saved to #.')
        ELSE
          if(debug) call str_out('Extrapolation failed.')
          DO J=1,NDIM
            P(IHI,J)=PR(J)
          enddo
          Y(IHI)=YPR
          if(debug) call str_integer_out(IHI,'Reflect saved to #.')
        ENDIF
      ELSE IF(YPR.GE.Y(INHI))THEN
        IF(YPR.LT.Y(IHI))THEN
          if(debug) call str_out(
     +         'Between highest and highest-1.')
          DO J=1,NDIM
            P(IHI,J)=PR(J)
          enddo
          Y(IHI)=YPR
          if(debug) call str_integer_out(IHI,'Saved to #.')
        else
          if(debug) call str_out('Worse than highest.')
        ENDIF
        DO J=1,NDIM
          PRR(J)=BETA*P(IHI,J)+(1.-BETA)*PBAR(J)
        enddo
        if(debug) then
          call str_out('Try intermediate.')
          call str_flush
        endif
        YPRR=FUNK(PRR)
        if(interrupted()) goto 900
        IF(YPRR.LT.Y(IHI))THEN
          if(debug) call str_out('Better than highest.')
          DO J=1,NDIM
            P(IHI,J)=PRR(J)
          enddo
          Y(IHI)=YPRR
          if(debug) call str_integer_out(IHI,'Saved to #.')
        ELSE
          if(debug) then
            call str_out('Cannot get rid of highest.')
            call str_out('Contract around lowest.')
          endif
          DO I=1,MPTS
            IF(I.NE.ILO)THEN
              DO J=1,NDIM
                PR(J)=0.5*(P(I,J)+P(ILO,J))
                P(I,J)=PR(J)
              enddo
              if(debug) then
                call str_integer_out(i,'Contracting #.')
                call str_flush
              endif
              Y(I)=FUNK(PR)
              if(debug) call str_integer_out(I,'Saved to #.')
              if(interrupted()) goto 900
            ENDIF
          enddo
        ENDIF
      ELSE
        if(debug) call str_out('Middling.')
        DO J=1,NDIM
          P(IHI,J)=PR(J)
        enddo
        Y(IHI)=YPR
        if(debug) call str_integer_out(IHI,'Saved to #.')
      ENDIF
      GO TO 1
  900 continue
      call str_flush
      call str_string_out(ROUTINE,'# interrupted')
      return
      END                       ! Amoeba

      subroutine BestSphere(d)
      character d*(*)           ! INPUT: refinement order
      include 'euhedral.inc'
c---- functions
      integer   length
      logical   interrupted
c---- local vars
      real      rmin
      integer   besttype
      character compoundsave*80
      integer   i
      character message*256
      integer   ier
      character order*10
      integer   j

      call upcase(d,order)
      ier = 0
      do i=1,length(order)
        if(order(i:i).ne.'V'.and.order(i:i).ne.'D'.and.
     +       order(i:i).ne.'A') then
          if(ier.eq.0) call str_out('unknown option:')
          call str_out(order(i:i))
          ier = 1
        endif
      enddo
      if(ier.ne.0) return

      rmin = badr
      besttype = 0
      compoundsave = compound
      do i=1,MaxSphere
        call str_integer(i,'model#',compound)
        call str_integer_out(i,'Trying type #')
        if(windowexists) then
          call str_out('Set title')
          call WindowTitle(compound)
        endif
        call BuildSphere(i,ier)
        if(ier.ne.0) return
        call FaceTest(NoExtraOutput,DoNotDeactivate,message,ier)
        if(ier.eq.0) then
          call WriteInsFile(compound,message,ier)
          call errorwarning(message,ier)
          if(ier.ne.0) return
          do j=1,length(order)
            call setuprefinement(ier)
            if(ier.ne.0) return
            if(movie) call ViewPartialRotation
            if(order(j:j).eq.'V') then
              call RefineVolume(message,ier)
            else if(order(j:j).eq.'D') then
              call RefineDistanceParabolic(message,ier)
            else if(order(j:j).eq.'A') then
              call RefineDistanceSimplex(message,ier)
            else
              call str_string_out(order(j:j),'Unknown #')
            endif
            call errorwarning(message,ier)
            if(ier.ne.0) goto 10
            if(interrupted()) goto 10
          enddo
          if(initr.lt.rmin) then
            besttype = i
            rmin = initr
          endif
        else
          call str_out(message)
        endif
        call str_flush
      enddo
   10 continue
      compound = compoundsave
      if(besttype.ne.0) then
        call str_integer_out(besttype,'Best type=#')
        call str_real_out(rmin,'rmin=#')
        call str_flush
        call str_integer(besttype,'model#',compound)
        call readins(message,ier)
        call str_out(message)
        if(ier.eq.0) then
          call FaceTest(ExtraOutput,DoNotDeactivate,message,ier)
        endif
        if(ier.eq.0) then
          compound = compoundsave
          call WriteAndRead(ier)
          if(windowexists) call WindowTitle(compound)
        endif
      endif
      return
      end                       ! BestSphere

      subroutine BuildCorners
      include 'euhedral.inc'
c---- constants
      logical debug
      parameter (debug=.false.)
      character*(*) ROUTINE
      parameter (ROUTINE='BuildCorners')
c---- functions
      real    MatVol
      real    ScalProd
c---- local vars
      integer iface
      integer jface
      integer kface
      integer i
      real    V(3)
      real    V1(3)
      real    V2(3)
      real    V3(3)
      real    W(3)
      real    D(3)
      real    M1(3,3)
      real    M2(3,3)
      real    det1
      integer pos
      real    dist
      integer bingo
      integer icorner
c
      ncorners = 0
      if(debug) call str_real_len_push(6)

      do iface=1,nfaces-2
        if(faceactive(iface)) then
          call getvec(facexyz,iface,V1)
          call Vec2row(V1,M1,1)
          D(1) = facefactor(iface)*facedist(iface)
          do jface=iface+1,nfaces-1
            if(faceactive(jface)) then
              call getvec(facexyz,jface,V2)
              call Vec2row(V2,M1,2)
              D(2) = facefactor(jface)*facedist(jface)
              do kface=jface+1,nfaces
                if(faceactive(kface)) then
                  call getvec(facexyz,kface,V3)
                  call vec2row(V3,M1,3)
                  D(3) = facefactor(kface)*facedist(kface)
                  det1 = MatVol(M1)
                  if(debug) then
                    call str_3integer_out(iface,jface,kface,'(#,#,#)')
                    call str_real_out(det1,'det=#')
                  endif
                  if(abs(det1).gt.zeroDET) then
                    do i=1,3
                      call CopyMat(M1,M2)
                      call Vec2col(D,M2,i)
                      V(i) = MatVol(M2)/det1
                    enddo
                    if(debug) then
                      call str_real_len_push(5)
                      call str_vector_out(V,'point')
                      call str_real_len_pop
                    endif
                    call PosVec(V,cornerxyz,ncorners,zeroXYZ,pos)
                    if(pos.ne.0) then ! already found
                      if(debug) call str_integer_out(pos,'listpos=#')
                      FaceCornerTable(iface,pos) = .true.
                      FaceCornerTable(jface,pos) = .true.
                      FaceCornerTable(kface,pos) = .true.
                      goto 10
                    endif
c---- point v lies in iface,jface and kface
c---- calculate distance from v to other faces
c---- if distance is larger, this is no valid point
                    do i=1,nfaces
                      if(faceactive(i)) then
                        if(debug) then
                          call str_integer_out(i,'Face #')
                          call str_real_out(facefactor(i)*facedist(i),
     +                         'Dist #')
                          call getvec(facexyz,i,W)
                          call str_real_out(scalprod(V,W),'dist=#')
                          call str_flush
                        endif
                        if(i.ne.iface.and.i.ne.jface.and.
     +                       i.ne.kface) then
                          call getvec(facexyz,i,W)
                          dist = scalprod(V,W)
                          if(dist.gt.facefactor(i)*facedist(i)+zeroDIST)
     +                         then ! outside
                            if(debug) then
                              call str_integer_out(i,'face #')
                              call str_real_out(dist,'dist=#')
                            endif
                            goto 10
                          endif
                        endif
                      endif     ! faceactive(i)
                    enddo
                    if(ncorners.ge.maxcorners) then
                      call str_flush
                      call str_out(ROUTINE)
                      call TooManyFatal(ncorners,maxcorners,'corners')
                    endif
                    ncorners = ncorners + 1
                    do i=1,nfaces
                      FaceCornerTable(i,ncorners) = .false.
                    enddo
                    FaceCornerTable(iface,ncorners) = .true.
                    FaceCornerTable(jface,ncorners) = .true.
                    FaceCornerTable(kface,ncorners) = .true.
                    call SaveVec(V,cornerxyz,ncorners)
                    if(debug) call str_integer_out(ncorners,'==>#')
                  endif
   10             continue
                  if(debug) call str_flush
                endif           ! faceactive(kface)
              enddo             ! kface
            endif               ! faceactive(jface)
          enddo                 ! jface
          if(debug) call operator
        endif                   ! faceactive(iface)
      enddo                     ! iface

      nedges = 0
      do iface=1,nfaces-1
        if(faceactive(iface)) then
          do jface = iface+1,nfaces
            if(faceactive(jface)) then
              bingo = 0
              do icorner=1,ncorners
                if(FaceCornerTable(iface,icorner).and.
     +               FaceCornerTable(jface,icorner)) then
                  if(bingo.eq.0) then
                    if(debug) then
                      call str_integer_out(iface,'Face1=#')
                      call str_integer_out(jface,'Face2=#')
                      call str_integer_out(icorner,'from corner #')
                    endif
                    bingo = icorner
                  else
                    if(nedges.ge.maxedges) then
                      call str_flush
                      call str_out(ROUTINE)
                      call TooManyFatal(nedges,maxedges,'edges')
                    endif
                    nedges = nedges + 1
                    edgecorner(1,nedges) = bingo
                    edgecorner(2,nedges) = icorner
                    edgeface(1,nedges) = iface
                    edgeface(2,nedges) = jface
                    if(debug) then
                      call str_integer_out(icorner,'to #')
                      call str_flush
                    endif
                  endif
                endif
              enddo             ! icorner
              if(debug) call str_flush
            endif               ! faceactive(jface)
          enddo                 ! jface
        endif                   ! faceactive(iface)
      enddo                     ! iface
      if(debug) call str_real_len_pop
      return
      end                       ! BuildCorners

      subroutine BuildCube(n,ier)
      integer n                 ! INPUT
      integer ier               ! OUTPUT
      include 'axes.inc'
      include 'euhedral.inc'
      real      v(3)
      character message*256
      nfaces = 0
      ncorners = 0
      nedges = 0
      newworld = .true.
      ier = 0
      if(n.eq.1) then
        call AddFace(NoExtraOutput,AddFriedel,x_axis,initdistance,ier)
        call AddFace(NoExtraOutput,AddFriedel,y_axis,initdistance,ier)
        call AddFace(NoExtraOutput,AddFriedel,z_axis,initdistance,ier)
      else if(n.eq.2) then
        call build_vector(1.0,1.0,0.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
        call build_vector(1.0,-1.0,0.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
        call AddFace(NoExtraOutput,AddFriedel,z_axis,initdistance,ier)
      else if(n.eq.3) then
        call build_vector(1.0,0.0,1.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
        call AddFace(NoExtraOutput,AddFriedel,y_axis,initdistance,ier)
        call build_vector(1.0,0.0,-1.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
      else if(n.eq.4) then
        call AddFace(NoExtraOutput,AddFriedel,x_axis,initdistance,ier)
        call build_vector(0.0,1.0,1.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
        call build_vector(0.0,1.0,-1.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
      else if(n.eq.5) then
        call build_vector(1.0,1.0,1.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
        call build_vector(1.0,-1.0,0.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
        call build_vector(-1.0,-1.0,1.0,v)
        call AddFace(NoExtraOutput,AddFriedel,v,initdistance,ier)
      else
        call str_integer(n,'Unknown Cube type #',message)
        call warning(message)
        ier = 1
      endif
      return
      end                       ! BuildCube
      
      subroutine BuildFaces(extra,deactivate,ndeactivated,message,ier)
      logical   extra
      logical   deactivate
      integer   ndeactivated
      character message*(*)
      integer   ier
      include 'euhedral.inc'
c---- constants
      logical   debug
      parameter (debug=.false.)
      integer   FlagIdentical
      integer   FlagNew
      integer   FlagLost
      parameter (FlagIdentical = 1)
      parameter (FlagNew = 2)
      parameter (FlagLost = 3)
      integer   Flag1
      integer   Flag2
      parameter (Flag1 = FlagIdentical)
      parameter (Flag2 = FlagLost)
      character*(*) ROUTINE
      parameter (ROUTINE='BuildFaces')
c---- functions
      real      scalprod
      integer   length
c---- local vars
      integer   i
      integer   iface
      integer   icorner
      integer   jcorner
      integer   kcorner
      real      s

      real      V(3)
      real      W(3)
      integer   pos
      real      V1(3)
      real      V2(3)
      real      V3(3)
      integer   newfaces
      real      newfacexyz(3,maxfaces)
      real      newfacedist(maxfaces)
      integer   flags(maxfaces)
      integer   VI(3)
      integer   normerr
      real      V13(3)
      real      V21(3)
      real      V32(3)
      integer   counter(Flag1:Flag2)
      character string*80
      integer   n
      character counterlabel(Flag1:Flag2)*20
      data counterlabel /'unchanged','new','lost'/

c
c--- copy all the existing faces to a new list
c--- and flag them LOST
c
      ndeactivated = 0
      do iface=1,nfaces
        call getvec(facexyz,iface,V)
        call savevec(V,newfacexyz,iface)
        newfacedist(iface) = facedist(iface)*facefactor(iface)
        flags(iface) = FlagLost
      enddo
      newfaces = nfaces
c
c---- build a face from every 3 corners
c---- face normal is perpendicular to v2-v1 and v3-v1
c
      if(debug) call str_real_len_push(5)
      do icorner=1,ncorners
        call getvec(cornerxyz,icorner,V1)
        do jcorner=icorner+1,ncorners
          call getvec(cornerxyz,jcorner,V2)
          call DifVec(V2,V1,V21)
          do kcorner=jcorner+1,ncorners
            if(debug) then
              call str_3integer_out(icorner,jcorner,kcorner,'(#,#,#)')
            endif
            call getvec(cornerxyz,kcorner,V3)
            call DifVec(V1,V3,V13)
            call DifVec(V3,V2,V32)
            call getNormal(V13,V21,V32,W)
            call normvec(W,W,normerr)
            if(normerr.ne.0) then
              if(debug) call str_out('normerr')
              goto 20
            endif
            s = scalprod(V1,W)
            if(s.lt.0.0) then
              s = -s
              call facvec(-1.0,W,W)
            endif
            if(debug) then
              call str_vector_out(W,'w')
              call str_real_out(s,'s')
            endif
            if(s.lt.zeroDIST) then
              if(debug) call str_out('zero distance')
              goto 20
            endif
c
c---- check if all corners points are within this face
c
            do i=1,ncorners
              if(debug) then
                call str_integer_out(i,'Point #')
                call getvec(cornerxyz,i,V)
                call str_real_out(scalprod(V,W),'dist=#')
              endif
              if(i.ne.icorner.and.i.ne.jcorner.and.i.ne.kcorner) then
                call getvec(cornerxyz,i,V)
                if(scalprod(V,W)-s.gt.zeroDIST) then
c--- this says: point i is too far away from this face
c---            so this face is not ok
                  if(debug)call str_integer_out(i,'Point # outside')
                  goto 20
                endif
              endif
            enddo
c
c---- now see if this is a new face
c
            call PosVec(W,newfacexyz,newfaces,0.01,pos)
            if(pos.ne.0) then   ! face already in list
              if(debug) call str_integer_out(pos,'face #')
              if(abs(newfacedist(pos)-s).lt.zeroDIST) then ! not a new face                
                if(flags(pos).ne.FlagNew) flags(pos) = FlagIdentical
                goto 20
              else
                message = '# Face # (# # #) dist problem. old=# new=#'
                call str_string(ROUTINE,message,message)
                call str_integer(pos,message,message)
                call str_3integer(icorner,jcorner,kcorner,
     +               message,message)
                call str_real_len_push(5)
                call str_real(newfacedist(pos),message,message)
                call str_real(s,message,message)
                call str_real_len_pop
                ier = 1
                goto 900
              endif
            endif
            if(debug)call str_out('new face?')
            newfaces = newfaces + 1
            if(debug)call str_integer_out(newfaces,'==>#')
            call SaveVec(W,newfacexyz,newfaces)
            newfacedist(newfaces) = s
            flags(newfaces) = FlagNew
   20       continue
            if(debug)call str_flush
          enddo
        enddo
      enddo
      if(debug) call str_num_write(out,newfaces,'# face^ build')

      if(debug) then
        call str_flush
        call str_write(out,'old faces')
        do iface=1,nfaces
          call getvec(facexyz,iface,V)
          write(out,700) iface,V,facefactor(i)*facedist(i)
  700     format(i3,3f9.5,f9.5)
        enddo

        call str_write(out,'new faces')
        do iface=1,newfaces
          call getvec(newfacexyz,iface,V)
          write(out,700) iface,V,newfacedist(i)
        enddo
      endif

      do i=Flag1,Flag2
        counter(i) = 0
      enddo
      do iface=1,newfaces
        if(faceactive(iface)) then
          counter(flags(iface)) = counter(flags(iface)) + 1
        endif
      enddo

      if(extra) then
        do i=Flag1,Flag2
          if(counter(i).gt.0) then
            call str_flush
            call str_write(out,' ')
            call str_num(counter(i),'# # face^',string)
            call str_string_write(out,counterlabel(i),string)
            write(out,620)
  620       format('Face   x        y        z        ',
     +           'H   K   L    Dist')
            do iface=1,newfaces
              if(flags(iface).eq.i.and.faceactive(iface)) then
                call getvec(newfacexyz,iface,V)
                call matvec(mydmat,V,W)
                call scalevector(25,W,W)
                call CopyVecFI(W,VI)
                write(6,630) iface,V,VI,newfacedist(iface)
  630           format(i4,3f9.5,3i4,f9.5)
              endif
            enddo               ! iface
          endif
        enddo                   ! i
      endif ! extra

      if(counter(FlagNew).eq.0.and.counter(FlagLost).eq.0) then
        ier = 0
        n = 0
        do iface=1,nfaces
          if(faceactive(iface))  n = n + 1
        enddo
        call str_num(n,'# active face^.',message)
      else
        message = ' '
        do i=Flag1,Flag2
          if(i.ne.FlagIdentical) then
            if(counter(i).gt.0) then
              string = '# # face^ ('
              call str_num(counter(i),string,string)
              call str_string(counterlabel(i),string,string)
              do iface=1,newfaces
                if(flags(iface).eq.i.and.faceactive(iface)) then
                  call str_integer(iface,string,string)
                endif
              enddo
              call str_string(string,'# ).',string)
              if(length(message).eq.0) then
                message = string
              else
                call str_2string(message,string,'# #',message)
              endif
            endif
          endif
        enddo
        ier = 1
        if(counter(FlagNew).eq.0) then
          if(deactivate) then
            call str_out('Deactivating')
            do iface=1,newfaces
              if(flags(iface).eq.FlagLost) then
                ndeactivated = ndeactivated + 1
                faceactive(iface) = .false.
                call str_integer_out(iface,'#')
              endif
            enddo
            call str_flush
            ier = 0
          endif
        else
c---- here are new faces          
          if(.not.extra) then
            call str_flush
            write(out,620)
            do iface=1,newfaces
              if(flags(iface).eq.FlagNew) then
                call getvec(newfacexyz,iface,V)
                call matvec(mydmat,V,W)
                call scalevector(25,W,W)
                call CopyVecFI(W,VI)
                write(6,630) iface,V,VI,newfacedist(iface)
              endif
            enddo               ! iface
          endif
        endif
      endif
  900 continue
      if(debug) call str_real_len_pop
      return
      end                       ! BuildFaces

      subroutine BuildFibonacciOrient(cycles)
      integer   cycles
      include 'const.inc'
      include 'euhedral.inc'
c---- constants
      logical   debug
      parameter (debug=.false.)
c---- local vars
      integer   iqa,iqb,iqc
      integer   k
      integer   jj
      real      xb
      real      yb
      real      v(3)
      real      ph
      real      cost
      real      sint
      call str_num_out(cycles,'Build Fibonacci rotaxes in # cycle^')
      call str_flush
      call plotcircle

      nrotaxes = 0
      iqa = 0
      iqb = 1
      do while((iqa+iqb)/2.lt.cycles)
        iqc = iqa + iqb
        iqa = iqb
        iqb = iqc
      enddo

      jj=0

      do k=1,iqb/2
        xb=float(k)/float(iqb)
        jj=(jj+iqa)
        if(jj.gt.iqb) jj = jj - iqb
        yb=float(jj)/float(iqb)
        ph=2*Pi*yb
        cost=1-2.0*xb
        sint=sqrt(1-cost**2)
        call Build_Vector(sint*cos(ph),sint*sin(ph),cost,v)
        if(nrotaxes.lt.maxrotaxes) then
          if(graphics) call plotvector(v,cFG)
          nrotaxes = nrotaxes + 1
          call savevec(v,rotaxes,nrotaxes)
        endif
      enddo                     ! k
      call str_integer_out(nrotaxes,'Generated # rotaxes')
      return
      end                       ! BuildFibonacciOrient

      subroutine BuildModel(d,nd,dirty,*)
      character d*(*)
      integer   nd
      logical   dirty
      include 'euhedral.inc'
c---- local vars
      character d2*80
      integer   i
      character prompt*80
      integer   ier
c
      d2 = d(6:)
      if(menu) call InputDistance(initdistance,dirty,nd,*900)
      if(d2.eq.'cube') then
        call str_integer(maxCube,'cube type (1-#) [1]',prompt)
      else if(d2.eq.'sphere') then
        call str_integer(maxSphere,'sphere type (1-#) [1]',prompt)
      else
        call str_string(d2,'Unknown Build option:#',prompt)
        call warning(prompt)
      endif
      call get_inp_default(d,nd,prompt,*900)
      call get_integer(i,d)
      if(d2.eq.'cube') then
        call BuildCube(i,ier)
      else if(d2.eq.'sphere') then
        call BuildSphere(i,ier)
      endif
      if(ier.ne.0) return
      call showfaces(' ')
      call TestAndSave(Ier)
      return
  900 continue
      return 1
      end                       ! BuildModel

      subroutine BuildMother
      include 'const.inc'
      include 'euhedral.inc'
c---- constants      
      logical   debug
      parameter (debug=.false.)
c---- local vars
      real    AbsVec
      integer ih,ik,il
      integer im,hb
      integer i
      integer k
      integer p
      real    hkl(3)
      real    c(3)
      real    sumint
      real    coso(3)
      real    sv(3)
      real    minint
      real    maxint
      character form*80
      real    f
c
      Nunique = 0

      do hb=1,HASHBINS
        nmother(hb) = 0
      enddo

      im = 0
      do i=1,nref
        call mylaue(npoint,dI(pDh,i),dI(pDk,i),dI(pDl,i),ih,ik,il)
        if(debug) then
          call str_integer_out(i,'nrefl=#')
          call str_3integer_out(dI(pDh,i),dI(pDk,i),dI(pDl,i),
     +         '(#,#,#)')
          call str_3integer_out(ih,ik,il,'->(#,#,#)')
        endif
        if(im.ne.0) then
          if(ih.ne.mI(pMh,im,hb).or.ik.ne.mI(pDk,im,hb).or.
     +         il.ne.mI(pMl,im,hb)) then
            if(debug) call str_out('search')
            call searchmother(ih,ik,il,hb,im)
          endif
        else
          if(debug) call str_out('initial search')
          call searchmother(ih,ik,il,hb,im)
        endif
        if(debug) then
          call str_2integer_out(hb,im,'hb=# im=#')
          call str_flush
        endif
        if(im.eq.0) then
          Nunique = Nunique + 1
          if(nmother(hb).ge.MAXMOTHER) then
            call str_2integer_write(out,hb,MAXMOTHER,
     +           'Too many reflections in hashbin #. Max=#')
            call exitfatal
          endif
          nmother(hb) = nmother(hb) + 1
          im = nmother(hb)
          mI(pMh,im,hb) = ih
          mI(pMk,im,hb) = ik
          mI(pMl,im,hb) = il
          mI(pMn,im,hb) = 1
          mI(pMfirst,im,hb) = i
        else
c----     add this reflection to to list
          mI(pMn,im,hb) = mI(pMn,im,hb) + 1
          p = mI(pMfirst,im,hb)
          do while(dI(pDsister,p).ne.0)
            p = dI(pDsister,p)
          enddo
          dI(pDsister,p) = i  ! last item in list points to new refl
        endif
        dI(pDsister,i) = 0 ! this is now the last
      enddo


      do hb = 1,hashbins
        do im = 1,nmother(hb)
          hkl(1) = mI(pMh,im,hb)
          hkl(2) = mI(pMk,im,hb)
          hkl(3) = mI(pMl,im,hb)

          call MatVec(myrmat,hkl,c)
          
          mR(pMtheta,im,hb) = asin(lambda / 2 * AbsVec(c))

          sumint = 0.0
          maxint = 0.0
          minint = 0.0
          do i=1,3
            sv(i) = 0.0
          enddo
          p = mI(pMfirst,im,hb)
          maxint = dR(pDintensity,p)
          minint = maxint
          do while(p.ne.0)
            
            do i=1,3
              coso(i) = dR(pDcosinesF+2*i-1,p)*axstar(i)   ! still in a*b*c* system, as cosines
            enddo
            call MatVec(mydmatTrans,coso,coso) ! now in XYZ system. coso = outray????
            call SumVec(coso,sv,sv)
            f = dR(pDintensity,p)
            sumint = sumint + f
            maxint = max(maxint,f)
            minint = min(minint,f)
            p = dI(pDsister,p)
          enddo
          mR(pMsum,im,hb) = sumint
          mR(pMmean,im,hb) = sumint/mI(pMn,im,hb)
          mR(pMmax,im,hb) = maxint
          mR(pMmin,im,hb) = max(minint,0.01) ! i.e. minint > 0.0
          if(mI(pMn,im,hb).gt.1) then
            mR(pMangular,im,hb) = AbsVec(sv)/(mI(pMn,im,hb)-1)
          else
            mR(pMangular,im,hb) = 999.9
          endif
        enddo
      enddo

      if(debug) then
        call str_flush
        do hb=1,hashbins
          if(nmother(hb).gt.0) then
            call str_integer(pMIMAX,'(#i5)',form)
            write(out,form) (mI(k,1,hb),k=1,pMIMAX)
            call str_integer(pMRMAX,'(#F14.5)',form)
            write(out,form) (mR(k,1,hb),k=1,pMRMAX)
            p = mI(pMfirst,1,hb)
            do while(p.ne.0)
              call str_integer(pDIMAX,'(2x,#i5)',form)
              write (out,form) (dI(k,p),k=1,pDIMAX)
              call str_integer(pDRMAX,'(2x,#f14.5)',form)
              write (out,form) (dR(k,p),k=1,pDRMAX)
              p = dI(pDsister,p)
            enddo
          endif
        enddo
      endif
      return
      end                       ! BuildMother

      subroutine BuildOrient
      include 'euhedral.inc'
      nrotaxes = 0
      call AddRotax( 0.0, 1.0, 1.0)
      call AddRotax( 1.0, 0.0, 1.0)
      call AddRotax( 1.0, 1.0, 0.0)
      call AddRotax( 0.0,-1.0, 1.0)
      call AddRotax(-1.0, 0.0, 1.0)
      call AddRotax(-1.0, 1.0, 0.0)
      call AddRotax(-1.0, 1.0, 1.0)
      call AddRotax( 1.0,-1.0, 1.0)
      call AddRotax(-1.0,-1.0, 1.0)
      call AddRotax( 1.0, 0.0, 0.0)
      call AddRotax( 0.0, 1.0, 0.0)
      call AddRotax( 0.0, 0.0, 1.0)
      call AddRotax( 1.0, 1.0, 1.0)
      return
      end                       ! BuildOrient


      subroutine BuildOrientationList
      include 'const.inc'
      include 'euhedral.inc'
c---- constants
      logical   debug
      parameter (debug=.false.)
c---- local vars
      real      v(3)
      real      f
      real      rax(3)
      real      phi
      real      rotmat(3,3)
      real      m1(3,3)
      real      m2(3,3)
      integer   irotax
      integer   imatrix
      integer   nmat
      integer   nrejected

      call str_flush
      call str_out('Building orientation list')
      call str_grad_out(phistart,'From #')
      call str_grad_out(phistop,'to #')
      call str_grad_out(delphi,'in # degree steps.')
      call str_grad_out(angmin,'angmin = # ')
      call str_flush
      nrejected = 0
      nmatrices = 1
      call unit_matrix(rotmat)
      call savemat(rotmat,matrices,nmatrices)
      do irotax=1,nrotaxes
        call str_integer_out(irotax,'rotax #')
        call getvec(rotaxes,irotax,rax)
        if(debug) then
          call str_vector_out(rax,'rotax')
          call str_flush
        endif
        nmat = nmatrices
        do phi=phistart,phistop,delphi
          if(debug) call str_grad_out(phi,'phi=#')
          call GeneralRotmat(rax,phi,rotmat)
          do imatrix=1,nmatrices
            call getmat(matrices,imatrix,m1)
            call GetRotMat(rotmat,m1,m2)
            call GetRotAng(m2,v,f)
            if(abs(f).le.angmin) then
              nrejected = nrejected + 1
              if(debug) then
                call str_integer_out(imatrix,'Angle with matrix #')
                call str_grad_out(f,'= #')
                call str_out('rejected')
              endif
              goto 10
            endif
          enddo
          if(nmatrices.lt.maxmatrices) then
            nmatrices = nmatrices + 1
            call savemat(rotmat,matrices,nmatrices)
            if(debug) call str_integer_out(nmatrices,'stored at #')
          else
            call str_integer_out(maxmatrices,
     +           'Maximum number of orientations = #')
            goto 900
          endif
   10     continue
          if(debug) call str_flush
        enddo
        call str_integer_out(nmatrices-nmat,': #')
        call str_flush
      enddo
  900 continue
      call str_num_out(nrejected,'Rejected # orientation^.')
      call str_num_out(nmatrices,'Generated # orientation^')
      call str_flush
      return
      end                       ! BuildOrientationList

      subroutine BuildSphere(itype,ier)
      integer itype
      integer ier
      include 'euhedral.inc'
      character message*256
      nfaces = 0
      ncorners = 0
      nedges = 0
      newworld = .true.
      ier = 0
      if(itype.eq.1) then
        call BuildSphereTemplate(1,1,initdistance)
      else if(itype.eq.2) then
        call BuildSphereTemplate(2,1,initdistance)
      else if(itype.eq.3) then
        call BuildSphereTemplate(1,2,initdistance)
      else if(itype.eq.4) then
        call BuildSphereTemplate(3,1,initdistance)
      else if(itype.eq.5) then
        call BuildSphereTemplate(1,3,initdistance)
      else if(itype.eq.6) then
        call BuildSphereTemplate(3,2,initdistance)
      else if(itype.eq.7) then
        call BuildSphereTemplate(2,3,initdistance)
      else
        call str_integer(itype,'Bad Sphere Type:#',message)
        call warning(message)
        ier = 1
      endif
      return
      end                       ! BuildSphere

      subroutine BuildSphereTemplate(ia,ib,d)
      integer ia
      integer ib
      real    d
c---- local vars
      real    a
      real    b
      real    v(3)
      logical extra
      logical friedel
      integer ier
c
      extra = .false.
      friedel = .true.
      a = float(ia)
      b = float(ib)
      call build_vector(a,b,0.0,v)
      call AddFace(extra,friedel,v,d,ier)
      call build_vector(b,0.0,a,v)
      call AddFace(extra,friedel,v,d,ier)
      call build_vector(0.0,a,b,v)
      call AddFace(extra,friedel,v,d,ier)
      call build_vector(-a,b,0.0,v)
      call AddFace(extra,friedel,v,d,ier)
      call build_vector(-b,0.0,a,v)
      call AddFace(extra,friedel,v,d,ier)
      call build_vector(0.0,-a,b,v)
      call AddFace(extra,friedel,v,d,ier)
      return
      end                       ! BuildSphereTemplate

      subroutine CalculateSurface
      include 'euhedral.inc'
c---- local vars
      integer   iface
      integer   icorner
      real      V(3)
      real      W(3)
      real      VIEWMAT(3,3)
      integer   n
      real      hor(maxcorners)
      real      ver(maxcorners)
c
      do iface=1,nfaces
        facesurface(iface) = -1.0
        if(faceactive(iface)) then
          call getvec(facexyz,iface,V)
          call makeviewmat(V,VIEWMAT)
          n = 0
          do icorner=1,ncorners
            if(FaceCornerTable(iface,icorner)) then
              call getvec(cornerxyz,icorner,V)
              call matvec(VIEWMAT,V,W)
              n = n + 1
              hor(n) = W(1)
              ver(n) = W(2)
            endif
          enddo
          if(n.gt.2) then
            call surface(n,hor,ver,faceSurface(iface))
          endif
        endif
      enddo
      return
      end                       ! CalculateSurface

      subroutine Cell2D(ier)
      integer ier
      include 'euhedral.inc'
c
      call str_vector_out(ax,'Axes')
      call str_gradvector_out(ang,'Angles')
      mydmat(1,1) = ax(1)
      mydmat(1,2) = 0.0
      mydmat(1,3) = 0.0

      mydmat(2,1) = ax(2) * COS(ang(3))
      mydmat(2,2) = ax(2) * SIN(ang(3))
      mydmat(2,3) = 0.0

      mydmat(3,1) = ax(3) * COS(ang(2))
      mydmat(3,2) = (ax(2)*ax(3)*COS(ang(1))-mydmat(2,1)*mydmat(3,1))/
     +     mydmat(2,2)
      mydmat(3,3) = ax(3)**2 - mydmat(3,1)**2 - mydmat(3,2)**2
      if(mydmat(3,3).lt.0.01) then
        call str_write(out,'impossible cell...')
        ier = 1
      else
        mydmat(3,3) = sqrt(ABS(mydmat(3,3)))
        call InvertMat(mydmat,myrmat,ier)
      endif
      return
      end                       ! Cell2D

      subroutine CheckAbort
      integer   unit
      character message*256
      integer   ier
c---- constants
      character*(*) abortfile
      parameter (abortfile='euhedral.abort')
c
      call open_file(abortfile,unit,message,ier)
      if(ier.eq.0) then
        close(unit,status='delete')
        call str_string_out(abortfile,'removed #')
        call ccself
      endif
      return
      end                       ! CheckAbort

      subroutine CheckCoinciding(message,ier)
      character message*(*)
      integer   ier
      include 'euhedral.inc'
c---- functions
      real      angv2
c---- local vars
      integer   iface
      integer   jface
      real      V(3)
      real      W(3)
      real      angle
c
      ier = 0
      do iface=1,nfaces
        call getvec(facexyz,iface,V)
        do jface=iface+1,nfaces
          call getvec(facexyz,jface,W)
          angle = angv2(v,w)
          if(angle.lt.zeroang) then
            if(ier.eq.0) then
              message = 'Coinciding FaceNormals #,#'
            else
              call str_string(message,'# #,#',message)
            endif
            call str_2integer(iface,jface,message,message)
            ier = 1
          endif
        enddo
      enddo
      if(ier.eq.0) then
        message = 'No coinciding face normals'
      endif
      return
      end                       ! CheckCoinciding

      subroutine CompareFaces(AllTheSame)
      logical   AllTheSame
      include 'euhedral.inc'
c---- functions
      logical   SameVecI
c---- local vars
      integer   i
      integer   hkl1(3)
      integer   hkl2(3)
      character same*1
c
      AllTheSame = .true.
      call str_flush
      write(out,600)
  600 format('   previous      current')
      write(out,610)
  610 format('   H   K   L     H   K   L')
      do i=1,nfaces
        call getVecI(facesave,i,hkl1)
        call getVecI(facehkl,i,hkl2)
        if(.not.SameVecI(hkl1,hkl2)) then
          AllTheSame = .false.
          same = '*'
        else
          same = ' '
        endif
        write(out,620) hkl1,hkl2,same
  620   format(3i4,2x,3i4,1x,a)
      enddo
      return
      end                       ! CompareFaces

      subroutine correct(r)
      real      r
      include 'euhedral.inc'
c---- functions
      real      r2NoFilter
      logical   interrupted
c---- local vars
      integer   unit
      character message*256
      character string*256
      integer   ier
      character command*80
      character hkpfile*80
      character name*80
      integer   i

      name = workfile

      call FaceTest(NoExtraOutput,DoDeactivate,message,ier)
      if(ier.ne.0) then
        call str_out(correctlabel)
        call str_out(message)
        call logwrite2(correctlabel,message)
        goto 900
      endif

      call WriteInsFile(name,message,ier)
      if(ier.ne.0) then
        call warning(message)
        call exitfatal
      endif
c
c---- delete existing 'workfile'.hkp
c
      call str_string(name,'#.hkp',hkpfile)
      call open_file(hkpfile,unit,message,ier)
      if(ier.eq.0) then
        close(unit,status='delete')
      endif

      if(menu) then
        call str_out(correctlabel)
      else 
        call str_flush
        call str_out('IGNORE:')
      endif
c      call str_string_out(correctlabel,'[#]')
C
C PERFORM TOMPA ABSORPTION-CORRECTION WITH PLATON
C
      call str_string(name,'platon #.ins > /dev/null',command)
      if(interrupted()) then
        ier = 1
        goto 900
      endif
      call spawncc(command,ier)
      if(ier.ne.0) then
        call str_string_out(command,'Cannot #')
        call str_integer_out(ier,'error = #')
        goto 900
      endif
      if(.not.menu) then
        call str_out('platon finished')
        call str_flush
        call str_out(correctlabel)
      endif
      call CheckAbort
      if(interrupted()) then
        ier = 1
        goto 900
      endif
      call testshape(name,string,ier)
      if(ier.ne.0) then
        call warning(string)
      else
        call readhkl(hkpfile,string,ier)
        useablehklfile = .false.
        if(interrupted()) goto 900
        if(ier.ne.0) then
          call warning(string)
          call ccself
          goto 900
        else
          r = r2NoFilter()
          call outr(r,string)
          call str_out(string)
          call str_flush
        endif
      endif
      call logwrite2(correctlabel,string)
  900 continue
      do i=1,nfaces
        faceactive(i) = .true.
      enddo
      call str_flush
      if(ier.ne.0) then
        r = badr
      endif
      return
      end                       ! correct

      subroutine DeleteFace(nd,ier)
      integer   nd
      integer   ier
      include 'euhedral.inc'
c---- functions
      logical   FaceInRange
c---- local vars
      integer   np
      character d*80
      integer   i
      integer   fn(maxfaces)
      character message*256
c
      if(nd.gt.0) then
        np = 0
        do while(nd.gt.0)
          call get_inp(d,nd,' ',*900)
          call get_integer(i,d)
          if(FaceInRange(i)) then
            np = np + 1
            fn(np) = i
          endif
        enddo
        do i=1,np
          faceactive(fn(i)) = .false.
        enddo
        call FaceTest(ExtraOutput,DoNotDeactivate,message,ier)
        call errorwarning(message,ier)
        if(ier.eq.0) then
          call WriteAndRead(ier)
        else
          do i=1,np
            faceactive(fn(i)) = .true.
          enddo
          call str_out('Faces not removed')
          call FaceTest(NoExtraOutput,DoNotDeactivate,message,ier)
        endif
      endif
      ier = 0
      return
  900 continue
      ier = 1
      return
      end                       ! DeleteFace

      subroutine EditFace(nd,*)
      integer nd
      include 'euhedral.inc'
c---- functions
      logical FaceInRange
c---- local vars
      integer i
      integer j
      integer ier
      character d*80
      character prompt*80
      integer   ihkl(3)

      if(nfaces.gt.0) then
        call str_integer(nfaces,'face number 1-# [1]',prompt)
        call get_inp_default(d,nd,prompt,*900)
        call get_integer(j,d)
        if(FaceInRange(j)) then
          call GetVecI(facehkl,j,ihkl)
          do i=1,3
            call str_string(hklname(i),'# [#]',prompt)
            call str_integer(ihkl(i),prompt,prompt)
            call get_inp_default(d,nd,prompt,*900)
            call get_integer(ihkl(i),d)
          enddo
          call str_real_len_push(4)
          call str_real(facedist(j),'Distance [#]',prompt)
          call str_real_len_pop
          call get_inp_default(d,nd,prompt,*900)
          call get_real(facedist(j),d)
          call SaveVecI(ihkl,facehkl,j)
          call TestAndSave(Ier)
        endif
      else
        call str_out('No faces available')
      endif
      return
  900 continue
      return 1
      end                       ! EditFace

      subroutine ErrorWarning(message,ier)
      character message*(*)
      integer   ier
      if(ier.eq.0) then
        call str_out(message)
      else
        call warning(message)
      endif
      return
      end                       ! ErrorWarning

      subroutine FaceHigh(ihkl)
      integer ihkl(3)
      include 'euhedral.inc'
      integer i
      integer j

      if(abs(ihkl(1)).le.highindex.and.
     +     abs(ihkl(2)).le.highindex.and.
     +     abs(ihkl(3)).le.highindex) then
        j = highindex
        do while(j*abs(ihkl(1)).gt.highindex.or.
     +       j*abs(ihkl(2)).gt.highindex.or.
     +       j*abs(ihkl(3)).gt.highindex)
          j = j - 1
        enddo
        do i=1,3
          ihkl(i) = j*ihkl(i)
        enddo
      endif
      return
      end                       ! FaceHigh

      subroutine FaceHighAll
      include 'euhedral.inc'
      integer i
      integer ihkl(3)

      do i=1,nfaces
        call GetVecI(facehkl,i,ihkl)
        call FaceHigh(ihkl)
        call SaveVecI(ihkl,facehkl,i)
      enddo
      return
      end                       ! FaceHighAll

      subroutine FaceHKL2XYZ
      include 'euhedral.inc'
      integer ihkl(3)
      real    v(3)
      integer normerr
      integer i
      do i=1,nfaces
        call getvecI(facehkl,i,ihkl)
        call MatVecFIF(myrmat,ihkl,v)
        call NormVec(v,v,normerr)
        call SaveVec(v,facexyz,i)
      enddo
      return
      end                       ! FaceHKL2XYZ

      subroutine EuhedralInit
      include 'axes.inc'
      include 'const.inc'
      include 'euhedral.inc'

c---- local vars
      integer i
      integer ier
c---- init
      hklname(1) = 'h'
      hklname(2) = 'k'
      hklname(3) = 'l'
      lambda = -1.0
      pointgroup = '?'
      npoint = 0
      compound = ' '
      rmatfile = ' '
      subrname = ' '
      workfile = 'subfile'
      correctionmethod = 'ABST'
      tol = 0.01
      extreme = 1.0
      nref = 0
      initdistance = 0.07
      myrmatavailable = .false.
      facesavailable = .false.
      cellcheck = .false.
      useablehklfile = .false.
      do i=1,3
        xtaldim(i) = initdistance
        xtalrot(i) = 0.0
      enddo
c---- filter
      nhklmin = 2
      thetamin = 12.0 / Grad
      intmin = 200.0
      angdistmax = 0.6
c---- misc
      centricshape = .true.
      weight1 = 10.0
      weight2 = 0.1
      initr = -1.0
      badr = 999.9
      zeroDET  = 0.001
      zeroDIST = 0.0001
      zeroXYZ  = 0.0001
      zeroANG  = 0.01/grad
      improve  = 0.001
      distype = 'S'
      deltatilt = 1
      OrientSimplexAngle = PiOver2
c---- graphics
      windowexists = .false.
      call initwindow(ier)
      graphics = (ier.eq.0)
      if(graphics) then
        colournames(cFG)='fg'
        colournames(cBG)='bg'
        colournames(cCOSI)='inray'
        colournames(cCOSO)='outray'
        colournames(cCIRCLE)='circle'
        colournames(cAXESFG)='axesfg'
        colournames(cAXESBG)='axesbg'
        colournames(cFACESFG)='facesfg'
        colournames(cFACESBG)='facesbg'
        colournames(cCVECTOR)='cvector'
        colours(cFG) = 'white'
        colours(cBG) = 'grey30'
        colours(cCOSI) = 'green'
        colours(cCOSO) = 'magenta'
        colours(cCIRCLE) = 'grey40'
        colours(cAXESFG) = 'yellow'
        colours(cAXESBG) = 'yellow4'
        colours(cFACESFG) = 'orange'
        colours(cFACESBG) = 'brown'
        colours(cCVECTOR) = 'yellow'
        call ScreenDimensions(screenx,screeny)
        size = screeny/3
        posx = (screenx-size)/4
        posy = (screeny-size)*3/4 ! size
        newwindow = .true.
        pointsize = 3
        vectorsync = .true.
        newworld = .true.
        call CopyVec(z_axis,viewvec)
        viewspeed = 0.5/grad
        viewrot = 0.0
        viewpartial = 20.0/grad
        doplotcorner = .false.
        doplotaxes = .true.
        doplotlab = .true.
        movie = .true.
        WindowShape = ShapeSmall
        vertical = .true.
      endif
c---- projections
      ang109 = acos(-1.0/3.0)
      phifact = (TwoPi - ang109) / TwoPi
      cona = acos(sqrt(1.0-phifact**2))
      coscona = cos(cona)
      projmodes(1) = 'cone'
      projmodes(2) = 'stereographic'
      projmodes(3) = 'ajmd'
      projmodes(4) = 'z'
      projectionmode = 2
      HighIndex = 5
      delphi = 45.0/grad
      phistart = 0.0
      phistop = 359.9/grad
      angmax = 15.0/grad
      Angmin=15.0/grad
      nrotaxes = 0
      nmatrices = 0

      return
      end                       ! EuhedralInit

      subroutine FaceOrient(extra,rotmat)
      logical   extra
      real      rotmat(3,3)
      include 'axes.inc'
      include 'const.inc'
      include 'euhedral.inc'
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='FaceOrient')
c---- functions
      real      angv2
c---- local vars
      real      rax(3)
      real      rang

      integer   ihkl(3)
      real      hkl(3)
      real      c(3)
      real      crot(3)
      real      shkl(3)
      integer   nhkl(3)
      integer   i
      real      c2(3)
      real      f
      character message*256
      real      rm(3,3)

      if(extra) then
        call GetRotAng(rotmat,rax,rang)
        message = '# Rotax=[#,#,#] phi=#'
        call str_string(ROUTINE,message,message)
        call str_real_len_push(5)
        call str_vector(rax,message,message)
        call str_real_len_pop
        call str_grad(rang,message,message)
        call logwrite(message)
      endif

      call MatMat(rotmat,myrmat,rm)

      if(extra) then
        call str_flush
        write(out,700)
  700   format('nr   h   k   l   cx      cy      cz      ',
     +       'cxrot   cyrot   czrot   fh    fk    fl    ',
     +       'h   k   l     ang')
      endif

      do i=1,nfaces
        call getvecI(facehkl,i,ihkl)
        call MatVecFIF(myrmat,ihkl,c) ! goto laboratory
        call matvec(rotmat,c,crot) ! rotate
c---- previous two lines can also be done by next line
c        call MatVec(rm,hkl,crot)
        call matvec(mydmat,crot,hkl) ! goto reciprocal
        call ScaleVector(40,hkl,shkl)
        call CopyVecFI(shkl,nhkl)

        if(extra) then
          call matvecFIF(myrmat,nhkl,c2)
          f = AngV2(crot,c2)*grad
          write(out,710) i,ihkl,c,crot,hkl,nhkl,f
  710     format(i2,3i4,3f8.4,3f8.4,3f6.2,3i4,f8.2)
        endif
        call SaveVecI(nhkl,facehkl,i)
      enddo
      return
      end                       ! FaceOrient

      subroutine FaceReduce(hkl1,hkl2,changed)
      integer hkl1(3)
      integer hkl2(3)
      logical changed
      integer j
      integer k
      logical apply
      integer m
c
      m = 0
      do j=1,3
        m = max(m,abs(hkl1(j)))
      enddo
      call CopyVecI(hkl1,hkl2)
      changed = .false.
      do k=m,2,-1
        apply = .true.
        do j=1,3
          if( k*(hkl1(j)/k).ne.hkl1(j)) apply = .false.
        enddo
        if(apply) then
          do j=1,3
            hkl2(j) = hkl1(j)/k
          enddo
          changed = .true.
          return
        endif
      enddo
      return
      end                       ! FaceReduce

      subroutine FaceReduceAll
      include 'euhedral.inc'
      integer i
      integer hkl1(3)
      integer hkl2(3)
      logical changed
      do i=1,nfaces
        call GetVecI(facehkl,i,hkl1)
        call FaceReduce(hkl1,hkl2,changed)
        if(changed) then
          call str_integer_out(i,'Face #')
          call str_iarray_out(3,hkl1,'(#,#,#)')
          call str_iarray_out(3,hkl2,'-> (#,#,#)')
          call str_flush
          call saveVecI(hkl2,facehkl,i)
        endif
      enddo
      return
      end                       ! FaceReduceAll

      subroutine FaceTest(extra,deactivate,message,ier)
      logical   extra
      logical   deactivate
      character message*(*)
      integer   ier
      include 'euhedral.inc'
      integer   ndeactivated
c
      call FaceTestPart1(extra,message,ier)
      if(ier.ne.0) return
      call BuildFaces(extra,deactivate,ndeactivated,message,ier)
      if(ndeactivated.gt.0) then
        call FaceTestPart1(extra,message,ier)
      endif
      if(ier.ne.0) return
      call plotfaces
      return
      end                       ! FaceTest

      subroutine FaceTestPart1(extra,message,ier)
      logical   extra
      character message*(*)
      integer   ier
      include 'euhedral.inc'
      integer   n
      integer   i
      n = 0
      do i=1,nfaces
        if(faceactive(i)) n = n + 1
      enddo
      if(n.lt.4) then
        call str_num(n,'only # active face^',message)
        ier = 1
        return
      endif
      call FaceHKL2XYZ
      call CheckCoinciding(message,ier)
      if(ier.ne.0) return
      call BuildCorners
      if(ncorners.eq.0) then
        message = 'no corners'
        ier = 1
        return
      endif 
      call CalculateSurface
      if(extra) then
        call PrintCorners
        call PrintFaceAngles
      endif
      return
      end                       ! FaceTestPart1

      subroutine FaceTilt(iface,ier)
      integer   iface
      integer   ier
      include 'const.inc'
      include 'euhedral.inc'
c---- constants
      integer   maxsol
      parameter (maxsol=(2*MaxDeltaTilt+1)**3)
c---- functions
      logical   interrupted
      real      AngV2
      character*(*) ROUTINE
      parameter (ROUTINE='FaceTilt')
c---- local vars
      character message*256
      integer   dh
      integer   dk
      integer   dl
      real      r
      integer   besthkl(3)
      integer   ihkl(3)
      integer   starthkl(3)
      logical   allzero
      integer   hklred(3)
      logical   changed
      real      c0(3)
      real      c(3)
      integer   npos
      logical   new
      real      startr
      real      angle
      logical   skipped
      integer   removedfaces
      integer   j
      integer   nsol
      integer   solhkl(3,maxsol)
c
      call logstart(ROUTINE)
      skipped = .false.         ! compiler warning
    1 continue

      call showfaces(' ')
      call GetVecI(facehkl,iface,ihkl)
      call CopyVecI(ihkl,starthkl)

      message = 'Tilting Face # (#,#,#)'
      call str_integer(iface,message,message)
      call str_iarray(3,ihkl,message,message)
      call logwrite(message)
      call str_out(message)
      call str_flush

c      initr = -1.0
      write(out,620)
  620 format('   H   K   L     h   k   l Angle    R2')

      call MatVecFIF(myrmat,starthkl,c0)
      call FaceReduce(starthkl,hklred,changed)
      if(changed) then
        write(correctlabel,100) starthkl,hklred,0.0
  100   format(3i4,2x,3i4,f6.1)
      else
        write(correctlabel,110) starthkl,0.0
  110   format(3i4,2x,12x,f6.1)
      endif
      call correct(startr)
      call CopyVecI(starthkl,besthkl)
      nsol = 0
      call AddVecUniqueI(hklred,solhkl,nsol,npos,new)
      do dh=-DeltaTilt,DeltaTilt
        ihkl(1) = starthkl(1) + dh
        do dk=-DeltaTilt,DeltaTilt
          ihkl(2) = starthkl(2) + dk
          do dl=-DeltaTilt,DeltaTilt
            ihkl(3) = starthkl(3) + dl
            allzero = (ihkl(1).eq.0.and.ihkl(2).eq.0.and.ihkl(3).eq.0)
            if(allzero) goto 10
            call FaceReduce(ihkl,hklred,changed)
            call MatVecFIF(myrmat,ihkl,c)
            if(nsol.ge.maxsol) then
              call str_out('Increase parameter maxsol')
              call TooManyFatal(nsol,maxsol,'solutions')
            endif
            call AddVecUniqueI(hklred,solhkl,nsol,npos,new)
            if(.not.new) then
              write(correctlabel,100) ihkl,hklred
              skipped = .true.
              message = '*'
              goto 10
            endif
            call SaveVecI(ihkl,facehkl,iface)
            angle = angv2(c0,c)
            if(changed) then
              write(correctlabel,100) ihkl,hklred,grad*angle
            else
              write(correctlabel,110) ihkl,grad*angle
            endif
            if(abs(angle).gt.angmax) then
              message = 'angle too big'
              skipped = .true.
              goto 10
            endif
            skipped = .false.
            call correct(r)
            if(interrupted()) goto 20
            if(improved) then
              call CopyVecI(ihkl,besthkl)
            endif
   10       continue
            if(skipped) then
              call str_out(correctlabel)
              call str_out(message)
              call str_flush
              call logwrite2(correctlabel,message)
            endif
          enddo                 ! L
        enddo                   ! k
      enddo                     ! h

   20 continue
      if(bestr.lt.startr) then
        message = '# Face # Best R2=# (#,#,#) set to (#,#,#)'
        call str_string(ROUTINE,message,message)
        call str_integer(iface,message,message)
        call str_real_len_push(5)
        call str_real(bestr,message,message)
        call str_real_len_pop
        call str_iarray(3,starthkl,message,message)
        call str_iarray(3,besthkl,message,message)
        call SaveVecI(besthkl,facehkl,iface)
      else
        message = '# Face # (#,#,#) No Improvement'
        call str_string(ROUTINE,message,message)
        call str_integer(iface,message,message)
        call str_iarray(3,starthkl,message,message)
        call SaveVecI(starthkl,facehkl,iface)
      endif
      call outflushlog(message)
      call facetest(NoExtraOutput,DoDeactivate,message,ier)
      call errorwarning(message,ier)

      removedfaces = 0
      do j=1,nfaces
        if(.not.faceactive(j)) removedfaces = removedfaces + 1
      enddo
      if(removedfaces.gt.0) then
        call outflushlog('Restart with less faces')
        call WriteAndRead(ier)
        if(ier.ne.0) return
        if(movie) then
          call ViewPartialRotation
        else
          call PlotFaces
        endif
        goto 1                  ! restart all over
      endif

      if(movie) then
        call ViewPartialRotation
      else
        call PlotFaces
      endif
      if(interrupted()) then
        call loginterrupt(message)
      else 
        call logend(message)
      endif
      call str_out(message)
      call str_flush
      return
      end                       ! FaceTilt

      subroutine Filter(nd,dirty,*)
      integer   nd
      logical   dirty
      include 'euhedral.inc'
c---- functions
      logical   str_equal
      logical   interrupted
c---- local vars
      character prompt*80
      character d*80
      integer   i
      integer   j
      character message*256
      integer   ier

      if(.not.useablehklfile) then
        call warning('read suitable hkl data first')
        return
      endif
      call printratio
      call OutputR2
   20 continue
      call str_flush
      if(menu) then
        if(graphics) then
          prompt = 'quit or'
          call str_string(projmodes(1),'quit or #',prompt)
          do i=2,maxmodes
            call str_2string(prompt,projmodes(i),'#/#',prompt)
          enddo
          call str_2string(prompt,projmodes(projectionmode),'# [#]',
     +         prompt)
          call get_inp_default(d,nd,prompt,*900)
          j = 0
          do i=1,maxmodes
            if(str_equal(d,projmodes(i))) then
              j = i
              if(projectionmode.ne.j) call logwrite2('mode',d)
              projectionmode = j
            endif
          enddo
        else
          prompt = 'quit or retry [retry]'
          call get_inp_default(d,nd,prompt,*900)
          if(str_equal(d,'Q')) then
            j = 0
          else
            j = 1
          endif
        endif
        
        if(j.eq.0) then
          call str_string(workfile,'write #.hkl [Yes]',prompt)
          call get_inp_default(d,nd,prompt,*900)
          if(str_equal(d,'Y')) then
            call writehklfile(workfile,message,ier)
            call errorwarning(message,ier)
            if(ier.ne.0) goto 900
          endif
        else
          call InputInteger(nhklmin,'nhklmin',dirty,nd,*900)
          if(nhklmin.lt.2) then
            call str_integer_out(nhklmin,'nhklmin (#) not allowed.')
            call str_out('Set to minimum (=2).')
            call str_flush
            nhklmin = 2
          endif
          call InputPosAngle(thetamin,'thetamin',dirty,nd,*900)
          call InputReal(intmin,'intmin',dirty,nd,*900)
          call InputReal(angdistmax,'angdistmax',dirty,nd,*900)
          call PlotCosines(message,ier)
          if(ier.ne.0) call warning(message)
          if(interrupted()) return
          goto 20
        endif
      else
        call PlotCosines(message,ier)
        if(ier.ne.0) call warning(message)
      endif
      return
  900 continue
      return 1
      end                       ! Filter

      subroutine GetFaceList(nd,*)
      integer   nd
      include 'euhedral.inc'
c---- functions
      logical   FaceInRange
c---- local vars
      character d*80
      logical   busy
      integer   i
c
      nlist = 0
      call get_inp_default(d,nd,'face number/all [all]',*900)
      if(d.eq.'all') then
        do i=1,nfaces
          facelist(i) = i
        enddo
        nlist = nfaces
      else
        busy = .true.
        do while(busy)
          call get_integer(i,d)
          if(FaceInRange(i)) then
            nlist = nlist + 1
            facelist(nlist) = i
          else
            nlist = 0
            return
          endif
          if(nd.gt.0) then
            call get_inp(d,nd,'next face',*900)
          else
            busy = .false.
          endif
        enddo
      endif
      return
  900 continue
      return 1
      end                       ! GetFaceList

      subroutine GetNormal(v1,v2,v3,normal)
      real v1(3)
      real v2(3)
      real v3(3)
      real normal(3)
c---- functions
      real absvec
c---- local vars
      real v12(3)
      real v23(3)
      real v31(3)
      real av12
      real av23
      real av31

      call vecProd(v1,v2,v12)
      call vecProd(v2,v3,v23)
      call vecProd(v3,v1,v31)

      call SumVec(v12,v23,normal)
      call SumVec(v31,normal,normal)
      if(absvec(normal).lt.0.1) then
        av12 = AbsVec(v12)
        av23 = AbsVec(v23)
        av31 = AbsVec(v31)
        call CopyVec(v12,normal)
        if(av23.gt.av12) then
          if(av23.gt.av31) then
            call CopyVec(v23,normal)
          else
            call CopyVec(v31,normal)
          endif
        else if(av31.gt.av12) then
          if(av31.gt.av23) then
            call CopyVec(v31,normal)
          else
            call copyvec(v23,normal)
          endif
        endif
      endif
      return
      end                       ! GetNormal

      subroutine GetNpoint(s,ipoint)
      character s*(*)
      integer   ipoint
      integer maxpointgroup
      parameter (maxpointgroup=42)
      character pointgroupnames(maxpointgroup)*6
      character s6*6
      integer   i
      data pointgroupnames /'1','-1','2','M','2/M','222','MM2','MMM',
     +     '4','-4','4/M','422','4MM','-42M','-4M2','4/MMM','3',
     +     '-3','312','321','31M','3M1','-31M','-3M1','6','-6',
     +     '6/M','622','6MM','-6M2','-62M','6/MMM','R3','R-3','R32',
     +     'R3M','R-3M','23','M3','432','-43M','M3M'/
      call upcase(s,s6)
      do i=1,maxpointgroup
        if(s6.eq.pointgroupnames(i)) then
          ipoint = i
          return
        endif
      enddo
      ipoint = 0
      call str_string_out(s,'point group # not found.')
      call str_out('Available:')
      call str_flush
      do i=1,maxpointgroup
        call str_out(pointgroupnames(i))
      enddo
      call str_flush
      return
      end                       ! GetNpoint

      subroutine GetRotAng(inputrotmat,rax,rang)
      real inputrotmat(3,3)
      real rax(3)
      real rang
c---- local vars
      real    dv(3,3)
      integer i
      real    v1(3)
      real    v2(3)
      real    v3(3)
      integer normerror
      real    viewmat(3,3)
      real    t1(3,3)
      real    t2(3,3)
      integer inverror
      real    rotmat(3,3)
      real    rotmatz(3,3)

      call CopyMat(inputrotmat,rotmat)
      call CopyMat(rotmat,dv)
c  dv = dv - Unit_Matrix
      do i=1,3
        dv(i,i) = dv(i,i) - 1.0
      enddo

      call Row2Vec(1,dv,v1)
      call Row2Vec(2,dv,v2)
      call Row2Vec(3,dv,v3)

      call GetNormal(v1,v2,v3,rax)
      call NormVec(rax,rax,normerror)
      if(normerror.ne.0) then
        rang = 0.0
        call unit_matrix(rotmat)
        call Row2Vec(3,rotmat,rax)
        return
      endif

      if(abs(rax(3)).lt.0.001) then
c--- make largest component positive
        if(abs(rax(1)).gt.abs(rax(2))) then
          if(rax(1).lt.0) then
            call facvec(-1.0,rax,rax)
          endif
        else
          if(rax(2).lt.0) then
            call facvec(-1.0,rax,rax)
          endif
        endif
      else if(rax(3).lt.0.0) then
c---- make z positive
        call facvec(-1.0,rax,rax)
      endif
c
c--- to rotate around a vector rotax:
c     build viewmat with z-axis along rotvec: [V]
c     rotate around Z-axis  [R]
c     and back: [V-1]
c     so [V-1].[R].[V] = [T]
c        postmultiply by [V-1]
c        [V-1].[R].[V].[V-1] = [T].[V-1]      [V].[V-1] = [E]
c        [V-1].[R] = [T].[V-1]
c        premultiply by [V]
c        [V].[V-1].[R] = [V].[T].[V-1]        [V-1].[V] = [E]
c        [R] = [V].[T].[V-1]
      call MakeViewMat(rax,viewmat)    ! ==> [V]
      call InvertMat(viewmat,t1,inverror)  ! ==> [t1] = [V-1]
      call MatMat(rotmat,t1,t2)            ! ==> [t2] = [T].[V-1]
      call MatMat(viewmat,t2,rotmatz)      ! ==> [R] = [V].[T].[V-1]
c---- and [R] is a rotation around the Z-axis
      rang = atan2(rotmatz(2,1),rotmatz(1,1))
c   rotmatz:
c                 cosphi -sinphi  0
c                +sinphi  cosphi  0
c                      0       0  1
c

c     call writemat(6,rotmatz,'RotmatZ')

      return
      end                       ! GetRotang

      subroutine GetRotMat(ra,rb,rotmat)
      real      ra(3,3)
      real      rb(3,3)
      real      rotmat(3,3)
c   ra is rotated to rb as: [rotmat].[ra] = [rb].
c   the rotational matrix [rotmat] is obtained from:
c   [rotmat].[ra] = [rb]
c   [rotmat].[ra].[da] = [rb].[da]
c   note: ra and rb are taken rather than a and b because the relation
c   [rotmatt].[da] = [db] does not express the rotation looked for,
c   as [da] and [db] are ROW and not COLUMN vector matrices.
c   ( [rotmat].[da,transposed] = [db,transposed] is an alternative. )
c
c---- local vars
      integer   inverror
      real      t1(3,3)
      real      t2(3,3)
      integer   i
      integer   j
      real      da(3,3)
c
      call InvertMat(ra,da,inverror)
      call MatMat(rb,da,t1)
      call InvertMat(t1,t2,inverror)
      call TransMat(t2,t2)

      do j=1,3
        do i=1,3
          rotmat(i,j) = (t1(i,j)+t2(i,j))/2
        enddo
      enddo
      return
      end                       ! GetRotMat

      subroutine InputAngle(angle,label,dirty,nd,*)
      real      angle
      character label*(*)
      logical   dirty
      integer   nd
      include 'const.inc'
      include 'euhedral.inc'
c---- local vars
      character prompt*80
      character d*80
      real      f

      f = TwoPi+1
      do while(abs(f).gt.TwoPi)
        call str_string(label,'# [#]',prompt)
        call str_grad(angle,prompt,prompt)
        call get_inp_default(d,nd,prompt,*900)
        call get_grad(f,d)
      enddo
      if(f.ne.angle) then
        dirty = .true.
        angle = f
        call logwrite2(label,d)
      endif
      return
  900 continue
      return 1
      end                       ! InputAngle

      subroutine InputBoolean(boolean,label,dirty,nd,*)
      logical   boolean
      character label*(*)
      logical   dirty
      integer   nd
c---- local vars
      character prompt*80
      character d*80
      logical   old
c
      old = boolean
      call str_string(label,'# [#]',prompt)
      call str_onoff(boolean,prompt,prompt)
      call get_inp_default(d,nd,prompt,*900)
      call upcase(d,d)
      if(d.eq.'ON') then
        boolean = .true.
      else if(d.eq.'OFF') then
        boolean = .false.
      else
        call str_2string(label,d,'#: expected ON/OFF. Found #',prompt)
        call warning(prompt)
      endif
      if(old.neqv.boolean) dirty = .true.
      return
  900 continue
      return 1
      end                       ! InputBoolean

      subroutine InputBooleanNodirty(boolean,label,nd,*)
      logical   boolean
      character label*(*)
      integer   nd
c---- local vars
      character prompt*80
      character d*80
c
      call str_string(label,'# [#]',prompt)
      call str_onoff(boolean,prompt,prompt)
      call get_inp_default(d,nd,prompt,*900)
      call upcase(d,d)
      if(d.eq.'ON') then
        boolean = .true.
      else if(d.eq.'OFF') then
        boolean = .false.
      else
        call str_2string(label,d,'#: expected ON/OFF. Found #',prompt)
        call warning(prompt)
      endif
      return
  900 continue
      return 1
      end                       ! InputBooleanNoDirty

      subroutine InputColour(nd,dirty,*)
      integer nd
      logical dirty
      include 'euhedral.inc'
c---- functions
      logical str_equal
c---- local vars
      character prompt*80
      character d*80
      integer   i

      prompt = colournames(1)
      do i=2,cMAX
        call str_2string(prompt,colournames(i),'#/#',prompt)
      enddo
c      prompt = 'axes/bg/circle/cosi/coso/faces/fg'
      call get_inp(d,nd,prompt,*900)
      do i=1,CMAX
        if(str_equal(d,colournames(i))) then
          call str_2string(colournames(i),colours(i),'# colour [#]',
     +         prompt)
          call get_inp_default(colours(i),nd,prompt,*900)
          if(WindowExists) then
            call getcolour(colours(i),col(i))
            if(i.eq.cBG) then
              call DeleteWindow(1)
              WindowExists = .false.
              newwindow = .true.
            endif
          endif
        endif
      enddo
      dirty = .true.
c      newwindow = .true.
      if(WindowExists) then
        call plotfaces
      endif

c      if(str_equal(d,'AXES')) then
c        call str_string(colouraxesfg,'axes fg colour [#]',prompt)
c        call get_inp_default(colouraxesfg,nd,prompt,*900)
c        call str_string(colouraxesbg,'axes bg colour [#]',prompt)
c        call get_inp_default(colouraxesbg,nd,prompt,*900)
c        if(WindowExists) call plotfaces
c      else if(str_equal(d,'BG')) then
c        call str_string(colourbg,'background colour [#]',prompt)
c        call get_inp_default(colourbg,nd,prompt,*900)
c        if(WindowExists) then
c          call DeleteWindow(1)
c          WindowExists = .false.
c          newwindow = .true.
c          call plotfaces
c        endif
c      else if(str_equal(d,'CIRCLE')) then
c        call str_string(colourcircle,'circles [#]',prompt)
c        call get_inp_default(colourcircle,nd,prompt,*900)
c      else if(str_equal(d,'COSI')) then
c        call str_string(colourcosi,'inrays [#]',prompt)
c        call get_inp_default(colourcosi,nd,prompt,*900)
c      else if(str_equal(d,'COSO')) then
c        call str_string(colourcoso,'outrays [#]',prompt)
c        call get_inp_default(colourcoso,nd,prompt,*900)
c      else if(str_equal(d,'FACES')) then
c        call str_string(colourfacesfg,'faces fg colour [#]',prompt)
c        call get_inp_default(colourfacesfg,nd,prompt,*900)
c        call str_string(colourfacesbg,'faces bg colour [#]',prompt)
c        call get_inp_default(colourfacesbg,nd,prompt,*900)
c        if(WindowExists) call plotfaces
c      else if(str_equal(d,'FG')) then
c        call str_string(colourfg,'fg colour [#]',prompt)
c        call get_inp_default(colourfg,nd,prompt,*900)
c      else
c        call str_flush
c        call str_string_out(d,'Unknown: colour #')
c        call get_newline
c      endif
c      dirty = .true.
c      newwindow = .true.
      return
  900 continue
      return 1
      end                       ! InputColour

      subroutine InputDistance(initdistance,dirty,nd,*)
      real      initdistance
      logical   dirty
      integer   nd
c
      call InputReal(initdistance,'initdistance',dirty,nd,*900)
      return
  900 continue
      return 1
      end                       ! InputDistance

      subroutine InputFace(nd,*)
      integer   nd
      include 'axes.inc'
      include 'euhedral.inc'
      integer   i
      real      v(3)
      real      f
      logical   ignoredirty
      character d*80
      character prompt*80
      integer   ier
c
      do i=1,3
        call str_string(xyz(i),'Face normal (xyz) #',prompt)
        call get_inp(d,nd,prompt,*900)
        call get_real(v(i),d)
      enddo
      f = initdistance
      call InputReal(f,'distance',ignoredirty,nd,*900)
      call AddFace(ExtraOutput,NoFriedel,v,f,ier)
      if(ier.eq.0) then
        call TestAndSave(Ier)
      endif
      return
  900 continue
      return 1
      end                       ! InputFace

      subroutine InputFaceHKL(nd,*)
      integer   nd
      include 'axes.inc'
      include 'euhedral.inc'
      integer   i
      character d*80
      character prompt*80
      real      f
      logical   ignoredirty
      integer   ier
      integer   ihkl(3)
c
      do i=1,3
        call str_string(hklname(i),'Face index #',prompt)
        call get_inp(d,nd,prompt,*900)
        call get_integer(ihkl(i),d)
      enddo
      f = initdistance
      call InputReal(f,'distance',ignoredirty,nd,*900)
      call AddFacehkl(ihkl,f,ier)
      if(ier.eq.0) then
        call TestAndSave(ier)
      endif
      return
  900 continue
      return 1
      end                       ! InputFaceHKL

      subroutine InputFacePerp(nd,*)
      integer   nd
      include 'euhedral.inc'
c---- functions
      logical   FaceSOK
      logical   FaceInRange
c---- local vars
      character d*80
      integer   i
      integer   j
      real      v(3)
      real      w(3)
      real      f
      integer   ier
      logical   ignoredirty
c
      if(FacesOK()) then
        call get_inp(d,nd,'enter first face',*900)
        call get_integer(i,d)
        if(FaceInRange(i)) then
          call getvec(facexyz,i,v)
          call get_inp(d,nd,'enter second face',*900)
          call get_integer(j,d)
          if(FaceInRange(j)) then
            call getvec(facexyz,j,w)
            call VecProd(v,w,v)
            f = 0.5*(facedist(i)+facedist(j))
            call InputReal(f,'distance',ignoredirty,nd,*900)
            call AddFace(ExtraOutput,NoFriedel,v,f,ier)
            if(ier.eq.0) then
              call TestAndSave(Ier)
            endif
          endif
        endif
      endif
      return
  900 continue
      return 1
      end                       ! InputFacePerp

      subroutine InputFaceSum(nd,*)
      integer nd
      include 'euhedral.inc'
c---- functions
      logical   FaceSOK
      logical   FaceInRange
c---- local vars
      character d*80
      integer   i
      integer   j
      integer   k
      real      v(3)
      real      w(3)
      real      f
      integer   ier
      integer   normerr
c
      if(FacesOK()) then
        call build_vector(0.0,0.0,0.0,w)
        f = 0.0
        k = 0
        do i=1,nd
          call get_inp(d,nd,'enter face',*900)
          call get_integer(j,d)
          if(FaceInRange(j)) then
            call getvec(facexyz,j,v)
            call sumvec(v,w,w)
            f = f + facedist(j)
            k = k + 1
          endif
        enddo
        if(k.gt.1) then
          call normvec(w,w,normerr)
          f = f/k
          call AddFace(ExtraOutput,NoFriedel,w,f,ier)
          if(ier.eq.0) then
            call TestAndSave(Ier)
          endif
        else
          call str_out('nothing added')
        endif
      endif
      return
  900 continue
      return 1
      end                       ! InputFaceSum

      subroutine InputHkl(ihkl,nd,*)
      integer ihkl(3)
      integer nd
c---- local vars
      character d*80
      character prompt*80
      character hkl*3
      integer   i
c
      hkl = 'hkl'
      do i=1,3
        call str_string(hkl(i:i),'# [#]',prompt)
        call str_integer(ihkl(i),prompt,prompt)
        call get_inp_default(d,nd,prompt,*900)
        call get_integer(ihkl(i),d)
      enddo
      return
  900 continue
      return 1
      end                       ! Inputhkl

      subroutine InputInteger(n,label,dirty,nd,*)
      integer   n
      character label*(*)
      logical   dirty
      integer   nd
c---- local vars
      character d*80
      character prompt*80
      integer   old
      old = n
      call str_string(label,'# [#]',prompt)
      call str_integer(n,prompt,prompt)
      call get_inp_default(d,nd,prompt,*900)
      call get_integer(n,d)
      if(n.ne.old) then
        dirty = .true.
        call logwrite2(label,d)
      endif
      return
  900 continue
      return 1
      end                       ! InputInteger

      subroutine Inputpointgroup(nd,*)
      integer nd
      include 'euhedral.inc'
c---- local vars
      character prompt*80
      character d*80

c
      call str_string(pointgroup,'pointgroup [#]',prompt)
      call get_inp_default(d,nd,prompt,*900)
      call getnpoint(d,npoint)
      if(npoint.eq.0) then
        call str_string(d,'point group # not found',prompt)
        call warning(prompt)
        pointgroup = '?'
      else
        pointgroup = d
        call logwrite2('pg',pointgroup)
      endif
      return
  900 continue
      return 1
      end                       ! Inputpointgroup

      subroutine InputPhi(dirty,nd,*)
      logical dirty
      integer nd
      include 'euhedral.inc'
c
      call InputAngle(phistart,'Phi start',dirty,nd,*900)
      call InputAngle(phistop,'Phi stop',dirty,nd,*900)
      call InputPosAngle(delphi,'Rotation Increment',dirty,nd,*900)
      return
  900 continue
      return 1
      end                       ! InputPhi

      subroutine InputPosAngle(angle,label,dirty,nd,*)
      real      angle
      character label*(*)
      logical   dirty
      integer   nd
      include 'const.inc'
      include 'euhedral.inc'
c---- local vars
      character prompt*80
      character d*80
      real      f
      real      old
      old = angle
      f = -1.0
      do while(f.lt.0.0.or.f.gt.TwoPi)
        call str_string(label,'# [#]',prompt)
        call str_grad(angle,prompt,prompt)
        call get_inp_default(d,nd,prompt,*900)
        call get_grad(f,d)
      enddo
      angle = f
      if(abs(angle-old).gt.0.001) then
        dirty=.true.
        call logwrite2(label,d)
      endif
      return
  900 continue
      return 1
      end                       ! InputPosAngle

      subroutine InputReal(f,label,dirty,nd,*)
      real      f
      character label*(*)
      logical   dirty
      integer   nd
c---- local vars
      character d*80
      character prompt*80
      real      old
      old = f
      call str_string(label,'# [#]',prompt)
      call str_real(f,prompt,prompt)
      call get_inp_default(d,nd,prompt,*900)
      call get_real(f,d)
      if(f.ne.old) then
        dirty = .true.
        call logwrite2(label,d)
      endif
      return
  900 continue
      return 1
      end                       ! InputReal

      subroutine InputRotax(dirty,nd,*)
      logical   dirty
      integer   nd
      include 'axes.inc'
      include 'euhedral.inc'
c---- local vars
      character label*80
      integer   i

      call str_real_len_push(5)
      do i=1,3
        call str_string(xyz(i),'Rotax(#)',label)
        call InputReal(rotax(i),label,dirty,nd,*900)
      enddo
      call str_real_len_pop
      return
  900 continue
      return 1
      end                       ! InputRotax
 
      subroutine InputRotaxFace(dirty,nd,ier,*)
      logical   dirty
      integer   nd
      integer   ier
      include 'euhedral.inc'
c---- functions
      logical FacesOK
      logical FaceInRange
c---- local vars
      character prompt*80
      character d*80
      integer   i
c
      ier =  1
      if(facesOK()) then
        call str_integer(nfaces,'Face number 1-# [1]',prompt)
        call get_inp_default(d,nd,prompt,*900)
        call get_integer(i,d)
        if(FaceInRange(i)) then
          call getvec(facexyz,i,rotax)
          dirty = .true.
          ier = 0
        endif
      endif
      return
  900 continue
      return 1
      end                       ! InputRotaxFace
      
      subroutine InputRotaxHKL(dirty,nd,ier,*)
      logical dirty
      integer nd
      integer ier
      include 'euhedral.inc'
c---- functions
      logical rmatok
c---- local vars
      real    v(3)
      integer ihkl(3)
      if(RmatOK()) then
        call MatVec(mydmat,rotax,v)
        call scalevector(25,v,v)
        call copyvecFI(v,ihkl)
        call Inputhkl(ihkl,nd,*900)
        call MatVecFIF(myrmat,ihkl,rotax)
        dirty = .true.
        ier = 0
      else
        ier =  1
      endif
      return
  900 continue
      return 1
      end                       ! InputRotaxHKL

      subroutine InputTol(tol,dirty,nd,*)
      real      tol
      logical   dirty
      integer   nd
c
      call str_real_len_push(5)
      call InputReal(tol,'Tolerance',dirty,nd,*900)
      call str_real_len_pop
      return
  900 continue
      return 1
      end                       ! InputTol

      subroutine InputXtal(nd,*)
      integer  nd
      include 'axes.inc'
      include 'euhedral.inc'
      real  xtalmat(3,3)
      character d*80
      real      v1(3)
      integer   ier
      integer   i
      character prompt*80

      call unit_matrix(xtalmat)
      nfaces = 0
      ncorners = 0
      nedges = 0
      newworld = .true.

      do i=1,3
        prompt = 'xtal dimension x# [#]'
        call str_string(abc(i),prompt,prompt)
        call str_real(xtaldim(i),prompt,prompt)
        call get_inp_default(d,nd,prompt,*900)
        call get_real(xtaldim(i),d)
      enddo

      call str_write(6,
     +     '''mis-setting'' angles around LAB system axes X,Y,Z')
      call str_write(6,
     +     'to bring crystal edges along X,Y,Z, at Goniostat 0,0,0')

      call str_grad(xtalrot(3),
     +     'angle around Z to bring Xa in plane (+X,Z) [#]',prompt)
      call get_inp_default(d,nd,prompt,*900)
      call get_grad(xtalrot(3),d)

      call str_grad(xtalrot(2),
     +     'angle around Y to bring Xa along +X',prompt)
      call get_inp_default(d,nd,prompt,*900)
      call get_grad(xtalrot(2),d)

      call str_grad(xtalrot(1),
     +     'angle around X to bring Xc along +Z',prompt)
      call get_inp_default(d,nd,prompt,*900)
      call get_grad(xtalrot(1),d)

      do i=1,3
        xtalmat(i,i) = xtaldim(i)
      enddo
      call writemat(6,xtalmat,'before')
      call RotateMat(xtalmat,X_axis,-xtalrot(1),xtalmat) ! these 3 put crystal in actual
      call writemat(6,xtalmat,'after xrot')
      call RotateMat(xtalmat,Y_axis,-xtalrot(2),xtalmat) ! orientation at Goniostat 0,0,0
      call writemat(6,xtalmat,'after yrot')
      call RotateMat(xtalmat,Z_axis,-xtalrot(3),xtalmat) !
      call writemat(6,xtalmat,'after zrot')

      do i=1,3
        call col2vec(i,xtalmat,v1)
        call AddFace(ExtraOutput,AddFriedel,v1,xtaldim(i)/2,ier)
      enddo
      return
  900 continue
      return 1
      end                       ! InputXtal

      subroutine logend(message)
      character message*(*)
      include 'euhedral.inc'
      call str_string(subrname,'# finished',message)
      call logwrite(message)
      subrname = ' '
      return
      end                       ! logend

      subroutine loginterrupt(message)
      character message*(*)
      include 'euhedral.inc'
      call str_string(subrname,'# interrupted',message)
      call logwrite(message)
      subrname = ' '
      return
      end                       ! loginterrupt

      subroutine logstart(label)
      character label*(*)
      include 'euhedral.inc'
      subrname = label
      call logwrite2(label,'started')
      return
      end                       ! logstart

      subroutine logwrite2(s1,s2)
      character s1*(*)
      character s2*(*)
      include 'euhedral.inc'
      character message*256
      call str_2string(s1,s2,'# #',message)
      call logwrite(message)
      return
      end                       ! Logwrite2

      subroutine logwrite(s)
      character s*(*)
      include 'euhedral.inc'
      character datetime*20
      integer   lenstr
      write(log,100) datetime(),s(1:lenstr(s))
  100 format(a,' :',a)
      call flush(log)
      return
      end                       ! logwrite

      subroutine longtilt(nd,dirty,*)
      integer   nd
      logical   dirty
      include 'euhedral.inc'
c---- functions
      logical   str_equal
      logical   interrupted
c---- local vars
      integer   ier
      integer   noodrem
      logical   same
      integer   i
      character prompt*80
      character d*80
      character message*256
c
      call setuprefinement(ier)
      if(ier.ne.0) return
      if(menu) then
        call InputInteger(highindex,'highindex',dirty,nd,*900)
      endif
      call FaceHighAll
      if(menu) then
        call InputPosAngle(Angmax,'angmax',dirty,nd,*900)
      endif
        
      call str_string(distype,
     +     'Distance Refinement Type Parabolic/Simplex/None [#]',
     +     prompt)
      call get_inp_default(d,nd,prompt,*900)
      if(str_equal(d,'P').or.str_equal(d,'S').or.
     +     str_equal(d,'N')) then
        call upcase(d(1:1),distype)
      endif
      call GetFaceList(nd,*900)
      if(nlist.eq.0) return
      noodrem = 1
      initr = -1.0
   30 continue
      call str_integer(noodrem,'Start Facetilt Cycle #',message)
      call outflushlog(message)
      call savefaces
      do i=1,nlist
        call facetilt(facelist(i),ier)
        if(ier.ne.0) return
        if(interrupted()) then
          call saveresult(nd,*900)
          return
        endif
      enddo
      if(distype.ne.'N') then
        if(distype.eq.'P') then
          call RefineDistanceParabolic(message,ier)
        else if(distype.eq.'S') then
          call RefineDistanceSimplex(message,ier)
        endif
        call errorwarning(message,ier)
        if(ier.ne.0) return
        if(interrupted()) then
          call saveresult(nd,*900)
          return
        endif
      endif
      noodrem = noodrem + 1
      call CompareFaces(same)
      if(.not.same.and.noodrem.lt.100) goto 30
      if(.not.same) then
        call str_out('Refinement did not converge')
      endif
      call saveresult(nd,*900)
      return
  900 continue
      return 1
      end                       ! LongTilt

      subroutine ManualRot(dirty,nd,ier)
      logical   dirty
      integer   nd
      integer   ier
      include 'axes.inc'
      include 'const.inc'
      include 'euhedral.inc'
c---- functions
      logical   str_equal
c---- local vars
      real      rotmat(3,3)
      real      f
      character d*80
      character prompt*80
      real      rotang
      logical   ignoredirty
c
      rotang = 0.0
      call setuprefinement(ier)
      if(ier.ne.0) then
        ier = 0
        return
      endif
      call savefaces
      call InputRotax(dirty,nd,*900)
      call InputAngle(rotang,'Angle',ignoredirty,nd,*900)
      call GeneralRotMat(rotax,rotang,rotmat)
      call FaceOrient(.true.,rotmat)
      write(correctlabel,100) rotax,rotang*grad
  100 format(3f7.4,f7.2)
      call correct(f)
      prompt = 'Keep result [yes]'
      call get_inp_default(d,nd,prompt,*900)
      if(str_equal(d,'Y')) then
        call WriteAndRead(ier)
      else
        call restorefaces
      endif
      ier = 0
      return
  900 continue
      ier = 1
      return
      end                       ! ManualRot

      subroutine MotherFromDaughter(p,im,hb)
      integer p
      integer im
      integer hb
      include 'euhedral.inc'
      integer ih,ik,il

      call mylaue(npoint,dI(pDh,p),dI(pDk,p),dI(pDl,p),ih,ik,il)
      call searchmother(ih,ik,il,hb,im)
      return
      end                       ! MotherFromDaughter


      subroutine NewColour(n)
      integer n
      include 'euhedral.inc'
      call SetColourDirect(col(n))
      return
      end

      subroutine outflushlog(message)
      character message*(*)
      call str_out(message)
      call str_flush
      call logwrite(message)
      return
      end                       ! OutFlushLog

      subroutine OutputR2
      real r2WithFilter
      real R2NoFilter
      call str_real_len_push(5)
      call str_real_out(R2NoFilter(),'R2=#')
      call str_real_out(r2WithFilter(),'(with filter #)')
      call str_real_len_pop
      return
      end                       ! OutputR2

      subroutine outr(r,message)
      real      r
      character message*(*)
      include 'euhedral.inc'
      if(initr.lt.0.0) then
        initr = r
        bestr = r
        improved = .false.
        write(message,100) r
  100   format(f10.5)
      else
        improved = (r.lt.bestr)
        if(improved) then
          bestr = r
          write(message,110) r
  110     format(f10.5,' <--')
        else
          write(message,100) r
        endif
      endif
      return
      end                       ! outr

      subroutine PlotAxes(rm)
      real      rm(3,3)
      include 'axes.inc'
      include 'euhedral.inc'
      integer   i
      real      v(3)
c
      do i = 1,3
        call Col2Vec(i,rm,v)
        call PlotVectorLabel(v,abc(i),cAXESFG)
      enddo
      return
      end                       ! plotaxes

      subroutine PlotCircle
      include 'const.inc'
      include 'axes.inc'
      include 'euhedral.inc'
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='PlotCircle')
c---- local vars
      integer i
      real    f
      integer ier
      real    v(3)
      real    x
      real    y
      real    rotmat(3,3)
      real    phi
      real    dphi

      if(.not.graphics) return
      if(windowShape.eq.ShapeSmall) then
        windowShape = ShapeMakeBig
        newwindow = .true.
      endif
      if(newwindow) call startgraphics(ROUTINE)
      do i=1,2
        call ActivateWorld(i,ier)
        if(ier.ne.0) call exitfatal
        call ClearWindow
        call NewColour(cCIRCLE)
        if(i.eq.1) then
          call DrawString(-1.05,-1.05,'Upper')
        else
          call DrawString(-1.05,-1.05,'Lower')
        endif
        if(projectionmode.eq.1) then ! cone
          f = Pi - ang109 / 2
          call DrawArc(0.0,0.0, 1.0,-f,2*f)
          call ClearPoint
          call addpoint(cos(f),sin(f))
          call addpoint(0.0,0.0)
          call addpoint(cos(f),-sin(f))
          call DrawLines
        else
          call DrawCircle(0.0,0.0,1.0)
c          call DrawFatPoint(0.0,0.0,3)
          call CopyVec(x_axis,v)
          dphi = 10.0/grad
          call GeneralRotMat(y_axis,dphi,rotmat)
          do phi=dphi,PiOver2,dphi
            call MatVec(rotmat,v,v)
            call ProjectXY(v,x,y)
            if(x.gt.0.0) call DrawCircle(0.0,0.0,x)
          enddo
          dphi = 30.0/grad
          call GeneralRotmat(z_axis,dphi,rotmat)
          call CopyVec(x_axis,v)
          do phi=dphi,TwoPi,dphi
            call MatVec(rotmat,v,v)
            call Projectxy(v,x,y)
            call DrawLine(0.0,0.0,x,y)
          enddo
        endif
      enddo
      call WindowSync
      return
      end                       ! PlotCircle

      subroutine PlotCosines(message,ier)
      character message*(*)
      integer   ier
      include 'euhedral.inc'
c---- constants
      integer   debug
      parameter (debug=0)       ! 0=No 1=Accepted 2=accepted+rejected
      character*(*) ROUTINE
      parameter (ROUTINE='PlotCosines')
c---- functions
      logical   allowed
      real      R2WithFilter
      logical   interrupted
c---- local vars
      integer   hb,im
      real      v(3)
      integer   i
      integer   p
      real      hkl(3)
      real      c(3)
      integer   nreject

      if(nref.eq.0) then
        call str_out('No reflections')
        return
      endif

      vectorsync = .false.
      call plotcircle
      call plotfaces

      ngroups = 0
      naccept = 0
      nreject = 0
      do hb=1,hashbins
        if(interrupted()) then
          call str_string(ROUTINE,'# interrupted',message)
          ier = 1
          return
        endif
        do im = 1,nmother(hb)
          if(allowed(im,hb)) then
            naccept = naccept + mI(pMn,im,hb)
            ngroups = ngroups + 1
            if(debug.gt.0) then
              call str_integer_out(naccept,'accepted refls:#')
              if(nref.gt.0) then
                call str_real_out(100.0*naccept/float(nref),'= #%')
                call str_integer_out(ngroups,'(unique:#)')
              endif
              call str_flush
            endif
            p = mI(pMfirst,im,hb)
            do while(p.ne.0)
              if(debug.gt.1) then
                hkl(1) = dI(pDh,p)
                hkl(2) = dI(pDk,p)
                hkl(3) = dI(pDl,p)
                call str_integer_out(p,'Daughter')
                call str_vector_out(hkl,'hkl')
                call MatVec(myrmat,hkl,c)
                call normvec(c,c,ier)
                if(graphics) call PlotVector(c,cCVECTOR)
              endif
c----  input cosines              
              do i=1,3
                v(i) = dR(pDcosinesF+2*i-2,p) * axstar(i) ! still in a*b*c* system, as cosines
              enddo
              if(debug.gt.1) then
                call str_real_len_push(5)
                call str_vector_out(v,'cosi1')
                call str_real_len_pop
              endif
              call MatVec(mydmatTrans,v,v) !now in XYZ system
              if(debug.gt.1) then
                call str_real_len_push(5)
                call str_vector_out(v,'cosi2')
                call str_real_len_pop
              endif
              if(graphics) call PlotVector(v,cCOSI)
c---- output cosines
              do i=1,3
                v(i) = dR(pDcosinesF+2*i-1,p) * axstar(i) ! still in a*b*c* system, as cosines
              enddo
              if(debug.gt.1) then
                call str_real_len_push(5)
                call str_vector_out(v,'coso1')
                call str_real_len_pop
              endif
              call MatVec(mydmatTrans,v,v) !now in XYZ system
              if(debug.gt.1) then
                call str_real_len_push(5)
                call str_vector_out(v,'coso2')
                call str_real_len_pop
              endif
              if(graphics) call PlotVector(v,cCOSO)

              if(debug.gt.1) call str_flush

              p = dI(pDsister,p)
            enddo
          else
            if(debug.gt.2) then
              call str_out('reject')
              call str_integer_out(mI(pMn,im,hb),'Mn')
              call str_grad_out(mR(pMtheta,im,hb),'Mtheta')
              call str_real_out(mR(pMmean,im,hb),'Mmean')
              call str_real_out(mR(pMangular,im,hb),'Mangular')
              call str_flush
            endif
            nreject  = nreject + 1
          endif
        enddo
      enddo
      vectorsync = .true.

      if(graphics) call PlotAxes(myrmat)

      call str_integer_out(nreject,'rejected:#')
      call str_integer_out(naccept,'accepted:#')
      call str_real_len_push(1)
      call str_real_out(100.0*naccept/float(nref),'=#%')
      call str_real_len_pop
      call str_integer_out(ngroups,'(unique:#)')
      call str_real_len_push(5)
      call str_real_out(r2WithFilter(),'R2=#')
      call str_real_len_pop
      call str_flush

      ier = 0
      call str_string(ROUTINE,'# ok',message)
      return
      end                       ! PlotCosines

      subroutine PlotFaces
      include 'axes.inc'
      include 'const.inc'
      include 'euhedral.inc'
c---- constants
      logical   debug
      parameter (debug=.false.)
      character*(*) ROUTINE
      parameter (ROUTINE='PlotFaces')
c---- local vars
      integer   iface
      real      v(3)
      character string*3
      integer   icorner
      real      v1(3)
      real      v2(3)
      real      f
      real      w(3)
      real      x
      real      y
      integer   j
      integer   ier
      integer   i
      real      xmin,xmax
      real      ymin,ymax
      real      worldmin,worldmax
      real      w1(3)
      real      w2(3)
      logical   testok
      integer   iedge
      real      viewmat(3,3)
      integer   normerr
      character s2*2
      real      u(3,3)
      character s10*10
      save worldmin,worldmax
      data worldmin,worldmax /2*0.0/

      if(ncorners.eq.0) return
      if(.not.graphics) return
      if(newwindow) call startgraphics(ROUTINE)

      call MakeViewmat(viewvec,viewmat)
      if(vertical) then
        call RotateMat(viewmat,x_axis,-piover2,viewmat)
      endif
      if(debug) then
        call str_flush
        call writemat(6,viewmat,'viewmat')
      endif
      call RotateMat(viewmat,y_axis,viewrot,viewmat)
      ymin = 0.0
      ymax = 0.0
      xmin = 0.0
      xmax = 0.0
      do icorner=1,ncorners
        call getvec(cornerxyz,icorner,V)
        call Matvec(viewmat,v,v)
        x = V(1)
        y = V(2)
        if(icorner.eq.1) then
          xmin = x
          ymin = y
          xmax = x
          ymax = y
        else
          xmin = min(x,xmin)
          ymin = min(y,ymin)
          xmax = max(x,xmax)
          ymax = max(y,ymax)
        endif
      enddo

      if(debug) then
        call str_2real_out(xmin,xmax,'xmin=# xmax=#')
        call str_2real_out(ymin,ymax,'ymin=# ymax=#')
      endif
      if(newworld) then
        worldmin = min(1.5*xmin,1.5*ymin)
        worldmax = max(1.5*xmax,1.5*ymax)
      endif
      if(newworld.or.xmin.lt.0.9*Worldmin.or.xmax.gt.0.9*worldmax.or.
     +     ymin.lt.0.9*worldmin.or.ymax.gt.0.9*worldmax) then
        if(.not.newworld.and.debug) then
          call str_2real_out(xmin,xmax,'xmin=# xmax=#')
          call str_2real_out(ymin,ymax,'ymin=# ymax=#')
          call str_2real_out(worldmin,worldmax,
     +         'current world min=# max=#')
        endif
        worldmin = min(1.5*xmin,1.5*ymin,worldmin)
        worldmax = max(1.5*xmax,1.5*ymax,worldmax)
        if(.not.newworld.and.debug) then
          call str_2real_out(worldmin,worldmax,'new world min=# max=#')
        endif
        call CreateWorld(3,5,worldmin,worldmin,worldmax,worldmax,ier)
        newworld = .false.
      endif
      call ActivateWorld(3,ier)
      call ClearWindow

      if(doplotcorner) then
        call NewColour(cFG)
        do icorner=1,ncorners
          call getvec(cornerxyz,icorner,V)
          call MatVec(viewmat,v,v)
          call str_integer(icorner,'#',string)
          call DrawString(V(1),V(2),string)
        enddo
      endif

      call NewColour(cFACESFG)
      call drawline(worldmax-0.1,worldmax,worldmax,worldmax)
      f = 0.01*(worldmax-worldmin)

      do i=0,10
        x = worldmax-i*0.01
        call drawline(x,worldmax-f,x,worldmax)
      enddo
      do i=1,2
        if(i.eq.1) then
          call NewColour(cFACESBG)
          f = -1.0
        else
          call NewColour(cFACESFG)
          f = 1.0
        endif
        do iface=1,nfaces
          if(faceactive(iface)) then
            call getvec(facexyz,iface,V)
            call matvec(viewmat,v,v)
            if(f*v(3).ge.0.0) then
              j = 0
              do icorner=1,ncorners
                if(FaceCornerTable(iface,icorner)) then
                  call getVec(cornerxyz,icorner,v)
                  call matvec(viewmat,v,v)
                  if(j.eq.0) then
                    call copyvec(v,w)
                  else
                    call sumvec(v,w,w)
                  endif
                  j = j + 1
                endif
              enddo
              if(j.ne.0) then
                call facvec(1.0/j,w,w)
                call str_integer(iface,'#',string)
                call drawcenteredstring(W(1),W(2),string)
              endif
            endif
          endif                 ! faceactive(iface)
        enddo
        do iedge = 1,nedges
          call getvec(facexyz,edgeface(1,iedge),w1)
          call matvec(viewmat,w1,w1)
          call getvec(facexyz,edgeface(2,iedge),w2)
          call matvec(viewmat,w2,w2)
          testok = (w1(3).lt.0.0.and.w2(3).lt.0.0)
          if(i.eq.2) testok = .not. testok
          if(testok) then
            call getvec(cornerxyz,edgecorner(1,iedge),v1)
            call Matvec(viewmat,v1,v1)
            call getvec(cornerxyz,edgecorner(2,iedge),v2)
            call Matvec(viewmat,v2,v2)
            call Drawline(v1(1),v1(2),v2(1),v2(2))
          endif
        enddo                   ! iedge
      enddo
      if(doplotaxes) then
        do i=1,2
          if(i.eq.1) then
            call NewColour(cAXESBG)
            f = -1.0
          else
            call NewColour(cAXESFG)
            f = 1.0
          endif
          call activateworld(4,ier)
          do j=1,3
            call col2vec(j,myrmat,v)
            call normvec(v,v,normerr)
            call Matvec(viewmat,v,w)
            if(f*w(3).ge.-0.001) then
              call DrawLine(0.0,0.0,w(1),w(2))
              call str_string(abc(j),'#*',s2)
              call DrawCenteredString(1.5*w(1),1.5*w(2),s2)
            endif
          enddo
          call activateworld(5,ier)
          do j=1,3
            call row2vec(j,mydmat,v)
            call normvec(v,v,normerr)
            call Matvec(viewmat,v,w)
            if(f*w(3).ge.0.0) then
              call DrawLine(0.0,0.0,w(1),w(2))
              call DrawCenteredString(1.5*w(1),1.5*w(2),abc(j))
            endif
          enddo
        enddo                   ! i
      endif
      if(doplotlab) then
        call activateworld(6,ier)
        call unit_matrix(u)
        do i=1,2
          if(i.eq.1) then
            call NewColour(cAXESBG)
            f = -1.0
          else
            call NewColour(cAXESFG)
            f = 1.0
          endif
          do j=1,3
            call col2vec(j,u,v)
            call Matvec(viewmat,v,w)
            if(f*w(3).ge.-0.0001) then
              call DrawLine(0.0,0.0,w(1),w(2))
              call DrawCenteredString(1.5*w(1),1.5*w(2),xyz(j))
            endif
          enddo
        enddo
        call activateworld(7,ier)
        call NewColour(cAXESBG)
        call str_integer(nint(grad*viewrot),'#',s10)
        call DrawCornerString(0.01,0.99,'UL',s10)
      endif
      call MapPixmap(5,ier)
      call WindowSync
      return
      end                       ! plotfaces

      subroutine PlotHkl(ihkl,colcode,c)
      integer ihkl(3)
      integer colcode
      real    c(3)
      include 'euhedral.inc'
      integer ier
c
      call MatVecFIF(myrmat,ihkl,c) ! goto laboratory
      call normvec(c,c,ier)
      call PlotVector(c,colcode)
      return
      end                       ! plothkl

      subroutine PlotShape(message,ier)
      character message*(*)
      integer   ier
      include 'const.inc'
      include 'euhedral.inc'
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='PlotShape')
c---- local vars
      real      v(3)
      integer   unit
      integer   i
      character filename*80
      character command*80
      logical   old
      integer   ihkl(3)

      call create_overwrite(.true.,old)
      call build_file(compound,' ','.fli',filename,i)
      call create_file(filename,unit,message,ier)
      call create_overwrite(old,old)
      if(ier.ne.0) return

      call str_write(unit,'TITL euhedral')
      call facvec(grad,ang,v)
      write(unit,110) lambda,ax,v
  110 format('CELL',f8.5,3f10.4,3f10.3)
      do i=1,nfaces
        call getVecI(facehkl,i,ihkl)
        write(unit,120) ihkl,max(0.001,facefactor(i)*facedist(i))
  120   format('FACE ',3i4,f7.4)
      enddo
      write(unit,130)
  130 format('XTAL')
      close(unit)
      call str_string(filename,'platon # > /dev/null',command)
      call str_string_out(command,'Executing #')
      call str_flush
      call spawncc(command,ier)
      if(ier.eq.0) then
        call str_string(ROUTINE,'# ok',message)
      else
        call str_2string(ROUTINE,command,'#: can not execute #',message)
      endif
      return
      end                       ! PlotShape

      subroutine PlotVector(vector,colcode)
      real      vector(3)
      integer   colcode
      include 'euhedral.inc'
c---- local vars
      real      x
      real      y
      call PlotVectorPrep(vector,colcode,x,y)
      call DrawFatPoint(x,y,pointsize)
      if(VectorSync) call WindowSync
      return
      end                       ! plotvector

      subroutine PlotVectorCircle(vector,radius,colcode)
      real      vector(3)
      real      radius
      integer   colcode
      include 'euhedral.inc'
c---- local vars
      real      x
      real      y
      call PlotVectorPrep(vector,colcode,x,y)
      call FillCircle(x,y,radius)
      call WindowSync
      return
      end                       ! plotvectorCircle

      subroutine PlotVectorLabel(vector,label,colcode)
      real      vector(3)
      character label*(*)
      integer   colcode
      include 'euhedral.inc'
c---- local vars
      real      x
      real      y
      call PlotVectorPrep(vector,colcode,x,y)
      call DrawCenteredString(x,y,label)
      call WindowSync
      return
      end                       ! plotvectorLabel

      subroutine PlotVectorPrep(vector,colcode,x,y)
      real vector(3)
      integer colcode
      real    x
      real    y
      include 'euhedral.inc'
c---- local vars      
      integer ier
      integer j
      real    v(3)
c
      call NormVec(vector,v,ier)
      if(v(3).ge.0) then
        j = 1
      else
        j = 2
      endif
      call ActivateWorld(j,ier)
      if(ier.ne.0) call exitfatal
      call ProjectXY(v,x,y)
      if(colcode.gt.0) then
        call NewColour(colcode)
      else
        call NewColour(cFG)
      endif
      return
      end                       ! Plotvectorprep

      subroutine PrintCorners
      include 'euhedral.inc'
c---- functions
      real      absvec
c---- local vars
      integer   icorner
      integer   iface
      real      V(3)
      character line*80
      real      d
      integer   i
      integer   n
      call str_num_out(ncorners,'# corner^')
      call str_flush
      write(out,600)
  600 format(' nr   x        y        z      distance Faces')
      do icorner=1,ncorners
        call getvec(cornerxyz,icorner,V)
        d = absvec(v)
        write(line,610) icorner,V,d
  610   format(i3,3f9.5,f9.5)
        call str_out(line)
        do iface = 1,nfaces
          if(FaceCornerTable(iface,icorner)) then
            call str_integer_out(iface,'#')
          endif
        enddo
        call str_flush
      enddo

      n = 0
      do i=1,nfaces
        if(faceactive(i)) n = n + 1
      enddo

      call str_num_out(n,'# active face^')
      call str_flush
      write(out,620)
  620 format(' nr   x        y        z      distance surface Corners')
      do iface=1,nfaces
        if(faceactive(iface)) then
          call getvec(facexyz,iface,V)
          write(line,630) iface,V,facedist(iface),facesurface(iface)
  630     format(i3,3f9.5,f9.5,f8.5)
          call str_out(line)
          do icorner = 1,ncorners
            if(FaceCornerTable(iface,icorner)) then
              call str_integer_out(icorner,'#')
            endif
          enddo
          call str_flush
        endif
      enddo

      return
      end                       ! PrintCorners

      subroutine PrintDaughter(n)
      integer n
      include 'euhedral.inc'
      integer i
      write (out,600) n,(dI(i,n),i=pDh,pDl),dR(pDintensity,n),
     +     dR(pDsigma,n),(dR(i,n),i=pDcosinesF,pDcosinesL)
  600 format(i6,3i4,2f10.2,6f9.5)
      return
      end                       ! PrintDaughter

      subroutine PrintDaughterHeader
      include 'euhedral.inc'
      write(out,600)
  600 format('    nr   h   k   l intensity     sigma')
      return
      end                       ! PrintDaughterHeader

      subroutine PrintFaceAngles
      include 'const.inc'
      include 'euhedral.inc'
c---- functions
      real      angv2
c---- local vars
      integer   iface
      integer   jface
      integer   icorner
      integer   i
      real      V(3)
      real      W(3)
      real      faceangles(maxfaces)
c
      call str_write(out,'Face Angles')
      do iface=1,nfaces
        if(faceactive(iface)) then
          call getvec(facexyz,iface,V)
          do i=1,nfaces
            faceangles(i) = -1.0
          enddo
          do icorner = 1,ncorners
            if(FaceCornerTable(iface,icorner)) then
              do jface = 1,nfaces
                if(iface.ne.jface.and.faceactive(jface)) then
                  if(FaceCornerTable(jface,icorner)) then
                    if(faceangles(jface).lt.0.0) then
                      call getvec(facexyz,jface,W)
                      faceangles(jface) = Pi-angv2(V,W)
                    endif
                  endif
                endif
              enddo
            endif
          enddo
          call str_integer_out(iface,'Face #')
          do jface=1,nfaces
            if(faceangles(jface).gt.0) then
              call str_integer_out(jface,'#:')
              call str_grad_out(faceangles(jface),'#')
            endif
          enddo
          call str_flush
        endif ! faceactive(iface)
      enddo
      return
      end                       ! PrintFaceAngles

      subroutine PrintHelp
      include 'euhedral.inc'
      logical   old

      call str_flush
      call str_write_compress(.false.,old)
      call str_write(out,'Reading data:')
      call str_indent(2)
      call str_write(out,'readins      insfile')
      call str_write(out,'readhkl      hklfile')
      call str_write(out,'pg           set point group')
      call str_write(out,'filter       filter parameters and plot')
      call str_indent(0)
      call str_write(out,'Build crystal description from scratch')
      call str_indent(2)
      call str_write(out,'addface      add face (x y z d)')
      call str_write(out,'addfacehkl   add face (h k l d)')
      call str_write(out,'addfaceperp  face1 face2 dist')
      call str_write(out,'addfacesum   face numbers')
      call str_write(out,'buildcube    a simple cube')
      call str_integer_write(out,maxSphere,
     +     'buildsphere  select one of # dodecahedron models')
      call str_write(out,'bestsphere   search best dodecahedron')
      call str_write(out,'deleteface   delete faces')
      call str_write(out,'face h k l d set face parameters')
      call str_write(out,'initdistance set initial distance')
      call str_write(out,'replaceface  replace faces')
      call str_write(out,'xtal         evalccd crystal description')
      call str_indent(0)
      call str_write(out,'Filter Parameters')
      call str_indent(2)
      call str_write(out,'angdistmax   maximum angular distribution')
      call str_write(out,'intmin       minimum intensity')
      call str_write(out,'nhklmin      minimum redundancy')
      call str_write(out,'thetamin     minimum theta')
      call str_write(out,'mode         set projection mode')
      call str_indent(0)
      call str_write(out,'Refinement options')
      call str_indent(2)
      call str_write(out,'angmin       minimum angle between rotaxes')
      call str_write(out,'angmax       set maximum tilt angle')
      call str_write(out,'centric      centric shape on/off')
      call str_write(out,'correction   ABST/ABSG')
      call str_write(out,'extreme      set initial refinement shift')
      call str_write(out,'highindex    set high index')
      call str_write(out,'initangle    set  CrystrotSimplex angle')
      call str_write(out,'phi          set phi start,stop and delta')
      call str_write(out,'tol          set refinement tolerance')
      call str_write(out,
     +     'weight       set weights for R2 calculation')
      call str_indent(0)
      call str_write(out,'Refinement commands')
      call str_indent(2)
      call str_write(out,'amoeba       amoeba distance ')
      call str_write(out,'crystrot     crystal rotations')
      call str_write(out,'crystrot1    single rotation')
      call str_write(out,'crystrotface single rotation')
      call str_write(out,'crystrothkl  single rotation')
      call str_write(out,'crystrotSimplex crystal rotations')
      call str_write(out,'distance     refine face distances')
      call str_write(out,'manualrot    single orientation')
      call str_write(out,'tilt         refine face tilt')
      call str_write(out,'volume       refine volume')
      call str_indent(0)
      call str_write(out,'Graphics')
      call str_indent(2)
      call str_write(out,
     +     'colour       axes/bg/circle/cosi/coso/faces/fg')
      call str_write(out,'croty        continuous rotation')
      call str_write(out,'movie        do/do not rotate result')
      call str_write(out,'onerot       rotate picture')
      call str_write(out,'partialrot   rotate picture')
      call str_write(out,'plotaxes     do/donot plot axes')
      call str_write(out,'plotcorner   do/donot plot corners')
      call str_write(out,'plotlab      do/donot plot cartesian axes')
      call str_write(out,'pointsize    point size in pixels')
      call str_write(out,'posx         window position')
      call str_write(out,'posy         window position')
      call str_write(out,'ps           create screen snapshot')
      call str_write(out,'size         window size')
      call str_write(out,'vertical     put view direction vertical')
      call str_write(out,'viewdir      view along hkl direction')
      call str_write(out,'viewpartial  set partialrot angle')
      call str_write(out,'viewperp     view perpendicular to face')
      call str_write(out,'viewrot      rotate view vector')
      call str_write(out,'viewspeed    rotation speed')
      call str_write(out,'viewxyz      view along cartesian vector')
      call str_indent(0)
      call str_write(out,'Misc')
      call str_indent(2)
      call str_write(out,'edit         edit insfile')
      call str_write(out,'exit         end program')
      call str_write(out,'hkl          single reflection information')
      call str_write(out,'name         change compound name')
      call str_write(out,'plot         use platon to display shape')
      call str_write(out,'reduce       reduce face indices')
      call str_write(out,'save         save settings in euhedral.sav')
      call str_write(out,'show         cell/reflection status')
      call str_write(out,'status       program status')
      call str_write(out,'test         consistency test')
      call str_write(out,'workfile     set workfile name (subfile)')
      call str_write(out,'writehkl     write (filtered) hkl file')
      call str_indent(0)
      call str_write_compress(old,old)
      return
      end                       ! PrintHelp

      subroutine PrintHKL(ihkl)
      integer ihkl(3)
      include 'euhedral.inc'
      integer nh2
      integer nk2
      integer nl2
      integer hb
      integer im
      integer j
c
      call mylaue(npoint,ihkl(1),ihkl(2),ihkl(3),nh2,nk2,nl2)
      call str_iarray_out(3,ihkl,'(#,#,#)')
      call str_3integer_out(nh2,nk2,nl2,'->(#,#,#)')
      call searchmother(nh2,nk2,nl2,hb,im)
      if(im.eq.0) then
        call str_out('Not Found')
      else
        call str_2integer_out(hb,im,'hb=# im=#')
        call str_num_out(mI(pMn,im,hb),'# daughter^')
        call str_real_len_push(2)
        call str_grad_out(mR(pMtheta,im,hb),'theta=#')
        call str_real_len(4)
        call str_real_out(mR(pMangular,im,hb),'angdist=#')
        call str_real_len(1)
        call str_real_out(mR(pMsum,im,hb),'Intensity: sum=#')
        call str_real_out(mR(pMmean,im,hb),'mean=#')
        call str_real_out(mR(pMmin,im,hb),'minimum=#')
        call str_real_out(mR(pMmax,im,hb),'maximum=#')
        call str_real_len_pop
        call str_flush
        call PrintDaughterHeader
        j = mI(pMfirst,im,hb)
        do while(j.ne.0)
          call PrintDaughter(j)
          j = dI(pDsister,j)
        enddo
      endif
      return
      end                       ! PrintHKL

      subroutine PrintMatrices
      include 'const.inc'
      include 'euhedral.inc'
      integer i
      real    m1(3,3)
      real    rax(3)
      real    rang
      call str_flush
      do i = 1,nmatrices
        call getmat(matrices,i,m1)
        call GetRotAng(m1,rax,rang)
        write(out,610) i,rax,grad*rang
  610   format(i4,3f9.5,f9.4)
      enddo
      return
      end                       ! PrintMatrices

      subroutine PrintOrient
      include 'const.inc'
      include 'euhedral.inc'
      integer i
      integer j
      real    m1(3,3)
      real    m2(3,3)
      real    rax(3)
      real    rang
      real    rotmat(3,3)
      call str_flush
      do i = 1,nmatrices
        call getmat(matrices,i,m1)
        call GetRotAng(m1,rax,rang)
        write(out,610) i,rax,grad*rang
  610   format(i4,3f9.5,f9.4)
      enddo

      do i = 1,nmatrices
        call getmat(matrices,i,m1)
        call MatMat(m1,myrmat,m1)
        do j=i+1,nmatrices
          call getmat(matrices,j,m2)
          call MatMat(m2,myrmat,m2)
          call GetRotMat(m1,m2,rotmat)
          call GetRotAng(rotmat,rax,rang)
          write(out,620) i,j,grad*rang
  620     format(2i4,f9.4)
        enddo
      enddo
      return
      end                       ! PrintOrient

      subroutine PrintRatio
      include 'euhedral.inc'
      integer hb
      integer im
      integer firstmotherhb
      real    ratio
      real    ratiomin
      real    ratiomax

      firstmotherhb = 0
      do hb=1,hashbins
        if(nmother(hb).gt.0) then
          firstmotherhb = hb
          goto 10
        endif
      enddo
   10 continue

      if(firstmotherhb.gt.0) then
        ratio = mR(pMmax,1,firstmotherhb)/mR(pMmin,1,firstmotherhb)
        ratiomin = ratio
        ratiomax = ratio
        do hb = 1,hashbins
          do im = 1,nmother(hb)
            ratio = mR(pMmax,im,hb)/mR(pMmin,im,hb)
            ratiomin = min(ratiomin,ratio)
            ratiomax = max(ratiomax,ratio)
          enddo
        enddo
        call str_2real_out(ratiomin,ratiomax,'Ratio from # to #')
      endif
      call str_flush
      return
      end                       ! PrintRatio

      subroutine ProjectXY(v,x,y)
      real      v(3)
      real      x
      real      y
      include 'euhedral.inc'
c---- functions
      real atan2AJMD
c---- local vars
      real      chi
      real      r
      real      phi
      real      w
      real      f

      if(projectionmode.eq.1) then
        w = sqrt(v(1)**2 + v(2)**2)
        chi = ABS(Atan2(v(3), w))
        r = COS(chi) * coscona / COS(chi - cona)
        phi = phifact * Atan2AJMD(v(2), v(1))
        x = r * COS(phi)
        y = r * SIN(phi)
      else if(projectionmode.eq.2) then ! Wulff projection
        f = 1 + ABS(v(3))
        x = v(1) / f
        y = v(2) / f
      else if(projectionmode.eq.3) then ! ajmd projection
        f = sqrt(1 + ABS(v(3)))
        x = v(1) / f
        y = v(2) / f
      else                      ! perpendicular projection
        x = v(1)
        y = v(2)
      endif
      return
      end                       ! projectXY

      subroutine RandomRot(nd,dirty,*)
      integer   nd
      logical   dirty
      include 'euhedral.inc'
c---- functions
      logical   str_equal
c---- local vars
      integer   ier
      character prompt*80
      character d*80
      integer   j
      character message*256

      call setuprefinement(ier)
      if(ier.ne.0) return
      prompt = 'nr of cycles (1-#) [100]'
      call str_integer(maxrotaxes,prompt,prompt)
      call get_inp_default(d,nd,prompt,*900)
      call get_integer(j,d)
      call BuildFibonacciOrient(j)
      if(menu) then
        call InputPhi(dirty,nd,*900)
        call InputPosAngle(Angmin,'angmin',dirty,nd,*900)
      endif
      call BuildOrientationList
        
      if(menu) then
        call get_inp_default(d,nd,'Do Rot refinement? [Yes]',*900)
      else
        d = 'Y'
      endif
      if(str_equal(d,'Y')) then
        call RefineOrient(message,ier)
        call errorwarning(message,ier)
        if(ier.eq.0) call saveresult(nd,*900)
      endif
      return
  900 continue
      return 1
      end                       ! RandomRot

      subroutine ReadHkl(hklfile,message,ier)
      character hklfile*(*)
      character message*(*)
      integer   ier
      include 'euhedral.inc'
c---- functions
      real      R2NoFilter
c---- local vars
      character filename*80
      integer   lf
      integer   unit
      integer   n
      integer   k
      real      weakfactor
      integer   nignored

      weakfactor = 3.0
      call build_file(hklfile,' ','.hkl',filename,lf)
      call open_file(filename,unit,message,ier)
      if(ier.ne.0) return

      n = 0
      nignored = 0
      do while(ier.eq.0)
        if(n.ge.MaxRefl) then
          call TooManyFatal(n,maxrefl,'reflections')
          goto 10
        endif
        n = n + 1
        read(unit,100,iostat=ier) (dI(k,n),k=pDh,pDl),
     +       dR(pDintensity,n),dR(pDsigma,n),dI(pDbatch,n),
     +       (dR(k,n),k=pDcosinesF,pDcosinesL)
  100   format(3i4,2f8.0,i4,6f8.5)
        if(dI(pDh,n).eq.0.and.dI(pDk,n).eq.0
     +       .and.dI(pDl,n).eq.0) goto 10
        if(dR(pDintensity,n).lt.weakfactor*dR(pDsigma,n)) then
c----     note that negative intensities are also ignored here
          n = n - 1
          nignored = nignored + 1
        endif
        dI(pDactive,n) = 1
      enddo
      if(ier.gt.0) call str_integer_out(ier,'ier=#')
   10 continue
      close(unit)
      nref = n-1
      if(nref.eq.0) then
        call str_string(filename,'no reflections in #',message)
        ier = 1
        return
      endif
      call BuildMother
      ier = 0
      message =
     +     '#:# reflection^ (# accepted, # ignored, # unique R2=#)'
      call str_string(filename,message,message)
      call str_num(nref+nignored,message,message)
      call str_integer(nref,message,message)
      call str_integer(nignored,message,message)
      call str_num(Nunique,message,message)
      call str_real_len_push(5)
      call str_real(R2NoFilter(),message,message)
      call str_real_len_pop
      return
      end                       ! readhkl

      subroutine ReadIns(message,ier)
      character message*(*)
      integer   ier
      include 'const.inc'
      include 'euhedral.inc'
c---- functions
      logical samevec
c---- local vars
      integer   li
      integer   unit
      character line*132
      character field*80
      integer   i
      character string*80
      integer   ihkl(3)
      real      f
      character insfile*80
c      myrmatavailable = .false.
      facesavailable = .false.
      call build_file(compound,' ','.ins',insfile,li)
      call open_file(insfile,unit,message,ier)
      if(ier.ne.0) return
      call filescan(compound,'NAME',compound,li)
      call str_out(message)
      call logwrite2('readins',insfile)
      nfaces = 0
      ncorners = 0
      nedges = 0
      mu = -1.0
      read(unit,100,iostat=ier) line
  100 format(a)
      do while(ier.eq.0)
        call upcase(line,line)
        if(line(1:4).eq.'CELL') then
          call str_header(line,field,line) ! skip CELL
          call str_header(line,field,line)
          call get_real(lambda,field)
          do i=1,3
            call str_header(line,field,line)
            call get_real(ax(i),field)
          enddo
          do i=1,3
            call str_header(line,field,line)
            call get_grad(ang(i),field)
          enddo
          if(cellcheck) then
            if(.not.samevec(ax,cellaxes,0.001).or.
     +           .not.SameVec(ang,cellangles,0.01)) then
              call str_vector_out(cellaxes,'stored:')
              call str_gradvector_out(cellangles,' ')
              call str_flush
              call str_vector_out(ax,'read:')
              call str_gradvector_out(ang,' ')
              call str_flush
              message = 'cell parameters differ'
              ier = 1
              return
            endif
          else
            call copyvec(ax,cellaxes)
            call copyvec(ang,cellangles)
            cellcheck = .true.
          endif
        else if(line(1:4).eq.'FACE') then
          call str_header(line,field,line)
          do i=1,3
            call str_header(line,field,line)
            call get_integer(ihkl(i),field)
          enddo
          call str_header(line,field,line)
          call get_real(f,field)
          call addfaceHKL(ihkl,f,ier)
        else if(line(1:4).eq.'ABST'.or.line(1:4).eq.'ABSG') then
          correctionmethod = line(1:4)
          call str_header(line,field,line)
          call str_header(line,field,line)
          call get_real(mu,field)
          call str_real_out(mu,'Mu=#')
        endif
        read(unit,100,iostat=ier) line
      enddo
      close(unit)
      ier = 0

      if(.not.myrmatavailable) then
        call cell2d(ier)        ! new mydmat and myrmat
        if(ier.ne.0) return
        call str_flush
        call writecell(out,mydmat,' ')
        call readrmatfile(message,ier)
        if(ier.ne.0) then
          if(pointgroup.eq.'?') then
            call getpg(ax,ang,0.1,0.1/grad,pointgroup,string)
          endif
        endif
        call SetupCell
        myrmatavailable = .true.
      endif

      if(mu.lt.0.0) then
        if(nfaces.gt.0) then
          message = 'No mu'
        else
          message = 'no faces and no mu'
        endif
        ier = 1
      else if(nfaces.eq.0) then
        message = 'no faces'
        ier = 0
      else if(nfaces.lt.4) then
        message = 'not enough faces'
        ier = 0
      else
        call FaceTest(NoExtraOutput,DoDeactivate,message,ier)
        call showfaces(' ')
        Facesavailable = (ier.eq.0)
        ier = 0
      endif
      return
      end                       ! ReadIns

      subroutine readrmatfile(message,ier)
      integer   ier
      include 'euhedral.inc'
      include 'rmat.inc'
c---- functions
      integer   length
c---- local vars
      logical   exist
      character message*256
      character filename*80
      integer   lf
c
      ier = 0
      if(length(rmatfile).eq.0) then
        call build_file(compound,' ','.rmat',filename,lf)
      else
        call build_file(rmatfile,' ','.rmat',filename,lf)
      endif
      inquire(file=filename,exist=exist)
      if(exist) then
        call ReadRmatList(filename,1,message,ier)
        if(ier.ne.0) then
          return
        endif
        call str_out(message)
        call str_flush
        call writecell(out,dmat,filename)
        if(pointgroup.eq.'?') pointgroup = pg
c---- use it
        call copymat(dmat,mydmat)
        call copymat(rmat,myrmat)
        rmatfile = filename
      else
        call str_string(filename,'No such file #',message)
        ier = 1
      endif
      return
      end                       ! readrmatfile

      subroutine RefineDistanceParabolic(message,ier)
      character message*(*)
      integer   ier
      include 'const.inc'
      include 'euhedral.inc'
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='RefineDistanceParabolic')
c---- functions
      real      ParabolicDistanceFunk
      external  ParabolicDistanceFunk
      real      brent
      external  brent
      logical   interrupted
c---- local vars
      real      rmin
      real      xmin
      integer   i
      integer   j
      logical   facedone(MaxFaces)
      integer   ihkl(3)
      integer   jhkl(3)
      integer   removedfaces
      call logstart(ROUTINE)

   10 continue
      initr = -1.0
      do i=1,nfaces
        facedone(i) = .false.
      enddo

      do i=1,nfaces
        if(.not.facedone(i)) then
          refineface1 = i
          refineface2 = 0
          call getVecI(facehkl,i,ihkl)
          if(centricshape) then
            do j=i+1,nfaces
              call GetVecI(facehkl,j,jhkl)
              if(ihkl(1).eq.-jhkl(1).and.
     +             ihkl(2).eq.-jhkl(2).and.
     +             ihkl(3).eq.-jhkl(3)) then
                facedone(j) = .true.
                refineface2 = j
              endif
            enddo
          endif
          facedone(i) = .true.
          if(refineface2.ne.0) then
            call str_2integer(refineface1,refineface2,
     +           'Refining distance face # and #',message)
          else
            call str_integer(refineface1,'Refining distance face #',
     +           message)
          endif
          call outflushlog(message)
          initr = -1.0
          write(out,100)
  100     format('FaceFac      R2')
          rmin = BRENT(1.0-extreme,1.0,1.0+extreme,
     +         ParabolicDistanceFunk,tol,xmin)
          if(interrupted()) then
            call loginterrupt(message)
            ier = 1
            return
          endif
          if(rmin.lt.initr-improve) then
            facefactor(refineface1) = xmin
            if(refineface2.gt.0) then
              facefactor(refineface2) = xmin
            endif
            call facetest(NoExtraOutput,DoDeactivate,message,ier)
            call errorwarning(message,ier)
            if(ier.ne.0) return
            removedfaces = 0
            do j=1,nfaces
              if(.not.faceactive(j)) removedfaces = removedfaces + 1
            enddo
            call WriteAndRead(ier)
            if(ier.ne.0) return
            if(removedfaces.gt.0) then
              call outflushlog('Restart with less faces')
              call readins(message,ier)
              if(ier.ne.0) return
              call str_out(message)
              if(movie) then
                call ViewPartialRotation
              else
                call PlotFaces
              endif
              goto 10           ! restart all over
            endif
            initr = rmin
            message = 'Rmin=# with face factor=#'
            call str_real_len_push(5)
            call str_real(rmin,message,message)
            call str_real(xmin,message,message)
            call str_real_len_pop
          else
            facefactor(refineface1) = 1.0
            if(refineface2.gt.0) then
              facefactor(refineface2) = 1.0
            endif
            message = 'Distance refinement did not improve'
            if(rmin.lt.initr) then
              call str_string(message,'# enough',message)
            endif
          endif
          call outflushlog(message)
          if(movie) then
            call ViewPartialRotation
          else
            call PlotFaces
          endif
        endif
      enddo
      ier = 0
      call logend(message)
      return
      end                       ! RefineDistanceParabolic

      subroutine RefineDistanceSimplex(message,ier)
      character message*(*)
      integer   ier
      include 'euhedral.inc'
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='RefineDistanceSimplex')
c---- local vars
      real      parnew(maxparam)
      real      parold(maxparam)
      logical   free(maxparam)
      integer   i
      real      rmin

      call logstart(ROUTINE)

   10 continue
      initr = -1.0
      do i=1,nfaces
        free(i) = .true.
        parold(i) = facefactor(i)
      enddo

      call SimplexDistance(free,parold,parnew,rmin)
      call str_real_out(rmin,'rmin=#')
      if(rmin.lt.initr) then
        do i=1,nfaces
          facefactor(i) = parnew(i)
        enddo
        call correct(rmin)

        call TestAndSave(ier)
        if(ier.ne.0) return
        initr = rmin
        message = 'Rmin=#'
        call str_real_len_push(5)
        call str_real(rmin,message,message)
        call str_real_len_pop
      else
        do i=1,nfaces
          facefactor(i) = parold(i)
        enddo
        message = 'Distance refinement did not improve'
      endif
      call outflushlog(message)
      if(movie) then
        call ViewPartialRotation
      else
        call PlotFaces
      endif
      ier = 0
      call logend(message)
      return
      end                       ! RefineDistanceSimplex
     
      subroutine RefineOrient(message,ier)
      character message*(*)
      integer   ier
      include 'axes.inc'
      include 'const.inc'
      include 'euhedral.inc'
c---- constants
      logical   debug
      parameter (debug=.false.)
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='RefineOrient')
c---- functions
      logical   interrupted
      logical   samevec
c---- local vars
      integer   imatrix
      real      r
      real      rax(3)
      real      oldrax(3)
c      integer   nc
      real      startr
      real      bestrotmat(3,3)
      integer   i
      real      rotmat(3,3)
      real      rang

      call logstart(ROUTINE)
      message = '# orientation^.'
      call str_num(nmatrices,message,message)
      call outflushlog(message)
c
c---- save the faces
c
      call savefaces
      initr = -1.0
      do i=1,3
        oldrax(i) = 0.0
      enddo
c      nc = 0
      startr = -1.0 ! actually not needed

      write(out,600)
  600 format('  nr   x        y        z         Phi     R2')
      do imatrix=1,nmatrices
        call getmat(matrices,imatrix,rotmat)
        call GetRotAng(rotmat,rax,rang)
        if(.not.SameVec(rax,oldrax,zeroXYZ)) then
c          call plotcircle
          call CopyVec(rax,oldrax)
c          nc = 0
          write(correctlabel,100) nmatrices-imatrix+1,rax,rang*grad
  100     format(i4,3f9.5,f7.1)
        else
          write(correctlabel,110) nmatrices-imatrix+1,rang*grad
  110     format(i4,27x,f7.1)
        endif
c        nc = mod(nc,maxcolour) + 1
        call FaceOrient(.false.,rotmat)
        call correct(r)
        call restorefaces
        if(improved) then
          call CopyMat(rotmat,BestRotMat)
        endif
        if(imatrix.eq.1) startr = r
        if(interrupted()) goto 10
      enddo                     ! irotax
   10 continue
      if(bestr.lt.startr) then
        call GetRotAng(bestrotmat,rax,rang)
        message = 'Best R2=# for rotax (#,#,#) angle #'
        call str_real_len_push(5)
        call str_real(bestr,message,message)
        call str_vector(rax,message,message)
        call str_real_len_pop
        call str_grad(rang,message,message)
        call outflushlog(message)
        call FaceOrient(.true.,bestrotmat)
      else
        call outflushlog('No improvement')
      endif
      if(interrupted()) then
        call loginterrupt(message)
      else
        call logend(message)
      endif
      ier = 0
      return
      end                       ! RefineOrient

      subroutine RefineOrientationSimplex(message,ier)
      character message*(*)
      integer   ier
      include 'euhedral.inc'
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='RefineOrientationSimplex')
c---- local vars
      real      parnew(3)
      real      parold(3)
      integer   i
      real      rmin
      real      rotmat(3,3)
      call logstart(ROUTINE)

   10 continue
      initr = -1.0
      do i=1,3
        parold(i) = 0.0
      enddo
      call savefaces
      call SimplexOrientation(parold,parnew,rmin)
      call str_real_out(rmin,'rmin=#')
      if(rmin.lt.initr) then
        call SimplexRotmat(parnew,rotmat)
        call FaceOrient(.false.,rotmat)
        call correct(rmin)
        call TestAndSave(ier)
        if(ier.ne.0) return
        initr = rmin
        message = 'Rmin=#'
        call str_real_len_push(5)
        call str_real(rmin,message,message)
        call str_real_len_pop
      else
        call restorefaces
        message = 'Orientation refinement did not improve'
      endif
      call outflushlog(message)
      if(movie) then
        call ViewPartialRotation
      else
        call PlotFaces
      endif
      ier = 0
      call logend(message)
      return
      end                       ! RefineOrientationSimplex

      subroutine RefineVolume(message,ier)
      character message*(*)
      integer   ier
      include 'const.inc'
      include 'euhedral.inc'
      real      VolumeFunk
      external  VolumeFunk
      real      brent
      external  brent
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='RefineVolume')
c---- functions
      logical   interrupted
c---- local vars
      real      rmin
      real      xmin
      integer   i

      call logstart(ROUTINE)
c
c--- volume refinement
c
      call str_flush
      initr = -1.0
      write(out,100)
  100 format(' VolFac      R2')

      rmin = BRENT(1.0-extreme,1.0,1.0+extreme,VolumeFunk,tol,xmin)
      if(interrupted()) then
        call loginterrupt(message)
        ier = 1
        return
      endif
      if(rmin.lt.initr-improve) then
        do i=1,nfaces
          facefactor(i) = xmin
        enddo
        call WriteInsFile(compound,message,ier)
        if(ier.ne.0) return
        call str_out(message)
        initr = rmin
        call str_flush
        call str_real_len_push(5)
        call str_real_out(rmin,'Rmin=#')
        call str_real_out(xmin,'with volume factor=#')
        call str_real_len_pop
      else
        do i=1,nfaces
          facefactor(i) = 1.0
        enddo
        call str_out('Volume refinement did not improve')
        if(rmin.lt.initr) call str_out('enough')
      endif
      call str_flush
c
      if(movie) then
        call ViewPartialRotation
      else
        call PlotFaces
      endif
      call logend(message)
      ier = 0
      return
      end                       ! RefineVolume

      subroutine replaceface(nd,ier)
      integer   nd
      integer   ier
      include 'euhedral.inc'
c---- functions
      logical   FaceInRange
c---- local vars
      real      w(3)
      character d*80
      integer   i
      real      f
      integer   np
      real      v(3)
      integer   fn(maxfaces)
      integer   normerr
      character message*256
c
      if(nd.gt.0) then
        call build_vector(0.0,0.0,0.0,w)
        f = 0.0
        np = 0
        do while(nd.gt.0)
          call get_inp(d,nd,' ',*900)
          call get_integer(i,d)
          if(FaceInRange(i)) then
            call getvec(facexyz,i,v)
            call sumvec(v,w,w)
            f = f + facedist(i)
            np = np + 1
            fn(np) = i
          endif
        enddo
        if(np.ge.2) then
          call normvec(w,w,normerr)
          f = f/np
c            call InputReal(f,'distance',dirty,nd,*900)
          call AddFace(ExtraOutput,NoFriedel,w,f,ier)
          if(ier.eq.0) then
            do i=1,np
              faceactive(fn(i)) = .false.
            enddo
            call FaceTest(ExtraOutput,DoNotDeactivate,message,ier)
            call errorwarning(message,ier)
            if(ier.eq.0) then
              call WriteAndRead(ier)
            else
              do i=1,np
                faceactive(fn(i)) = .true.
              enddo
              call str_out('Faces not removed')
              call FaceTest(NoExtraOutput,DoNotDeactivate,message,ier)
            endif
          else
            call str_out('New face not added')
          endif
        endif
      endif
      ier = 0
      return
  900 continue
      ier = 1
      return
      end                       ! ReplaceFace

      subroutine restorefaces
      include 'euhedral.inc'
      integer i
      integer ihkl(3)
      do i=1,nfaces
        call getVecI(facesave,i,ihkl)
        call SaveVecI(ihkl,facehkl,i)
      enddo
      return
      end                       ! RestoreFaces

      subroutine savefaces
      include 'euhedral.inc'
      integer i
      integer ihkl(3)
      do i=1,nfaces
        call getVecI(facehkl,i,ihkl)
        call SaveVecI(ihkl,facesave,i)
      enddo
      return
      end                       ! SaveFaces
 
      subroutine savefile(tofile,message,ier)
      logical   tofile
      character message*(*)
      integer   ier
      include 'euhedral.inc'
      character filename*80
      integer   unit
      logical   old
      character line*80
      integer   i
      if(tofile) then
        filename = 'euhedral.sav'
        call create_overwrite(.true.,old)
        call create_file(filename,unit,message,ier)
        if(ier.ne.0) return
        call str_unit_push(unit)
      else
        unit = 6
        if(.not.menu) call str_wrap(1)
      endif
      if(graphics) then
        do i=1,cMAX
          call str_2string_write(unit,colournames(i),colours(i),
     +         'colour # #')
        enddo
c        call str_2string_write(unit,colouraxesfg,colouraxesbg,
c     +       'colour axes')
c        call str_string_write(unit,colourbg,'colour bg')
c        call str_string_write(unit,colourcircle,'colour circle')
c        call str_string_write(unit,colourcosi,'colour cosi')
c        call str_string_write(unit,colourcoso,'colour coso')
c        call str_2string_write(unit,colourfacesfg,colourfacesbg,
c     +       'colour faces')
c        call str_string_write(unit,colourfg,'colour fg')

        call str_OnOff_out(movie,'movie')
        call str_OnOff_out(vertical,'vertical')
        call str_OnOff_out(doplotaxes,'plotaxes')
        call str_OnOff_out(doplotlab,'plotlab')
        call str_OnOff_out(doplotcorner,'plotcorner')
        call str_flush
        call str_integer_out(pointsize,'pointsize')
        call str_integer_out(posx,'posx')
        call str_integer_out(posy,'posy')
        call str_integer_out(size,'size')
        call str_flush
        call str_vector_out(viewvec,'viewxyz')
        call str_grad_out(viewrot,'viewrot')
        call str_grad_out(viewpartial,'viewpartial')
        call str_grad_out(viewspeed,'viewspeed')
        call str_flush
      endif
      call str_grad_out(angmin,'angmin')
      call str_grad_out(angmax,'angmax')
      call str_flush
      if(cellcheck) then
        call str_real_len_push(5)
        call str_vector(cellaxes,'cell',line)
        call str_gradvector(cellangles,line,line)
        call str_out(line)
        call str_flush
        call str_real_len_pop
      endif
      call str_onoff_out(centricshape,'centric')
      call str_string_out(correctionmethod,'correction #')
      call str_real_out(extreme,'extreme')
      call str_integer_out(highindex,'highindex')
      call str_flush
      call str_grad_out(OrientSimplexAngle,'initangle #')
      call str_real_out(initdistance,'initdistance')
      call str_flush
      if(projectionmode.gt.0) then
        call str_string_out(projmodes(projectionmode),'mode')
      endif
      call str_grad_out(thetamin,'thetamin')
      call str_real_out(intmin,'intmin')
      call str_integer_out(nhklmin,'nhklmin')
      call str_real_out(angdistmax,'angdistmax')
      call str_flush
      if(pointgroup.ne.'?') call str_string_write(unit,pointgroup,'pg')
      call str_grad_out(phistart,'phistart #')
      call str_grad_out(phistop,'phistop #')
      call str_grad_out(delphi,'delphi #')
      call str_flush
      call str_real_len_push(4)
      call str_real_out(tol,'tol')
      call str_2real_out(weight1,weight2,'weight')
      call str_real_len_pop
      call str_flush
      if(tofile) then
        close(unit)
        call str_unit_pop
        call create_overwrite(old,old)
      else
        if(.not.menu) call str_wrap(0) ! reset
        message = ' '
        ier = 0
      endif
      return
      end                       ! savefile

      subroutine saveresult(nd,*)
      integer   nd
      include 'euhedral.inc'
c---- functions
      logical   str_equal
c---- local vars
      character prompt*80
      character d*80
      character message*256
      integer   ier
c
      if(bestr.lt.initr) then
        if(menu) then
          prompt = 'Keep result [yes]'
          call get_inp_default(d,nd,prompt,*900)
        else
          d = 'Y'
        endif
        if(str_equal(d,'Y')) then
          call WriteInsFile(compound,message,ier)
          call errorwarning(message,ier)
        endif
      endif
      call ReadIns(message,ier)
      call errorwarning(message,ier)
      return
  900 continue
      return 1
      end                       ! SaveResult

      subroutine ScaleVector(nmax,v1,v2)
      integer nmax
      real    v1(3)
      real    v2(3)
c---- local vars
      real    vmax
      integer i
      integer d
      real    res
      real    resmin
      integer dmin
      real    f
c
      call CopyVec(v1,v2)
      vmax = max(abs(v2(1)),abs(v2(2)),abs(v2(3)))
      if(vmax.lt.0.0001) return
      do i=1,3
        v2(i) = v2(i)/vmax
      enddo
      resmin = 1.0e10
      dmin = 1.0 ! just be sure. I think it is not necessary
      do d=1,nmax
        res = 0.0
        do i=1,3
          f = mod(d*abs(v2(i)),1.0)
          if(f.gt.0.5) f = 1.0 - f
          res = res + f
        enddo
        if(res.lt.resmin) then
          dmin = d
          resmin = res
        endif
      enddo
      call FacVec(float(dmin),v2,v2)
      return
      end                       ! ScaleVector


      subroutine SearchMother(h1,k1,l1,hb,im)
      integer h1,k1,l1
      integer hb
      integer im
      include 'euhedral.inc'
c---- functions
      integer   hklhash
c---- local vars
      integer i
c
      hb=hklhash(h1,k1,l1)
      do i=1,nmother(hb)
        if(mI(pMh,i,hb).eq.h1.and.mI(pMk,i,hb).eq.k1.and.
     +       mI(pMl,i,hb).eq.l1) then
          im = i
          return
        endif
      enddo
      im = 0
      return
      end                       ! SearchMother

      subroutine setupcell
      include 'euhedral.inc'
c
      call celldim(mydmat,ax,ang)
      call celldimstar(myrmat,axstar,angstar)
      call transmat(mydmat,mydmatTrans) ! this tmat() is needed in ProcessLineAr
      return
      end                       ! setupcell

      subroutine setuprefinement(ier)
      integer   ier
      include 'euhedral.inc'
c---- functions
      real      r2NoFilter
      logical   facesOK
c---- local vars
      real      f
      character message*256
c
      call readins(message,ier)
      call errorwarning(message,ier)
      if(ier.ne.0) goto 900
      if(.not.facesOK()) then
        ier = 1
        goto 900
      endif
      call readhkl(workfile,message,ier)
      call errorwarning(message,ier)
      if(ier.ne.0) goto 900
      useablehklfile = .true.
      f = r2NoFilter()
      call str_real_len_push(5)
      call str_real(f,'uncorrected R2=#',message)
      call str_real_len_pop
      initr = -1.0
      call logwrite(message)
      call plotfaces
  900 continue
      call str_flush
      return
      end                       ! SetupRefinement
      
      subroutine showfaces(sortkey)
      character sortkey*(*)
      include 'euhedral.inc'
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='ShowFaces')
c---- local vars
      integer   i
      integer   j
      integer   ihkl(3)
      integer   fp(maxfaces)
c
      call logwrite(ROUTINE)
      call str_flush
      write(out,600)
      write(log,600)
  600 format('Face   H   K   L distance  surface')
      if(sortkey.eq.'s') then
        call sort_real_index(nfaces,facesurface,fp)
      else
        do i=1,nfaces
          fp(i) = i
        enddo
      endif
      do i=1,nfaces
        j = fp(i)
        call getvecI(facehkl,j,ihkl)
        write(out,610) j,ihkl,max(0.001,facefactor(j)*facedist(j)),
     +       facesurface(j)
        write(log,610) j,ihkl,max(0.001,facefactor(j)*facedist(j)),
     +       facesurface(j)
  610   format(i4,3i4,f9.4,f9.4)
      enddo
      return
      end                       ! ShowFaces

      subroutine SimplexDistance(free,old,new,rmin)
      include 'euhedral.inc'
      logical   free(maxparam)
      real      old(maxparam)
      real      new(maxparam)
      real      rmin
c---- functions
      real      SimplexDistanceFunk
      external  SimplexDistanceFunk
      logical   interrupted
c---- local
      integer   np
      parameter (np=maxparam)
      integer   mp
      parameter (mp=np+1)
      real      y(1:mp)
      real      p(1:mp,1:np)
      integer   iterations
      integer   i,j,k
      real      par(maxparam)
      logical   facedone(MaxFaces)
      integer   ihkl(3)
      integer   jhkl(3)
      integer   jmin

      integer   ndim
      integer   pointer(2,maxparam)
      common /amoeba_com/ ndim,pointer

      call str_real_out(tol,'Simplex Tolerance=#')
      
      if(centricshape) then
        do i=1,nfaces
          facedone(i) = .not.free(i)
        enddo
        
        ndim = 0
        do i=1,nfaces
          if(.not.facedone(i)) then
            ndim = ndim + 1
            pointer(1,ndim) = i
            pointer(2,ndim) = 0
            par(ndim) = old(i)
            call getVecI(facehkl,i,ihkl)
            do j=i+1,nfaces
              call GetVecI(facehkl,j,jhkl)
              if(ihkl(1).eq.-jhkl(1).and.
     +             ihkl(2).eq.-jhkl(2).and.
     +             ihkl(3).eq.-jhkl(3)) then
                facedone(j) = .true.
                pointer(2,ndim) = j
              endif
            enddo
          endif
        enddo
      else
        ndim=0
        do i=1,nfaces
          new(i) = old(i)
          if(free(i)) then
            ndim = ndim + 1
            pointer(1,ndim) = i
            pointer(2,ndim) = 0
            par(ndim) = old(i)
          endif
        enddo
      endif
      if(ndim.eq.0) then
        call str_out('Nothing to refine')
        call str_flush
        return
      endif
      call str_num_out(ndim,'# parameter^')
      call str_flush
      do i=1,ndim
        call str_integer_out(i,'par#:')
        k = pointer(1,i)
        call str_integer_out(k,'face #')
        if(pointer(2,i).ne.0) then
          call str_integer_out(pointer(2,i),'and #')
        endif
        call str_real_out(par(i),'=')
        call str_flush
      enddo
      call SimplexDistanceInit(mp,np,ndim,par,p)
      call SimplexDistanceSetup(mp,np,ndim,p,y,extreme)
c      call writep(mp,np,ndim,p,y)
      if(interrupted()) then
        rmin = 999.9
        return
      endif
      call str_write(6,'Refining....')
      call str_real_len_push(5)
      call Amoeba(P,Y,mp,np,ndim,tol,SimplexDistanceFunk,iterations)
      call str_real_len_pop
      call str_integer_write(6,iterations,'# iterations')
      rmin = y(1)
      jmin = 1
      do j=2,ndim+1
        if(y(j).lt.rmin) then
          rmin = y(j)
          jmin = j
        endif
      enddo
      do i=1,ndim
        new(pointer(1,i)) = p(jmin,i)
        if(pointer(2,i).ne.0) then
          new(pointer(2,i)) = p(jmin,i)
        endif
      enddo
      return
      end                       ! SimplexDistance

      subroutine SimplexDistanceInit(mp,np,ndim,old,p)
      integer   np
      integer   mp
      integer   ndim
      real      old(np)
      real      p(mp,np)
c---- local
      integer i,j
c
      do j=1,ndim
        do i=1,ndim+1
          p(i,j) = old(j)
        enddo
      enddo
      return
      end                       ! SimplexDistanceInit

      subroutine SimplexDistanceSetup(mp,np,ndim,p,y,extreme)
      integer   np
      integer   mp
      integer   ndim
      real      p(mp,np)
      real      y(mp)
      real      extreme
c---- functions
      real      SimplexDistanceFunk
      real      rnd
      logical   interrupted
c---- constants
      integer   localmaxpar
      parameter (localmaxpar=12)
      integer   method
      parameter (method=1)
      character*(*) ROUTINE
      parameter (ROUTINE='SimplexDistanceSetup')
c---- local
      real    par(localmaxpar)
      integer i
      integer j
      integer istart
      integer noodrem
      integer k
      logical same
c
      if(ndim.gt.localmaxpar) then
        call str_string_out(ROUTINE,'Error in #.')
        call str_integer_out(ndim,'ndim=#')
        call str_flush
        call exitfatal
      endif
      call str_out(ROUTINE)
      call str_flush

      if(method.eq.1) then
        do j=1,ndim
          p(j+1,j) = p(j+1,j) + 0.2*extreme
        enddo
      else
        if(ndim.ge.3) then
          do j=1,ndim
            p(1,j) = p(1,j) + 0.2*extreme
            p(2,j) = p(2,j) - 0.2*extreme
          enddo
          istart = 3
        else
          istart = 1
        endif
        do i=istart,ndim+1
          noodrem = 0
   10     continue
          do j=1,ndim
            if(rnd().gt.0.5) then
              par(j) = p(i,j) + 0.2*extreme
            else
              par(j) = p(i,j) - 0.2*extreme
            endif
          enddo
          
          do k=1,i-1
            same = .true.
            do j=1,ndim
              if(par(j).ne.p(k,j)) same = .false.
            enddo
            if(same) then
              noodrem = noodrem + 1
              if(noodrem.gt.30) then
                call str_string_out(ROUTINE,'Noodrem in #')
              else
                goto 10
              endif
            endif
          enddo
          do j=1,ndim
            p(i,j) = par(j)
          enddo
        enddo
      endif ! method
      do i=1,ndim+1
        do j=1,ndim
          par(j) = p(i,j)
        enddo
        call str_integer_out(i,'Corner#')
        y(i) = SimplexDistanceFunk(par)
        if(interrupted()) return
      enddo
      return
      end                       ! SimplexDistanceSetup

      subroutine SimplexOrientation(old,new,rmin)
      include 'euhedral.inc'
      real      old(3)
      real      new(3)
      real      rmin
c---- functions
      real      SimplexOrientationFunk
      external  SimplexOrientationFunk
      logical   interrupted
c---- local
      integer   np
      parameter (np=3)
      integer   mp
      parameter (mp=np+1)
      real      y(1:mp)
      real      p(1:mp,1:np)
      integer   iterations
      integer   i,j
      real      par(np)
      integer   jmin
      integer   ndim

      call str_real_out(tol,'Simplex Orientation Tolerance=#')
      call str_grad_out(OrientSimplexAngle,'Initial Angle=#')
      ndim = 3
      do i=1,ndim
        par(i) = old(i)
      enddo
      call SimplexOrientationInit(mp,np,ndim,par,p)
      call SimplexOrientationSetup(mp,np,ndim,p,y,OrientSimplexAngle)
      if(interrupted()) then
        rmin = 999.9
        return
      endif
      call str_write(6,'Refining....')
      call str_real_len_push(5)
      call Amoeba(P,Y,mp,np,ndim,tol,SimplexOrientationFunk,iterations)
      call str_real_len_pop
      call str_integer_write(6,iterations,'# iterations')
      rmin = y(1)
      jmin = 1
      do j=2,ndim+1
        if(y(j).lt.rmin) then
          rmin = y(j)
          jmin = j
        endif
      enddo
      do i=1,ndim
        new(i) = p(jmin,i)
      enddo
      return
      end                       ! SimplexOrientation

      subroutine SimplexOrientationInit(mp,np,ndim,old,p)
      integer   np
      integer   mp
      integer   ndim
      real      old(np)
      real      p(mp,np)
c---- local
      integer i,j
c
      do j=1,ndim
        do i=1,ndim+1
          p(i,j) = old(j)
        enddo
      enddo
      return
      end                       ! SimplexOrientationInit

      subroutine SimplexOrientationSetup(mp,np,ndim,p,y,initang)
      integer   np
      integer   mp
      integer   ndim
      real      p(mp,np)
      real      y(mp)
      real      initang
      include 'const.inc'
c---- functions
      real      SimplexOrientationFunk
      logical   interrupted
c---- constants
      character*(*) ROUTINE
      parameter (ROUTINE='SimplexOrientationSetup')
c---- local
      real    par(3)
      integer i
      integer j
c
      call str_out(ROUTINE)
      call str_flush
      do j=1,ndim
        p(j+1,j) = p(j+1,j) + initang
      enddo
      do i=1,ndim+1
        do j=1,ndim
          par(j) = p(i,j)
        enddo
        call str_integer_out(i,'Corner#')
        y(i) = SimplexOrientationFunk(par)
        if(interrupted()) return
      enddo
      return
      end                       ! SimplexOrientationSetup

      subroutine SimplexRotmat(par,rotmat)
      real par(3)
      real rotmat(3,3)
      include 'axes.inc'
      call GeneralRotmat(x_axis,par(1),rotmat)
      call RotateMat(rotmat,y_axis,par(2),rotmat)
      call RotateMat(rotmat,z_axis,par(3),rotmat)
      return
      end                       ! SimplexRotmat

      subroutine SquishHkl(dcos)
      real    dcos
      include 'euhedral.inc'
      integer hb
      integer im
      integer p
      integer q
      integer i
      integer nsquish

      nsquish = 0
      do hb=1,hashbins
        do im = 1,nmother(hb)
          p = mI(pMfirst,im,hb)
          do while(p.ne.0)
            q = dI(pDsister,p)
            do while(q.ne.0)
              if(dI(pDactive,q).gt.0) then
                if(dI(pDh,p).ne.dI(pDh,q)) goto 100
                if(dI(pDk,p).ne.dI(pDk,q)) goto 100
                if(dI(pDl,p).ne.dI(pDl,q)) goto 100
                do i=pDcosinesF,pDcosinesL
                  if(abs(dr(i,p)-dr(i,q)).gt.dcos) goto 100
                enddo
                nsquish = nsquish + 1
                dI(pDactive,q) = 0
                call PrintDaughterHeader
                call PrintDaughter(p)
                call PrintDaughter(q)
              endif
  100         continue
              q = dI(pDsister,q)
            enddo
            p = dI(pDsister,p)
          enddo
        enddo
      enddo
      call str_num_write(out,nsquish,'# reflection^ squished')
      p = 1
      do while(p.lt.nref)
        if(dI(pDactive,p).eq.0) then
c---- move last reflection to here
          do i=1,pDimax
            dI(i,p) = dI(i,nref)
          enddo
          do i=1,pDrmax
            dR(i,p) = dR(i,nref)
          enddo
          nref = nref - 1
        endif
        p = p + 1
      enddo
      
      call BuildMother
      call str_num_out(nref,'# reflection^')
      call str_integer_out(Nunique,'(# unique)')

      call OutputR2
      return
      end                       ! SquishHKL

      subroutine StartGraphics(calledfrom)
      character calledfrom*(*)
      include 'euhedral.inc'
      integer   ier
      integer   ws
      integer   margin
      integer   subwindow
      integer   widthsmall
      integer   widthlarge
      integer   height
      real      f1
      real      f2
      integer   i
      if(newwindow) then
        call str_string_out(calledfrom,'call from #')
        do i=1,cMAX
          call getcolour(colours(i),col(i))
        enddo
c        call getcolour(colourfg,colfg)
c        call getcolour(colourcosi,colcosi)
c        call getcolour(colourcoso,colcoso)
c        call getcolour(colourcircle,colcircle)
c        call getcolour(colourfacesfg,colfacesfg)
c        call getcolour(colourfacesbg,colfacesbg)
c        call getcolour('yellow',colc)
c        call getcolour(colouraxesfg,colaxfg)
c        call getcolour(colouraxesbg,colaxbg)
c        if(windowexists) then
c          call WindowInfo(6)
c          call GetWindowGeometry(posx,posy,w,h,b)
c          call str_2integer_out(posx,posy,'posx=# posy=#')
c        endif
        margin = 2
        ws = size - 2*margin
        subwindow = 0
        widthsmall = ws+2*margin
        widthlarge = 3*ws+4*margin
        height = ws+2*margin
        if(WindowExists) then
          call str_out('resizing')
          if(windowShape.eq.ShapeMakeBig) then
            call activatewindow(1,ier)
            if(ier.ne.0) goto 900
            call ResizeWindow(widthlarge,height)
            windowShape = ShapeBig
            subwindow = 3
          endif
        else
          if(WindowShape.eq.ShapeSmall) then
            call CreateWindow(1,compound,posx,posy,widthsmall,height,
     +           'white',ier)
            if(ier.ne.0) goto 900
            subwindow = 1
          else
            call CreateWindow(1,compound,posx,posy,widthlarge,height,
     +           'white',ier)
            windowShape = ShapeBig
            subwindow = 3
            if(ier.ne.0) goto 900
          endif
        endif
        call str_flush

        if(subwindow.eq.3) then
          call CreateSubWindow(2,1,margin,margin,ws,ws,0,
     +         colours(cBG),ier)
          if(ier.ne.0) goto 900
          call CreateSubWindow(3,1,margin+ws+margin,margin,ws,ws,0,
     +         colours(cBG),ier)
          if(ier.ne.0) goto 900
          call CreateSubWindow(4,1,margin+2*(ws+margin),margin,ws,ws,0,
     +         colours(cBG),ier)
          if(ier.ne.0) goto 900
          newworld = .true.
        else if(subwindow.eq.1) then
          call CreateSubWindow(4,1,margin,margin,ws,ws,0,
     +         colours(cBG),ier)
          if(ier.ne.0) goto 900
          newworld = .true.
        endif
        call CreatePixmap(5,4,ier)
        if(WindowShape.eq.ShapeBig) then
          call CreateWorld(1,2,-1.1,-1.1,1.1,1.1,ier)
          if(ier.ne.0) goto 900
          call CreateWorld(2,3,-1.1,-1.1,1.1,1.1,ier)
          if(ier.ne.0) goto 900
        endif
        f1 = 2.0
        f2 = 12.0
        call CreateWorld(4,5,-f1,-f1,f2,f2,ier) ! LL
        if(ier.ne.0) goto 900
        call CreateWorld(5,5,-f2,-f1,f1,f2,ier) ! LR
        if(ier.ne.0) goto 900
        call CreateWorld(6,5,-f2,-f2,f1,f1,ier) ! UR
        if(ier.ne.0) goto 900
        call CreateWorld(7,5,0.0,0.0,1.0,1.0,ier)
      endif
  900 continue
      call str_flush
      newwindow = .false.
      graphics = (ier.eq.0)
      if(graphics) windowexists = .true.
      return
      end                       !StartGraphics

      subroutine surface(np,xp,yp,result)
      integer np
      real    xp(np)
      real    yp(np)
      real    result
      include 'const.inc'
      include 'euhedral.inc'
c---- local vars
      real    x0
      real    y0
      integer i
      integer j
      real    angles(maxcorners)
      integer pointer(maxcorners)
      real    s
      integer p1
      integer p2
      real    xv(maxcorners)
      real    yv(maxcorners)
c
      x0 = xp(1)
      y0 = yp(1)
      do i=2,np
        x0 = x0 + xp(i)
        y0 = y0 + yp(i)
      enddo
      x0 = x0/np
      y0 = y0/np

      do i=1,np
        xv(i) = xp(i) - x0
        yv(i) = yp(i) - y0
        angles(i) = atan2(yv(i),xv(i))
      enddo

      call sort_real_index(np,angles,pointer)

      result = 0.0
      do i=1,np
        j = mod(i,np)+1
        p1 = pointer(i)
        p2 = pointer(j)
        s = 0.5*abs(xv(p1)*yv(p2) - xv(p2)*yv(p1))
        result = result + s
      enddo
      return
      end                       ! Surface

      subroutine TestAndSave(ier)
      integer   ier
      include 'euhedral.inc'
c---- functions
      logical   FacesOK
c---- local vars      
      character message*256

      call FaceTest(ExtraOutput,DoDeactivate,message,ier)
      if(ier.ne.0) call str_out(message)
      facesavailable = (ier.eq.0)
      if(FacesOK()) call WriteAndRead(ier)
      return
      end                       ! TestAndSave

      subroutine TestShape(name,message,ier)
      character name*(*)
      character message*(*)
      integer   ier
c---- common
      logical abortrefinement
      common /common_refinement/ abortrefinement
c---- local vars
      character filename*80
      integer   lf
      integer   unit
      character line*80
      integer   nerr1
      integer   nerr2
      call build_file(name,' ','.lis',filename,lf)
      call open_file(filename,unit,message,ier)
      if(ier.ne.0) return
      read(unit,100,iostat=ier) line
  100 format(a)
      if(ier.ne.0) then
        call str_string(filename,'No data in #',message)
        return
      endif
      nerr1 = 0
      nerr2 = 0
      do while(ier.eq.0)
        if(line(1:2).eq.'::') then
          if(index(line,'NO Intersection').gt.0) then
c=== :: Crystal not Finite
            nerr1 = nerr1 + 1
          else if(index(line,'Crystal not Finite').gt.0) then
c=== :: Note >> FACE  1  has NO Intersection
            nerr2 = nerr2 + 1
          endif
c        else if(line(1:14).eq.'Crystal Volume') then
c          call str_write(6,line)
        endif
        read(unit,100,iostat=ier) line
      enddo
      close(unit)
      if(nerr1.gt.0) then
        if(nerr2.gt.0) then
          message = '# No Intersection. # Crystal not Finite.'
          call str_integer(nerr1,message,message)
          call str_integer(nerr2,message,message)
        else
          message = '# No Intersection'
          call str_integer(nerr1,message,message)
        endif
        ier = 1
        abortrefinement = .true.
      else if(nerr2.gt.0) then
        message = '# Crystal not Finite.'
        call str_integer(nerr2,message,message)
        ier = 1
        abortrefinement = .true.
      else
        message = 'crystal shape ok'
        ier = 0
      endif
      return
      end                       ! TestShape

      subroutine tilt(nd,dirty,*)
      integer nd
      logical dirty
      include 'euhedral.inc'
c---- functions
      logical Interrupted
c---- local vars
      integer ier
      integer i
c
      call setuprefinement(ier)
      if(ier.ne.0) return
      if(menu) then
        call InputInteger(Highindex,'highindex',dirty,nd,*900)
        call FaceHighAll
        call InputPosAngle(Angmax,'angmax',dirty,nd,*900)
      endif
      call GetFaceList(nd,*900)
      if(nlist.eq.0) return
      initr = -1.0
      do i=1,nlist
        call facetilt(facelist(i),ier) ! this can remove faces
        if(ier.ne.0) return
        if(interrupted()) goto 10
      enddo
   10 continue
      call saveresult(nd,*900)
      return
  900 continue
      return 1
      end                       ! tilt

      subroutine ShowView
      include 'axes.inc'
      include 'const.inc'
      include 'euhedral.inc'
      real      viewmat(3,3)
      real      v(3)
      real      w(3)
      integer   wi(3)
      integer   i
      integer   ier
      real      viewmatinv(3,3)
      character hve(3)*3
      character line*80
      data hve /'hor','ver','eye'/
      
c
      call str_vector_out(viewvec,'viewvec')
      call str_onoff_out(vertical,'vertical')
      call str_grad_out(viewrot,'viewrot')
      call MakeViewmat(viewvec,viewmat)
      call str_flush
      call writemat(6,viewmat,'viewmat')
      if(vertical) then
        call RotateMat(viewmat,x_axis,-piover2,viewmat)
        call writemat(6,viewmat,'viewmat after vertical')
      endif
      call RotateMat(viewmat,y_axis,viewrot,viewmat)
      call writemat(6,viewmat,'viewmat after viewrot')
      call invertmat(viewmat,viewmatinv,ier)

      call tablestart(6)
      call tableadd(line)
      write(line,100) hve
  100 format(3x,3a8)
      call tableadd(line)
      do i=1,3
        call col2vec(i,myrmat,v)
        call Matvec(viewmat,v,w)
        call normvec(w,w,ier)
        write(line,110) abc(i)//'*',w
  110   format(a3,3f8.4,3i8)
        call tableadd(line)
      enddo
      call tableadd(' ')
      write(line,100) 'h','k','l'
      call tableadd(line)
      do i=1,3
        call row2vec(i,viewmat,v)
        call Matvec(mydmat,v,v)
        call scalevector(25,v,w)
        call copyvecFI(w,wi)
        write(line,120) hve(i),wi
  120   format(a3,3i8)
        call tableadd(line)
      enddo
      call tablestop
      return
      end                       ! ShowView

      subroutine ViewOneRotation
      include 'const.inc'
      include 'euhedral.inc'
c---- functions
      logical FacesOK
      logical interrupted
c---- local vars
      real f
      if(interrupted()) then
        call plotfaces
        return
      endif
      if(graphics) then
        if(FacesOK()) then
          f = viewrot
          do viewrot=f,f+TwoPi,viewspeed
            call plotfaces
            if(interrupted()) then
              call str_out('Interrupted')
              call resetinterrupt
              goto 900
            endif
          enddo
  900     continue
          viewrot = f
          call plotfaces
        endif
      endif
      return
      end                       ! ViewOneRotation
 
      subroutine ViewPartialRotation
      include 'const.inc'
      include 'euhedral.inc'
c---- functions
      logical FacesOK
      logical interrupted
c---- local vars
      real f
      if(interrupted()) then
        call plotfaces
        return
      endif
      if(graphics) then
        if(FacesOK()) then
          f = viewrot
          do viewrot=f,f+viewpartial,viewspeed
            call plotfaces
            if(interrupted()) then
              call resetinterrupt
              goto 900
            endif
          enddo
          do viewrot=f+viewpartial,f-viewpartial,-viewspeed
            call plotfaces
            if(interrupted()) then
              call resetinterrupt
              goto 900
            endif
          enddo
          do viewrot=f-viewpartial,f,viewspeed
            call plotfaces
            if(interrupted()) then
              call resetinterrupt
              goto 900
            endif
          enddo
  900     continue
          viewrot = f
          call plotfaces
        endif
      endif
      return
      end                       ! ViewPartialRotation
 
      subroutine warning(s)
      character s*(*)
      call str_flush
      call str_string_write(6,s,'WARNING: #')
      return
      end                       ! warning

      subroutine WriteAndRead(ier)
      integer  ier
      include 'euhedral.inc'
c---- local vars
      character message*256
c     
      call WriteInsFile(compound,message,ier)
      call errorwarning(message,ier)
      if(ier.eq.0) then 
        call readins(message,ier)
        call errorwarning(message,ier)
      endif
      return
      end                       ! WriteAndRead

      subroutine WriteHklFile(filename,message,ier)
      character filename*(*)
      character message*(*)
      integer   ier
      include 'euhedral.inc'
c---- functions
      logical   allowed
c---- local vars
      integer   unit
      integer   hb
      integer   im
      integer   p
      integer   k
      integer   n
      character hklfilename*80
      integer   lh
      real      intensity
      real      sigma
      logical   normalformat

      if(nref.eq.0) then
        message = 'No reflections available'
        ier = 1
        return
      endif
      call build_file(filename,' ','.hkl',hklfilename,lh)
      call create_file(hklfilename,unit,message,ier)
      if(ier.ne.0) return
      call str_out(message)
      n = 0
      do hb=1,hashbins
        do im = 1,nmother(hb)
          if(allowed(im,hb)) then
            p = mI(pMfirst,im,hb)
            do while(p.gt.0)
              n = n + 1
              intensity = dR(pDintensity,p)
              sigma = dR(pDsigma,p)
              normalformat = (intensity.lt.100000.0 .and.
     +             intensity.gt.-10000.0 .and. sigma.lt.100000.0)
              if(normalformat) then
                write(unit,100) (dI(k,p),k=pDh,pDl),intensity,sigma,1,
     +               (dR(k,p),k=pDcosinesF,pDcosinesL)
  100           format(3i4,2f8.2,i4,6f8.5)
              else
                write(unit,110) (dI(k,p),k=pDh,pDl),intensity,sigma,1,
     +               (dR(k,p),k=pDcosinesF,pDcosinesL)
  110           format(3i4,2f8.0,i4,6f8.5)
              endif
              p = DI(pDsister,p)
            enddo
          endif
        enddo
      enddo
      close(unit)
      message = 'writehkl. nhklmin=# thetamin=# intmin=# angdistmax=#'
      call str_integer(nhklmin,message,message)
      call str_grad(thetamin,message,message)
      call str_real(intmin,message,message)
      call str_real(angdistmax,message,message)
      call logwrite(message)
      message = '# reflection^ written to #'
      call str_num(n,message,message)
      call str_string(hklfilename,message,message)
      call logwrite(message)
      ier = 0
      return
      end                       ! WriteHklFile

      subroutine WriteInsFile(name,message,ier)
      character name*(*)
      character message*(*)
      integer   ier
      include 'const.inc'
      include 'euhedral.inc'
      real      v(3)
      integer   unit
      integer   i
      logical   old
      character filename*80
      integer   ihkl(3)

      call build_file(name,' ','.ins',filename,i)
      if(name.eq.workfile) then
        call create_overwrite(.true.,old)
        call create_file(filename,unit,message,ier)
        call create_overwrite(old,old)
      else
        call create_file(filename,unit,message,ier)
        call logwrite(message)
      endif
      if(ier.ne.0) return

      call str_write(unit,'TITL EUHEDRAL')
      call facvec(grad,cellangles,v)
      write(unit,110) lambda,cellaxes,v
  110 format('CELL',f8.5,3f10.4,3f10.3)
      do i=1,nfaces
        if(faceactive(i)) then
          call getVecI(facehkl,i,ihkl)
          write(unit,120) ihkl,max(0.001,facefactor(i)*facedist(i))
  120     format('FACE ',3i4,f7.4)
        endif
      enddo
      write(unit,130) correctionmethod,mu
  130 format(a,1x,f8.3)
      close(unit)
      ier = 0
      return
      end                       ! WriteInsFile

      subroutine MyLaue(NPOINT,IH,IK,IL,IHNEW,IKNEW,ILNEW)
      integer npoint
      integer ih,ik,il
      integer ihnew,iknew,ilnew
c---- local vars
      INTEGER X,Y,Z
      integer i
c
      IHNEW=IH
      IKNEW=IK
      ILNEW=IL
      if(npoint.eq.0) then
        call str_write(6,'npoint not defined')
        call exitfatal
      endif

      goto (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,
     +     23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,
     +     40,41,42) npoint
C
C     1
C
    1 RETURN
C
C     -1
C
    2 IF (ILNEW.GT.0) RETURN
      IHNEW=-IHNEW
      IKNEW=-IKNEW
      ILNEW=-ILNEW
      IF (ILNEW.GT.0) RETURN
      IF (IKNEW.GT.0) RETURN
      IHNEW=-IHNEW
      IKNEW=-IKNEW
      IF (IKNEW.GT.0) RETURN
      IF (IHNEW.GE.0) RETURN
      IHNEW=-IHNEW
      RETURN
C
C     2
C
    3 IF (ILNEW.GT.0) RETURN
      IHNEW=-IHNEW
      ILNEW=-ILNEW
      IF (ILNEW.GT.0) RETURN
      IF (IHNEW.GE.0) RETURN
      IHNEW=-IHNEW
      RETURN
C
C     M
C
    4 IKNEW=ABS(IKNEW)
      RETURN
C
C     2/M
C
    5 IKNEW=ABS(IKNEW)
      GO TO 3
C
C     222
C
    6 IF (IHNEW.EQ.0.OR.IKNEW.EQ.0.OR.ILNEW.EQ.0) THEN
        IHNEW=ABS(IHNEW)
        IKNEW=ABS(IKNEW)
        ILNEW=ABS(ILNEW)
        RETURN
      END IF
      IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      Z=ILNEW
      IHNEW=-X
      IKNEW=-Y
      ILNEW=Z
      IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=Y
      ILNEW=-Z
      IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      IHNEW=X
      IKNEW=-Y
      ILNEW=-Z
      RETURN
C
C     MM2
C
    7 IHNEW=ABS(IHNEW)
      IKNEW=ABS(IKNEW)
      RETURN
C
C     MMM
C
    8 ILNEW=ABS(ILNEW)
      GO TO 7
C
C     4
C
    9 IF (IHNEW.EQ.0.AND.IKNEW.EQ.0) RETURN
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      IHNEW=-Y
      IKNEW=X
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=-Y
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=-X
      RETURN
C
C     -4
C
   10 IF (IHNEW.EQ.0.AND.IKNEW.EQ.0) THEN
        ILNEW=ABS(ILNEW)
        RETURN
      END IF
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      IHNEW=-X
      IKNEW=-Y
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=-X
      ILNEW=-ILNEW
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=X
      RETURN
C
C     4/M
C
   11 ILNEW=ABS(ILNEW)
      GO TO 9
C
C     422
C
   12 IF (IHNEW.EQ.0.OR.IKNEW.EQ.0.OR.ABS(IHNEW).EQ.ABS(IKNEW))
     & ILNEW=ABS(ILNEW)
  512 IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      IHNEW=-X
      IKNEW=-Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=-X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=Y
      ILNEW=-ILNEW
      GO TO 512
C
C     4MM
C
   13 IHNEW=ABS(IHNEW)
      IKNEW=ABS(IKNEW)
      IF (IHNEW.GE.IKNEW) RETURN
      X=IHNEW
      IHNEW=IKNEW
      IKNEW=X
      RETURN
C
C     -42M
C
   14 IF (IHNEW.EQ.0.OR.IKNEW.EQ.0) ILNEW=ABS(ILNEW)
  514 IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      IHNEW=-X
      IKNEW=-Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=-X
      ILNEW=-ILNEW
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=Y
      GO TO 514
C
C     -4M2
C
   15 IHNEW=ABS(IHNEW)
      IKNEW=ABS(IKNEW)
      IF (IHNEW.EQ.IKNEW) ILNEW=ABS(ILNEW)
      IF (IHNEW.GE.IKNEW) RETURN
      X=IHNEW
      IHNEW=IKNEW
      IKNEW=X
      ILNEW=-ILNEW
      RETURN
C
C     4/MMM
C
   16 ILNEW=ABS(ILNEW)
      GO TO 13
C
C     3
C
   17 IF (IHNEW.EQ.0.AND.IKNEW.EQ.0) RETURN
      IF (IHNEW.LT.0.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.LT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=I
      RETURN
C
C     -3
C
   18 IF (IHNEW.EQ.0.AND.IKNEW.EQ.0) THEN
        ILNEW=ABS(ILNEW)
        RETURN
      END IF
  518 IF (IHNEW.LT.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     &  IHNEW.LT.-IKNEW))) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.LT.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     &  IHNEW.LT.-IKNEW))) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.LT.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     &  IHNEW.LT.-IKNEW))) RETURN
      IHNEW=-X
      IKNEW=-Y
      ILNEW=-ILNEW
      GO TO 518
C
C     312
C
   19 IF (IHNEW.EQ.0.OR.IKNEW.EQ.0.OR.-IHNEW.EQ.IKNEW) ILNEW=ABS(ILNEW)
  519 IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=-X
      ILNEW=-ILNEW
      GO TO 519
C
C     321
C
   20 IF (IHNEW.EQ.IKNEW.OR.-2*IHNEW.EQ.IKNEW.OR.-IHNEW.EQ.2*IKNEW)
     &  ILNEW=ABS(ILNEW)
  520 IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      IHNEW=Y
      IKNEW=X
      ILNEW=-ILNEW
      GO TO 520
C
C     31M
C
   21 IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      IHNEW=Y
      IKNEW=X
      GO TO 21
C
C     3M1
C
   22 IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GE.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=-X
      GO TO 22
C
C     -31M
C
   23 IF (IHNEW.EQ.0.OR.IKNEW.EQ.0.OR.-IHNEW.EQ.IKNEW) ILNEW=ABS(ILNEW)
  523 IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW.AND.
     & (ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IKNEW.GE.0))) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW.AND.
     & (ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IKNEW.GE.0))) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW.AND.
     & (ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IKNEW.GE.0))) RETURN
      IHNEW=Y
      IKNEW=X
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW.AND.
     & (ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IKNEW.GE.0))) RETURN
      IHNEW=X
      IKNEW=I
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW.AND.
     & (ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IKNEW.GE.0))) RETURN
      IHNEW=I
      IKNEW=Y
      IF (IHNEW.GE.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW.AND.
     & (ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IKNEW.GE.0))) RETURN
      IHNEW=-X
      IKNEW=-Y
      ILNEW=-ILNEW
      GO TO 523
C
C     -3M1
C
   24 IF (IHNEW.EQ.IKNEW.OR.-2*IHNEW.EQ.IKNEW.OR.-IHNEW.EQ.2*IKNEW)
     & ILNEW=ABS(ILNEW)
  524 IF (IHNEW.GE.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IHNEW.GE.IKNEW))) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GE.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IHNEW.GE.IKNEW))) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GE.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IHNEW.GE.IKNEW))) RETURN
      IHNEW=Y
      IKNEW=X
      ILNEW=-ILNEW
      IF (IHNEW.GE.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IHNEW.GE.IKNEW))) RETURN
      IHNEW=X
      IKNEW=I
      IF (IHNEW.GE.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IHNEW.GE.IKNEW))) RETURN
      IHNEW=I
      IKNEW=Y
      IF (IHNEW.GE.0.AND.IKNEW.GE.0.AND.(ILNEW.GT.0.OR.(ILNEW.EQ.0.AND.
     & IHNEW.GE.IKNEW))) RETURN
      IHNEW=-X
      IKNEW=-Y
      GO TO 524
C
C     6
C
   25 IF (IHNEW.EQ.0.AND.IKNEW.EQ.0) RETURN
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=-Y
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-I
      IKNEW=-X
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=-I
      RETURN
C
C     -6
C
   26 ILNEW=ABS(ILNEW)
      GO TO 17
C
C     6/M
C
   27 ILNEW=ABS(ILNEW)
      GO TO 25
C
C     622
C
   28 IF (IHNEW.EQ.0.OR.IKNEW.EQ.0.OR.ABS(IHNEW).EQ.ABS(IKNEW).OR.
     & ABS(2*IHNEW).EQ.ABS(IKNEW).OR.
     & ABS(IHNEW).EQ.ABS(2*IKNEW)) ILNEW=ABS(ILNEW)
  528 IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=-Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-I
      IKNEW=-X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=-I
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=X
      ILNEW=-ILNEW
      GO TO 528
C
C     6MM
C
   29 IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=-Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-I
      IKNEW=-X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=-I
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=X
      GO TO 29
C
C     -6M2
C
   30 ILNEW=ABS(ILNEW)
      IF (IHNEW.EQ.0.AND.IKNEW.EQ.0) RETURN
  530 IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GT.0.AND.IKNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=-X
      GO TO 530
C
C     -62M
C
   31 ILNEW=ABS(ILNEW)
      IF (IHNEW.EQ.0.AND.IKNEW.EQ.0) RETURN
  531 IF (IHNEW.GT.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      X=IHNEW
      Y=IKNEW
      I=-IHNEW-IKNEW
      IHNEW=I
      IKNEW=X
      IF (IHNEW.GT.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      IHNEW=Y
      IKNEW=I
      IF (IHNEW.GT.0.AND.-IHNEW/2.LE.IKNEW.AND.IKNEW.LE.IHNEW) RETURN
      IHNEW=Y
      IKNEW=X
      GO TO 531
C
C     6/MMM
C
   32 ILNEW=ABS(ILNEW)
      GO TO 29
C
C     R3
C
   33 STOP 'RE-INDEX ON HEXAGONAL AXES'
C
C     R-3
C
   34 GO TO 33
C
C     R32
C
   35 GO TO 33
C
C     R3M
C
   36 GO TO 33
C
C     R-3M
C
   37 GO TO 33
C
C     23
C
   38 IF (ABS(IHNEW).EQ.ABS(IKNEW).AND.ABS(IKNEW).EQ.ABS(ILNEW)) THEN
        IHNEW=ABS(IHNEW)
        IKNEW=ABS(IKNEW)
        ILNEW=ABS(ILNEW)*(IHNEW*IKNEW*ILNEW)/ABS(IHNEW*IKNEW*ILNEW)
        RETURN
      END IF
  538 IF (IHNEW.GT.ABS(ILNEW).AND.IKNEW.GE.ABS(ILNEW)) RETURN
      X=IHNEW
      Y=IKNEW
      Z=ILNEW
      IHNEW=-X
      IKNEW=-Y
      ILNEW=Z
      IF (IHNEW.GT.ABS(ILNEW).AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=-X
      IKNEW=Y
      ILNEW=-Z
      IF (IHNEW.GT.ABS(ILNEW).AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=X
      IKNEW=-Y
      ILNEW=-Z
      IF (IHNEW.GT.ABS(ILNEW).AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=Z
      IKNEW=X
      ILNEW=Y
      GO TO 538
C
C     M3
C
   39 IHNEW=ABS(IHNEW)
      IKNEW=ABS(IKNEW)
      ILNEW=ABS(ILNEW)
      IF (IHNEW.EQ.IKNEW.AND.IKNEW.EQ.ILNEW) RETURN
  539 IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW) RETURN
      X=IHNEW
      Y=IKNEW
      Z=ILNEW
      IHNEW=Z
      IKNEW=X
      ILNEW=Y
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW) RETURN
      IHNEW=Y
      IKNEW=Z
      ILNEW=X
      GO TO 539
C
C     432
C
   40 IF (ABS(IHNEW).EQ.ABS(IKNEW).AND.ABS(IKNEW).EQ.ABS(ILNEW)) THEN
        IHNEW=ABS(IHNEW)
        IKNEW=ABS(IKNEW)
        ILNEW=ABS(ILNEW)
        RETURN
      END IF
  540 IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      X=IHNEW
      Y=IKNEW
      Z=ILNEW
      IHNEW=Z
      IKNEW=X
      ILNEW=Y
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=Z
      ILNEW=X
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=-Y
      ILNEW=Z
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=Z
      IKNEW=-X
      ILNEW=-Y
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=Z
      ILNEW=-X
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=-X
      IKNEW=Y
      ILNEW=-Z
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=-Z
      IKNEW=-X
      ILNEW=Y
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=-Z
      ILNEW=-X
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=X
      IKNEW=-Y
      ILNEW=-Z
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=-Z
      IKNEW=X
      ILNEW=-Y
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=-Y
      IKNEW=-Z
      ILNEW=X
      IF (IHNEW.GT.ILNEW.AND.IKNEW.GE.ILNEW.AND.ILNEW.GE.0) RETURN
      IHNEW=Y
      IKNEW=X
      ILNEW=-Z
      GO TO 540
C
C     -43M
C
   41 IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      X=IHNEW
      Y=IKNEW
      Z=ILNEW
      IHNEW=Z
      IKNEW=X
      ILNEW=Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=Y
      IKNEW=Z
      ILNEW=X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=-X
      IKNEW=-Y
      ILNEW=Z
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=Z
      IKNEW=-X
      ILNEW=-Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=-Y
      IKNEW=Z
      ILNEW=-X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=-X
      IKNEW=Y
      ILNEW=-Z
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=-Z
      IKNEW=-X
      ILNEW=Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=Y
      IKNEW=-Z
      ILNEW=-X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=X
      IKNEW=-Y
      ILNEW=-Z
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=-Z
      IKNEW=X
      ILNEW=-Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=-Y
      IKNEW=-Z
      ILNEW=X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ABS(ILNEW)) RETURN
      IHNEW=Y
      IKNEW=X
      ILNEW=Z
      GO TO 41
C
C     M3M
C
   42 IHNEW=ABS(IHNEW)
      IKNEW=ABS(IKNEW)
      ILNEW=ABS(ILNEW)
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ILNEW) RETURN
      X=IHNEW
      Y=IKNEW
      Z=ILNEW
      IHNEW=Z
      IKNEW=X
      ILNEW=Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ILNEW) RETURN
      IHNEW=Y
      IKNEW=Z
      ILNEW=X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ILNEW) RETURN
      IHNEW=Z
      IKNEW=Y
      ILNEW=X
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ILNEW) RETURN
      IHNEW=X
      IKNEW=Z
      ILNEW=Y
      IF (IHNEW.GE.IKNEW.AND.IKNEW.GE.ILNEW) RETURN
      IHNEW=Y
      IKNEW=X
      ILNEW=Z
      RETURN
      END                       ! MyLaue

